Package: ggguides
Title: Simplified Legend and Guide Alignment for 'ggplot2'
Version: 1.1.10
Authors@R: 
    person("Gilles", "Colling", email = "gilles.colling051@gmail.com", role = c("aut", "cre"))
Description: Provides one-liner functions for common legend and guide operations
    in 'ggplot2'. Simplifies legend positioning, styling, wrapping, and
    collection across multi-panel plots created with 'patchwork' or 'cowplot'.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: ggplot2 (>= 3.5.0), grid, gridExtra, gtable, rlang
Suggests: patchwork, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://gillescolling.com/ggguides/,
        https://github.com/gcol33/ggguides
BugReports: https://github.com/gcol33/ggguides/issues
NeedsCompilation: no
Packaged: 2026-05-07 13:13:15 UTC; Gilles Colling
Author: Gilles Colling [aut, cre]
Maintainer: Gilles Colling <gilles.colling051@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-07 14:23:31 UTC
Built: R 4.6.0; ; 2026-05-07 16:33:51 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(ggplotGrob,gg_autofit_legend)
S3method(ggplotGrob,gg_centered_title)
S3method(ggplotGrob,gg_per_legend_just)
S3method(ggplot_add,ggguides_guide_update)
S3method(ggplot_add,ggguides_legend_order)
S3method(ggplot_add,legend_style_centered)
S3method(plot,gg_autofit_legend)
S3method(plot,gg_centered_title)
S3method(plot,gg_per_legend_just)
S3method(print,gg_autofit_legend)
S3method(print,gg_centered_title)
S3method(print,gg_per_legend_just)
export(align_guides_h)
export(center_legend_title)
export(collect_axes)
export(collect_legends)
export(colorbar_style)
export(colourbar_style)
export(get_legend)
export(legend_auto_fit)
export(legend_bottom)
export(legend_hide)
export(legend_horizontal)
export(legend_inside)
export(legend_keys)
export(legend_left)
export(legend_merge)
export(legend_none)
export(legend_order)
export(legend_order_guides)
export(legend_reverse)
export(legend_right)
export(legend_select)
export(legend_split)
export(legend_style)
export(legend_top)
export(legend_vertical)
export(legend_wrap)
export(shared_legend)
import(ggplot2)
importFrom(ggplot2,ggplot_add)
importFrom(grid,unit)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_filter)
importFrom(rlang,"%||%")
importFrom(rlang,check_installed)
importFrom(rlang,eval_tidy)
importFrom(rlang,is_installed)

# ggguides

[![CRAN status](https://www.r-pkg.org/badges/version/ggguides)](https://CRAN.R-project.org/package=ggguides)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/grand-total/ggguides)](https://cran.r-project.org/package=ggguides)
[![Monthly downloads](https://cranlogs.r-pkg.org/badges/ggguides)](https://cran.r-project.org/package=ggguides)
[![R-CMD-check](https://github.com/gcol33/ggguides/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/gcol33/ggguides/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/gcol33/ggguides/graph/badge.svg)](https://app.codecov.io/gh/gcol33/ggguides)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)

**Simple, Intuitive Legend Control for ggplot2**

The `ggguides` package provides one-liner functions for common legend operations in ggplot2. Instead of memorizing `theme()` arguments and guide specifications, use readable functions like `legend_left()`, `legend_style()`, and `legend_inside()` to position, style, and customize legends with minimal code.

## Quick Start

```r
library(ggplot2)
library(ggguides)

p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3)

# Position legends
p + legend_left()
p + legend_inside("topright")

# Style legends
p + legend_style(size = 14, title_face = "bold")

# Combine freely
p + legend_bottom() + legend_style(background = "grey95")
```

## Example

Six legends on four sides, each one positioned and styled independently. ggplot2 lets several legends share a side; ggguides lets you pick which legend goes where, slide it along the rail, and style it without touching the others.

<img src="man/figures/six_legends.svg" width="100%">

```r
p6 +
  # Side placement. Two legends share the top rail, two share the right.
  legend_top(by = "colour")    + legend_top(by = "fill")     +
  legend_right(by = "size")    + legend_right(by = "alpha")  +
  legend_bottom(by = "shape")  + legend_left(by = "linetype") +

  # Slide each one along its rail.
  legend_style(by = "colour",   justification = "left")   +
  legend_style(by = "fill",     justification = "right")  +
  legend_style(by = "size",     justification = "top")    +
  legend_style(by = "alpha",    justification = "bottom") +

  # Appearance (bold titles, smaller keys, smaller text, forced direction).
  legend_style(by = "colour", title_face = "bold",
               key_width = 0.4, key_height = 0.4) +
  legend_style(by = "size",   title_size = 9, size = 8)  +
  legend_style(by = "shape",  direction = "horizontal")
```

`legend_style(by = ...)` calls are additive, so one line per parameter is the intended shape. Full walkthrough in the [Multiple Legends vignette](https://gillescolling.com/ggguides/articles/multiple-legends.html#six-legends-stacked-per-side).

## Statement of Need

Legend customization in ggplot2 often requires verbose `theme()` calls with non-obvious argument names (`legend.position`, `legend.justification`, `legend.box.just`), and guide specifications scattered across `guides()` and `scale_*()` functions. Common tasks like positioning a legend inside the plot, styling the legend box, or managing multiple legends require looking up documentation repeatedly.

`ggguides` addresses this by providing:

- **Readable function names** that describe what they do (`legend_left()`, `legend_inside()`, `legend_reverse()`)
- **Sensible defaults** that handle related settings together (e.g., `legend_left()` sets position, justification, and box alignment)
- **Consistent API** across positioning, styling, and multi-legend operations
- **Patchwork integration** for multi-panel figures with shared legends

## Installation

```r
install.packages("ggguides")
```

Or install the development version from GitHub:

```r
# install.packages("pak")
pak::pak("gcol33/ggguides")
```

## Features

### Position Functions

- **`legend_left()` / `legend_right()`**: Side positioning with proper alignment
- **`legend_top()` / `legend_bottom()`**: Horizontal layout with optional plot alignment
- **`legend_inside()`**: Position inside plot using coordinates or shortcuts (`"topright"`, `"bottomleft"`, etc.)
- **`legend_none()`**: Remove legend entirely

### Style Functions

- **`legend_style()`**: Comprehensive styling (size, font, background, borders, margins)
- **`legend_wrap()`**: Wrap entries into columns or rows
- **`legend_reverse()`**: Reverse entry order
- **`legend_order()`**: Reorder legend entries
- **`legend_keys()`**: Customize key appearance
- **`colorbar_style()`**: Style continuous color legends

### Multiple Legend Control

- **`legend_hide()` / `legend_select()`**: Show/hide specific legends by aesthetic
- **`legend_order_guides()`**: Control display order of multiple legends
- **`legend_merge()` / `legend_split()`**: Combine or separate legend entries
- **`by` parameter**: Apply any function to specific aesthetics only

### Multi-Panel Support

- **`collect_legends()`**: Collect legends from patchwork compositions
- **`collect_axes()`**: Collect axes from patchwork compositions
- **`shared_legend()`**: Combine plots with shared legend (no patchwork required)
- **`get_legend()`**: Extract legend as standalone grob

## Usage Examples

### Position Helpers

```r
library(ggplot2)
library(ggguides)

p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")
```

#### `legend_left()` / `legend_right()`

Position with proper alignment (sets justification and box.just together):

```r
p + legend_left()
p + legend_right()
```

<img src="man/figures/legend_left.svg" width="45%"> <img src="man/figures/legend_right.svg" width="45%">

#### `legend_top()` / `legend_bottom()`

Horizontal layout with optional plot alignment:

```r
p + legend_top()
p + legend_bottom()

# Align to full plot (useful with titles)
p + labs(title = "My Title") + legend_top(align_to = "plot")
```

<img src="man/figures/legend_top.svg" width="45%"> <img src="man/figures/legend_bottom.svg" width="45%">

#### `legend_inside()`

Position inside the plot using coordinates or shortcuts:

```r
# Using shortcuts
p + legend_inside(position = "topright")
p + legend_inside(position = "bottomleft")

# Using coordinates
p + legend_inside(x = 0.95, y = 0.95, justification = c("right", "top"))

# With custom styling
p + legend_inside(position = "center", background = "grey95", border = "grey50")
```

<img src="man/figures/legend_inside_topright.svg" width="45%"> <img src="man/figures/legend_inside_bottomleft.svg" width="45%">

#### `legend_none()`

Remove the legend entirely:

```r
p + legend_none()
```

<img src="man/figures/legend_none.svg" width="45%">

---

### Style Helpers

#### `legend_style()`

Comprehensive styling in one call:

```r
# Change font size - affects both title and labels
p + legend_style(size = 14)

# Change font family
p + legend_style(family = "serif")
p + legend_style(family = "mono")

# Combine size and family
p + legend_style(size = 14, family = "serif")
```

<img src="man/figures/legend_style_size.svg" width="45%"> <img src="man/figures/legend_style_font.svg" width="45%">

```r
# Full styling with title emphasis
p + legend_style(
  size = 12,
  title_size = 14,
  title_face = "bold",
  key_width = 1.5,
  background = "grey95",
  background_color = "grey70",
  margin = 0.3
)
```

<img src="man/figures/legend_style_full.svg" width="60%">

#### `legend_wrap()`

Wrap legend entries into columns or rows:

```r
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2)

# Or by rows
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(nrow = 2)
```

<img src="man/figures/legend_wrap_ncol2.svg" width="45%"> <img src="man/figures/legend_wrap_nrow2.svg" width="45%">

#### `legend_reverse()`

Reverse legend entry order:

```r
p + legend_reverse()
```

<img src="man/figures/legend_reverse.svg" width="45%">

---

### Multiple Legends

When a plot has multiple aesthetics, control each legend separately:

#### `legend_hide()` / `legend_select()`

Hide specific legends or keep only certain ones:

```r
# Plot with multiple aesthetics
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point() +
  labs(color = "Cylinders", size = "Horsepower")

# Hide the size legend
p + legend_hide(size)

# Keep only the colour legend
p + legend_select(colour)
```

<img src="man/figures/legend_hide_size.svg" width="45%"> <img src="man/figures/legend_select_colour.svg" width="45%">

#### Position legends separately

Use the `by` parameter to position legends independently:

```r
# Colour legend on left, size legend at bottom
p +
  legend_left(by = "colour") +
  legend_bottom(by = "size")
```

<img src="man/figures/legend_position_separate.svg" width="60%">

#### Style legends separately

Apply different styles to different legends:

```r
p +
  legend_style(title_face = "bold", by = "colour") +
  legend_style(size = 10, by = "size")
```

<img src="man/figures/legend_style_separate.svg" width="60%">

#### `legend_order_guides()`

Control the display order of multiple legends:

```r
# Size legend first, then colour
p + legend_order_guides(size = 1, colour = 2)
```

<img src="man/figures/legend_order_guides.svg" width="60%">

---

### Patchwork Integration

#### `collect_legends()`

Collect legends from patchwork compositions:

```r
library(patchwork)

p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1")
p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2")

# Without collection (duplicate legends)
p1 | p2

# With collection
collect_legends(p1 | p2)

# Position at bottom
collect_legends(p1 | p2, position = "bottom")
```

<img src="man/figures/patchwork_no_collect.svg" width="80%">

<img src="man/figures/patchwork_collect.svg" width="80%">

#### Height Spanning

For stacked plots, use `span = TRUE` to make the legend span the full height.
Using different plot heights makes the spanning behavior more visible:

```r
library(patchwork)

p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3")

# Stack with different heights: 4, 2, 1
stacked <- (p1 / p2 / p3) + plot_layout(heights = c(4, 2, 1))

# Default: legend centered
collect_legends(stacked, position = "right")

# With spanning: legend fills full height
gt <- collect_legends(stacked, position = "right", span = TRUE)
grid::grid.draw(gt)
```

<img src="man/figures/patchwork_stacked_default.svg" width="45%"> <img src="man/figures/patchwork_stacked_span.svg" width="45%">

#### Row-Specific Attachment

Attach the legend to specific rows instead of spanning all:

```r
# Attach legend to row 1 only (the tallest plot)
gt <- collect_legends(stacked, position = "right", span = 1)
grid::grid.draw(gt)

# Attach legend to rows 1 and 2
gt <- collect_legends(stacked, position = "right", span = 1:2)
grid::grid.draw(gt)
```

<img src="man/figures/patchwork_span_row1.svg" width="45%"> <img src="man/figures/patchwork_span_row12.svg" width="45%">

---

### Combining Functions

Functions compose naturally:

```r
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_left() +
  legend_style(size = 12, title_face = "bold", background = "grey95")
```

<img src="man/figures/combined_left_styled.svg" width="60%">

```r
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2) +
  legend_bottom()
```

<img src="man/figures/combined_wrap_bottom.svg" width="60%">

---

### cowplot / Base Grid Support

ggguides also works without patchwork for cowplot users or anyone using base grid:

#### `get_legend()`

Extract a legend as a standalone grob:

```r
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(color = "Cylinders")

# Extract the legend
leg <- get_legend(p)

# Use with cowplot::plot_grid() or grid::grid.draw()
grid::grid.draw(leg)
```

<img src="man/figures/get_legend.svg" width="20%">

#### `shared_legend()`

Combine plots with a shared legend (no patchwork required):

```r
p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1", color = "Cylinders")
p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2", color = "Cylinders")
p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3", color = "Cylinders")

# Side-by-side with shared legend
gt <- shared_legend(p1, p2, ncol = 2, position = "right")
grid::grid.draw(gt)

# Stacked with legend at bottom
gt <- shared_legend(p1, p2, p3, ncol = 1, position = "bottom")
grid::grid.draw(gt)

# 2x2 grid
gt <- shared_legend(p1, p2, p3, p1, ncol = 2, nrow = 2, position = "right")
grid::grid.draw(gt)
```

<img src="man/figures/shared_legend_side.svg" width="80%">

<img src="man/figures/shared_legend_stacked.svg" width="50%">

<img src="man/figures/shared_legend_grid.svg" width="80%">

All ggguides styling functions (`legend_style()`, `legend_wrap()`, etc.) work on individual plots regardless of layout package.

## Documentation

- [Getting Started](https://gillescolling.com/ggguides/articles/getting-started.html)
- [Positioning](https://gillescolling.com/ggguides/articles/positioning.html)
- [Styling](https://gillescolling.com/ggguides/articles/styling.html)
- [Multiple Legends](https://gillescolling.com/ggguides/articles/multiple-legends.html)
- [Patchwork Integration](https://gillescolling.com/ggguides/articles/patchwork.html)

## Support

> "Software is like sex: it's better when it's free." — Linus Torvalds

I'm a PhD student who builds R packages in my free time because I believe good tools should be free and open. I started these projects for my own work and figured others might find them useful too.

If this package saved you some time, buying me a coffee is a nice way to say thanks. It helps with my coffee addiction.

[![Buy Me A Coffee](https://img.shields.io/badge/-Buy%20me%20a%20coffee-FFDD00?logo=buymeacoffee&logoColor=black)](https://buymeacoffee.com/gcol33)

## License

MIT (see the LICENSE.md file)

## Citation

```bibtex
@software{ggguides,
  author = {Colling, Gilles},
  title = {ggguides: Simplified Legend and Guide Alignment for ggplot2},
  year = {2025},
  url = {https://CRAN.R-project.org/package=ggguides},
  doi = {10.32614/CRAN.package.ggguides}
}
```

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)

## ----example-plot-------------------------------------------------------------
# Plot with multiple aesthetics
p <- ggplot(mtcars, aes(mpg, wt,
                        color = factor(cyl),
                        size = hp,
                        shape = factor(am))) +
  geom_point() +
  labs(color = "Cylinders", size = "Horsepower", shape = "Transmission")

p

## ----hide-legends-------------------------------------------------------------
# Hide the size legend
p + legend_hide(size)

# Hide multiple legends
p + legend_hide(size, shape)

## ----select-legends-----------------------------------------------------------
# Keep only the colour legend
p + legend_select(colour)

# Keep colour and shape
p + legend_select(colour, shape)

## ----order-legends------------------------------------------------------------
# Default order
p

# Size legend first, then colour, then shape
p + legend_order_guides(size = 1, colour = 2, shape = 3)

## ----merge-legends------------------------------------------------------------
# Plot where colour and fill map to the same variable
p_merge <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(shape = 21, size = 4, stroke = 1.5) +
  labs(color = "Cylinders", fill = "Cylinders")

# Legends merge automatically when titles and labels match
p_merge

# Explicitly request merge (reinforces default behavior)
p_merge + legend_merge(colour, fill)

## ----split-legends------------------------------------------------------------
# Force separate legends even when they could merge
p_merge + legend_split(colour, fill)

## ----position-separately------------------------------------------------------
# Place colour legend on the left, size legend at bottom
p +
  legend_hide(shape) +
  legend_left(by = "colour") +
  legend_bottom(by = "size")

## ----position-top-right-------------------------------------------------------
# Colour legend on top, size on right
p +
  legend_hide(shape) +
  legend_top(by = "colour") +
  legend_right(by = "size")

## ----style-separately---------------------------------------------------------
p +
  legend_hide(shape) +
  legend_style(title_face = "bold", background = "grey95", by = "colour") +
  legend_style(size = 10, by = "size")

## ----combined-----------------------------------------------------------------
# Complex example: hide shape, position colour on left with bold title,
# position size at bottom with smaller text
p +
  legend_hide(shape) +
  legend_left(by = "colour") +
  legend_style(title_face = "bold", title_size = 14, by = "colour") +
  legend_bottom(by = "size") +
  legend_style(size = 9, direction = "horizontal", by = "size")

## ----four-sides, fig.width = 8, fig.height = 6--------------------------------
p4 <- ggplot(mtcars, aes(mpg, wt,
                         colour = factor(cyl),
                         fill   = factor(gear),
                         size   = hp,
                         shape  = factor(am))) +
  geom_point(stroke = 1.2) +
  labs(colour = "Cyl", fill = "Gear", size = "HP", shape = "AM")

p4 +
  # 1. Send each legend to its side
  legend_top   (by = "colour") +
  legend_bottom(by = "fill")   +
  legend_left  (by = "size")   +
  legend_right (by = "shape")  +

  # 2. Slide each legend along its side
  legend_style(by = "colour", justification = "left") +
  legend_style(by = "fill",   justification = "right") +
  legend_style(by = "size",   justification = "top") +
  legend_style(by = "shape",  justification = "bottom") +

  # 3. Nudge each legend toward/away from the panel via margin (cm)
  legend_style(by = "colour", margin = c(0, 0, 0.3, 0)) +
  legend_style(by = "fill",   margin = c(0.3, 0, 0, 0)) +
  legend_style(by = "size",   margin = c(0, 0.3, 0, 0)) +
  legend_style(by = "shape",  margin = c(0, 0, 0, 0.3))

## ----six-legends, fig.width = 9, fig.height = 6.5-----------------------------
p6 <- ggplot(mtcars, aes(mpg, wt)) +
  geom_smooth(aes(linetype = factor(vs)), method = "lm", se = FALSE,
              colour = "grey40") +
  geom_point(aes(colour = factor(cyl),
                 fill   = factor(gear),
                 size   = hp,
                 alpha  = qsec,
                 shape  = factor(am)),
             stroke = 1.2) +
  scale_shape_manual(values = c(21, 24)) +
  labs(colour = "Cyl", fill = "Gear", size = "HP",
       alpha = "QSec", shape = "AM", linetype = "VS")

p6 +
  # 1. Side placement — two legends share the top, two share the right
  legend_top   (by = "colour")   +
  legend_top   (by = "fill")     +
  legend_right (by = "size")     +
  legend_right (by = "alpha")    +
  legend_bottom(by = "shape")    +
  legend_left  (by = "linetype") +

  # 2. Slide each legend along its rail
  #    top/bottom rails: "left" / "center" / "right" (or a number in [0,1])
  #    left/right rails: "top"  / "center" / "bottom"
  legend_style(by = "colour",   justification = "left")   +
  legend_style(by = "fill",     justification = "right")  +
  legend_style(by = "size",     justification = "top")    +
  legend_style(by = "alpha",    justification = "bottom") +
  legend_style(by = "shape",    justification = "center") +
  legend_style(by = "linetype", justification = "center") +

  # 3. Appearance — per-legend title weight, key size, direction, text size
  legend_style(by = "colour",   title_face = "bold",
               key_width = 0.4, key_height = 0.4) +
  legend_style(by = "fill",     title_face = "bold",
               key_width = 0.4, key_height = 0.4) +
  legend_style(by = "size",     title_size = 9, size = 8)  +
  legend_style(by = "alpha",    title_size = 9, size = 8)  +
  legend_style(by = "shape",    direction = "horizontal")  +
  legend_style(by = "linetype", direction = "vertical")    +

  # 4. Nudge each legend toward/away from the panel via margin (cm)
  #    order is c(top, right, bottom, left)
  legend_style(by = "colour",   margin = c(0, 0, 0.2, 0)) +
  legend_style(by = "fill",     margin = c(0, 0, 0.2, 0)) +
  legend_style(by = "size",     margin = c(0, 0, 0, 0.3)) +
  legend_style(by = "alpha",    margin = c(0, 0, 0, 0.3)) +
  legend_style(by = "shape",    margin = c(0.3, 0, 0, 0)) +
  legend_style(by = "linetype", margin = c(0, 0.3, 0, 0))

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)

## ----style-size, fig.show='hold', out.width='48%'-----------------------------
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p + ggtitle("Default size")
p + legend_style(size = 14) + ggtitle("size = 14")

## ----style-family, fig.show='hold', out.width='48%'---------------------------
p + legend_style(family = "serif") + ggtitle("serif")
p + legend_style(family = "mono") + ggtitle("mono")

## ----style-title--------------------------------------------------------------
p + legend_style(
  size = 12,
  title_size = 14,
  title_face = "bold"
)

## ----rotation-basic, fig.width=7----------------------------------------------
p_long <- ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point()

p_long + legend_style(angle = 45)

## ----rotation-vertical, fig.width=7-------------------------------------------
p_long + legend_style(angle = 90)

## ----rotation-title, fig.show='hold', out.width='48%'-------------------------
p_long + legend_style(title_angle = 90, title_hjust = 0.5) + ggtitle("Rotated title only")
p_long + legend_style(angle = 45, title_angle = 90) + ggtitle("Both rotated")

## ----style-background---------------------------------------------------------
p + legend_style(
  background = "#FFF3E0"
)

## ----style-border-------------------------------------------------------------
p + legend_style(
  background = "#FFF3E0",
  background_color = "#FF9800"
)

## ----style-key----------------------------------------------------------------
p + legend_style(
  key_width = 1.5,
  key_height = 1.5
)

## ----style-margin-------------------------------------------------------------
p + legend_style(
  background = "#FFF3E0",
  margin = 0.5
)

## ----style-full---------------------------------------------------------------
p + legend_style(
  size = 11,
  title_size = 13,
  title_face = "bold",
  family = "sans",
  key_width = 1.2,
  background = "#FFF3E0",
  background_color = "#FF9800",
  margin = 0.3
)

## ----wrap-ncol, fig.width=7---------------------------------------------------
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2)

## ----wrap-nrow, fig.width=7---------------------------------------------------
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(nrow = 2)

## ----wrap-bottom, fig.width=7-------------------------------------------------
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(nrow = 2) +
  legend_bottom()

## ----keys-size, fig.show='hold', out.width='48%'------------------------------
p_small <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 1) +
  labs(color = "Cylinders")

p_small + ggtitle("Small points in legend")
p_small + legend_keys(size = 4) + ggtitle("Enlarged legend keys")

## ----keys-alpha, fig.show='hold', out.width='48%'-----------------------------
p_alpha <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(alpha = 0.3, size = 3) +
  labs(color = "Cylinders")

p_alpha + ggtitle("Transparent legend keys")
p_alpha + legend_keys(alpha = 1) + ggtitle("Opaque legend keys")

## ----keys-combined------------------------------------------------------------
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(alpha = 0.3, size = 1) +
  legend_keys(size = 4, alpha = 1)

## ----keys-fill----------------------------------------------------------------
ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
  geom_boxplot(alpha = 0.5) +
  legend_keys(alpha = 1, aesthetic = "fill")

## ----keys-shape, fig.show='hold', out.width='48%'-----------------------------
p + legend_keys(shape = "square") + ggtitle("Square")
p + legend_keys(shape = "diamond") + ggtitle("Diamond")

## ----keys-outline, fig.show='hold', out.width='48%'---------------------------
# White fill with colored outline - works with color mapping only
p + legend_keys(shape = "circle_filled", fill = "white", stroke = 1.5) +
  ggtitle("White fill, colored outline")

# Colored fill with black outline - requires mapping BOTH color and fill
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(size = 3, shape = 21, stroke = 1) +
  legend_keys(colour = "black", stroke = 1) +
  ggtitle("Colored fill, black outline")

## ----order-explicit-----------------------------------------------------------
p + legend_order(c("8", "6", "4"))

## ----order-rev, fig.show='hold', out.width='48%'------------------------------
p + legend_order(rev) + ggtitle("Reversed")
p + legend_order(sort) + ggtitle("Sorted")

## ----order-fill---------------------------------------------------------------
ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
  geom_bar() +
  legend_order(c("8", "4", "6"), aesthetic = "fill")

## ----reverse------------------------------------------------------------------
p + legend_reverse()

## ----combined-styling, fig.width=7--------------------------------------------
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_left() +
  legend_style(
    size = 11,
    title_face = "bold",
    background = "#FFF3E0"
  )

## ----combined-wrap-style, fig.width=7-----------------------------------------
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2) +
  legend_bottom() +
  legend_style(size = 10, title_face = "bold")

## ----colorbar-basic-----------------------------------------------------------
p_cont <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
  geom_tile()

p_cont

## ----colorbar-size------------------------------------------------------------
p_cont + colorbar_style(width = 0.5, height = 10, aesthetic = "fill")

## ----colorbar-horizontal------------------------------------------------------
p_cont + colorbar_style(width = 10, height = 0.5, direction = "horizontal", aesthetic = "fill") +
  legend_bottom()

## ----colorbar-frame, fig.show='hold', out.width='48%'-------------------------
p_cont + colorbar_style(frame = TRUE, aesthetic = "fill") + ggtitle("Black frame")
p_cont + colorbar_style(frame = "#FF9800", aesthetic = "fill") + ggtitle("Orange frame")

## ----colorbar-noticks---------------------------------------------------------
p_cont + colorbar_style(ticks = FALSE, frame = "#FF9800", aesthetic = "fill")

## ----colorbar-full------------------------------------------------------------
p_cont + colorbar_style(
  width = 0.5,
  height = 8,
  frame = "#E65100",
  ticks_length = 0.3,
  aesthetic = "fill"
)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)

## ----left-right, fig.show='hold', out.width='48%'-----------------------------
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p + legend_left() + ggtitle("Left")
p + legend_right() + ggtitle("Right")

## ----top-bottom, fig.show='hold', out.width='48%'-----------------------------
p + legend_top() + ggtitle("Top")
p + legend_bottom() + ggtitle("Bottom")

## ----align-to, fig.show='hold', out.width='48%'-------------------------------
p_labeled <- p + labs(title = "Fuel Economy", y = "Weight (1000 lbs)")

# Default: align to panel (legend aligns with the plot area)
p_labeled + legend_top() + ggtitle("Panel alignment (default)")

# Align to full plot (legend spans title/axis labels too)
p_labeled + legend_top(align_to = "plot") + ggtitle("Plot alignment")

## ----inside-shortcuts, fig.show='hold', out.width='48%'-----------------------
p + legend_inside(position = "topright") + ggtitle("Top right")
p + legend_inside(position = "bottomleft") + ggtitle("Bottom left")

## ----inside-coords------------------------------------------------------------
p + legend_inside(x = 0.95, y = 0.95, justification = c("right", "top"))

## ----inside-background--------------------------------------------------------
p + legend_inside(
  position = "topright",
  background = "#FFF3E0",
  border = "#FF9800"
)

## ----direction, fig.show='hold', out.width='48%'------------------------------
p + legend_horizontal() + ggtitle("Horizontal")
p + legend_vertical() + ggtitle("Vertical")

## ----direction-position-------------------------------------------------------
p + legend_right() + legend_horizontal()

## ----legend-none--------------------------------------------------------------
p + legend_none()

# =============================================================================
# Generate Example Images for ggguides Documentation
# Run this script from the package root directory
# =============================================================================

library(ggplot2)
library(ggguides)
library(svglite)

# Theme colors matching pkgdown Sandstone theme (Bootstrap 5)
LIGHT_BG <- "#F5F6F8"
LIGHT_BORDER <- "#DFD7CA"
LIGHT_TEXT <- "#3E3F3A"
DARK_BG <- "#343739"
DARK_TEXT <- "#DFD7CA"
FONT_FAMILY <- '"Roboto", -apple-system, BlinkMacSystemFont, "Segoe UI", system-ui, sans-serif'

# Output directory
out_dir <- "man/figures"
if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

# Set transparent theme for ggplot2
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)

# Base plot for examples
base_plot <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders", title = "Example Plot")

#' Make an SVG file theme-aware (simplified version of build_pkgdown.R)
#'
#' @param svg_path Path to SVG file
make_svg_theme_aware <- function(svg_path) {
  svg_content <- paste(readLines(svg_path, warn = FALSE), collapse = "\n")

  # Add svg-bg class to main background rect and set light bg color
  svg_content <- sub(
    "<rect width='100%' height='100%' style='stroke: none; fill: none;'/>",
    sprintf("<rect class='svg-bg' width='100%%' height='100%%' style='stroke: none; fill: %s;'/>", LIGHT_BG),
    svg_content, fixed = TRUE
  )

  # Replace gridline colors
  svg_content <- gsub("stroke: #EBEBEB;", sprintf("stroke: %s;", LIGHT_TEXT), svg_content, fixed = TRUE)

  # Modify CSS rules - remove rect from combined rule, add separate rect rule

  line_rule_pat <- paste0(
    "    .svglite line, .svglite polyline, .svglite polygon, ",
    ".svglite path, .svglite rect, .svglite circle {\n",
    "      fill: none;\n",
    "      stroke: #000000;\n",
    "      stroke-linecap: round;\n",
    "      stroke-linejoin: round;\n",
    "      stroke-miterlimit: 10.00;\n",
    "    }"
  )
  new_line_rule <- sprintf(
    paste0(
      "    /* theme-aware-processed */\n",
      "    .svglite line, .svglite polyline, .svglite polygon, ",
      ".svglite path, .svglite circle {\n",
      "      fill: none;\n",
      "      stroke: %s;\n",
      "      stroke-linecap: round;\n",
      "      stroke-linejoin: round;\n",
      "      stroke-miterlimit: 10.00;\n",
      "    }\n",
      "    .svglite > g > rect {\n",
      "      fill: none;\n",
      "      stroke: %s;\n",
      "      stroke-linecap: round;\n",
      "      stroke-linejoin: round;\n",
      "      stroke-miterlimit: 10.00;\n",
      "    }"
    ),
    LIGHT_TEXT, LIGHT_TEXT
  )
  svg_content <- sub(line_rule_pat, new_line_rule, svg_content, fixed = TRUE)

  # Update text rule with fill color
  text_rule_pat <- paste0(
    "    .svglite text {\n",
    "      white-space: pre;\n",
    "    }"
  )
  new_text_rule <- sprintf(
    paste0(
      "    .svglite text {\n",
      "      white-space: pre;\n",
      "      fill: %s;\n",
      "    }"
    ),
    LIGHT_TEXT
  )
  svg_content <- sub(text_rule_pat, new_text_rule, svg_content, fixed = TRUE)

  # Update font family
  svg_content <- gsub(
    'font-family: "Arial";',
    sprintf("font-family: %s;", FONT_FAMILY),
    svg_content, fixed = TRUE
  )

  writeLines(svg_content, svg_path)
}

# Helper to save plots with theme-aware SVG processing
save_example <- function(p, name, width = 6, height = 4) {
  filepath <- file.path(out_dir, paste0(name, ".svg"))
  ggsave(
    filename = filepath,
    plot = p,
    width = width,
    height = height,
    device = svglite,
    bg = "transparent"
  )
  make_svg_theme_aware(filepath)
  message("Saved: ", name, ".svg")
}

# =============================================================================
# Position Examples
# =============================================================================

# legend_left
save_example(
  base_plot + legend_left(),
  "legend_left"
)

# legend_right
save_example(
  base_plot + legend_right(),
  "legend_right"
)

# legend_top
save_example(
  base_plot + legend_top(),
  "legend_top"
)

# legend_bottom
save_example(
  base_plot + legend_bottom(),
  "legend_bottom"
)

# legend_none
save_example(
  base_plot + legend_none(),
  "legend_none"
)

# legend_inside - topright
save_example(
  base_plot + legend_inside(position = "topright"),
  "legend_inside_topright"
)

# legend_inside - bottomleft
save_example(
  base_plot + legend_inside(position = "bottomleft"),
  "legend_inside_bottomleft"
)

# legend_inside - center with styling
save_example(
  base_plot + legend_inside(position = "center", background = "grey95", border = "grey50"),
  "legend_inside_styled"
)

# =============================================================================
# Direction Examples
# =============================================================================

# legend_horizontal
save_example(
  base_plot + legend_bottom() + legend_horizontal(),
  "legend_horizontal"
)

# legend_vertical (default, but explicit)
save_example(
  base_plot + legend_right() + legend_vertical(),
  "legend_vertical"
)

# =============================================================================
# Style Examples
# =============================================================================

# legend_style - size only
save_example(
  base_plot + legend_style(size = 14),
  "legend_style_size"
)

# legend_style - font family
save_example(
  base_plot + legend_style(family = "serif"),
  "legend_style_font"
)

# legend_style - basic (size + family)
save_example(
  base_plot + legend_style(size = 14, family = "serif"),
  "legend_style_basic"
)

# legend_style - comprehensive
save_example(
  base_plot + legend_style(
    size = 12,
    title_size = 14,
    title_face = "bold",
    key_width = 1.5,
    background = "grey95",
    background_color = "grey70",
    margin = 0.3
  ),
  "legend_style_full"
)

# legend_wrap - use mpg for more levels
wrap_plot <- ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  labs(color = "Class")

save_example(
  wrap_plot + legend_wrap(ncol = 2),
  "legend_wrap_ncol2"
)

save_example(
  wrap_plot + legend_wrap(nrow = 2),
  "legend_wrap_nrow2"
)

# legend_reverse
save_example(
  base_plot + legend_reverse(),
  "legend_reverse"
)

# =============================================================================
# Combined Examples
# =============================================================================

# Position + Style
save_example(
  base_plot +
    legend_left() +
    legend_style(size = 12, title_face = "bold", background = "grey95"),
  "combined_left_styled"
)

# Inside + Style
save_example(
  base_plot +
    legend_inside(position = "topright", background = "white", border = "grey50") +
    legend_style(size = 11, title_face = "bold"),
  "combined_inside_styled"
)

# Wrap + Position
save_example(
  wrap_plot +
    legend_wrap(ncol = 2) +
    legend_bottom(),
  "combined_wrap_bottom",
  width = 7, height = 5
)

# =============================================================================
# Patchwork Examples (if available)
# =============================================================================

if (requireNamespace("patchwork", quietly = TRUE)) {
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    labs(title = "Plot 1", color = "Cyl")

  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
    geom_point() +
    labs(title = "Plot 2", color = "Cyl")

  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
    geom_point() +
    labs(title = "Plot 3", color = "Cyl")

  # Without collection
  save_example(
    p1 | p2,
    "patchwork_no_collect",
    width = 8, height = 4
  )

  # With collection
  save_example(
    collect_legends(p1 | p2),
    "patchwork_collect",
    width = 8, height = 4
  )

  # Collection at bottom
  save_example(
    collect_legends(p1 | p2, position = "bottom"),
    "patchwork_collect_bottom",
    width = 8, height = 4
  )

  # Stacked plots - default (centered legend)
  save_example(
    collect_legends(p1 / p2 / p3, position = "right"),
    "patchwork_stacked_default",
    width = 6, height = 8
  )

  # Helper for grob-based saves (gtable objects)
  save_grob_example <- function(gt, name, width = 6, height = 8) {
    filepath <- file.path(out_dir, paste0(name, ".svg"))
    svglite(filepath, width = width, height = height, bg = "transparent")
    grid::grid.draw(gt)
    dev.off()
    make_svg_theme_aware(filepath)
    message("Saved: ", name, ".svg")
  }

  # Stacked plots - spanning legend
  gt <- collect_legends(p1 / p2 / p3, position = "right", span = TRUE)
  save_grob_example(gt, "patchwork_stacked_span", width = 6, height = 8)

  # Stacked plots - span row 1 only
  gt <- collect_legends(p1 / p2 / p3, position = "right", span = 1)
  save_grob_example(gt, "patchwork_span_row1", width = 6, height = 8)

  # Stacked plots - span rows 1:2
  gt <- collect_legends(p1 / p2 / p3, position = "right", span = 1:2)
  save_grob_example(gt, "patchwork_span_row12", width = 6, height = 8)
}

# =============================================================================
# Six Legends, Stacked per Side (README hero)
# =============================================================================

p6 <- ggplot(mtcars, aes(mpg, wt)) +
  geom_smooth(aes(linetype = factor(vs)), method = "lm", se = FALSE,
              colour = "grey40") +
  geom_point(aes(colour = factor(cyl),
                 fill   = factor(gear),
                 size   = hp,
                 alpha  = qsec,
                 shape  = factor(am)),
             stroke = 1.2) +
  scale_shape_manual(values = c(21, 24)) +
  labs(colour = "Cyl", fill = "Gear", size = "HP",
       alpha = "QSec", shape = "AM", linetype = "VS")

save_example(
  p6 +
    legend_top(by = "colour")    + legend_top(by = "fill")     +
    legend_right(by = "size")    + legend_right(by = "alpha")  +
    legend_bottom(by = "shape")  + legend_left(by = "linetype") +
    legend_style(by = "colour",   justification = "left")   +
    legend_style(by = "fill",     justification = "right")  +
    legend_style(by = "size",     justification = "top")    +
    legend_style(by = "alpha",    justification = "bottom") +
    legend_style(by = "shape",    justification = "center") +
    legend_style(by = "linetype", justification = "center") +
    legend_style(by = "colour", title_face = "bold",
                 key_width = 0.4, key_height = 0.4) +
    legend_style(by = "fill",   title_face = "bold",
                 key_width = 0.4, key_height = 0.4) +
    legend_style(by = "size",  title_size = 9, size = 8) +
    legend_style(by = "alpha", title_size = 9, size = 8) +
    legend_style(by = "shape",    direction = "horizontal") +
    legend_style(by = "linetype", direction = "vertical")   +
    legend_style(by = "colour",   margin = c(0, 0, 0.2, 0)) +
    legend_style(by = "fill",     margin = c(0, 0, 0.2, 0)) +
    legend_style(by = "size",     margin = c(0, 0, 0, 0.3)) +
    legend_style(by = "alpha",    margin = c(0, 0, 0, 0.3)) +
    legend_style(by = "shape",    margin = c(0.3, 0, 0, 0)) +
    legend_style(by = "linetype", margin = c(0, 0.3, 0, 0)),
  "six_legends",
  width = 9, height = 6.5
)

message("\nAll examples generated in: ", out_dir)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)

## ----problem, fig.height=4----------------------------------------------------
library(patchwork)

p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1", color = "Cylinders")
p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2", color = "Cylinders")

# Default patchwork: duplicate legends
p1 | p2

## ----collect-basic, fig.height=4----------------------------------------------
collect_legends(p1 | p2)

## ----collect-bottom, fig.height=5---------------------------------------------
collect_legends(p1 | p2, position = "bottom")

## ----collect-left, fig.height=4-----------------------------------------------
collect_legends(p1 | p2, position = "left")

## ----stacked-default, fig.height=7--------------------------------------------
# Create plots with different heights using plot_layout
p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3", color = "Cylinders")

# Stack with different heights: 1, 1/2, 1/4
stacked <- (p1 / p2 / p3) + plot_layout(heights = c(4, 2, 1))
collect_legends(stacked, position = "right")

## ----stacked-span, fig.height=7-----------------------------------------------
gt <- collect_legends(stacked, position = "right", span = TRUE)
grid::grid.draw(gt)

## ----span-row1, fig.height=7--------------------------------------------------
# Legend attached to row 1 only (the tallest plot)
gt <- collect_legends(stacked, position = "right", span = 1)
grid::grid.draw(gt)

## ----span-row12, fig.height=7-------------------------------------------------
# Legend attached to rows 1 and 2
gt <- collect_legends(stacked, position = "right", span = 1:2)
grid::grid.draw(gt)

## ----collect-axes, fig.height=5-----------------------------------------------
# Plots stacked vertically - x-axis is duplicated
p_top <- ggplot(mtcars, aes(mpg, wt)) +
  geom_point() + labs(y = "Weight")

p_bottom <- ggplot(mtcars, aes(mpg, disp)) +
  geom_point() + labs(y = "Displacement")

# Without axis collection (both have x-axis)
p_top / p_bottom

## ----collect-axes-fixed, fig.height=5-----------------------------------------
# With axis collection (removes redundant x-axis from top)
collect_axes(p_top / p_bottom)

## ----combined, fig.height=5---------------------------------------------------
p1_styled <- p1 + legend_style(size = 11, title_face = "bold")
p2_styled <- p2 + legend_style(size = 11, title_face = "bold")

collect_legends(p1_styled | p2_styled, position = "right")

## ----complex, fig.height=6----------------------------------------------------
p4 <- ggplot(mtcars, aes(qsec, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 4", color = "Cylinders")

# 2x2 grid
layout <- (p1 | p2) / (p3 | p4)
collect_legends(layout, position = "right")

## ----cowplot-setup, include=FALSE---------------------------------------------
# Fresh plots for cowplot examples (not modified by patchwork)
cp1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1", color = "Cylinders")
cp2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2", color = "Cylinders")
cp3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3", color = "Cylinders")
cp4 <- ggplot(mtcars, aes(qsec, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 4", color = "Cylinders")

## ----get-legend---------------------------------------------------------------
# Extract legend from a plot
leg <- get_legend(cp1)

# Use with cowplot::plot_grid() or grid::grid.draw()
grid::grid.newpage()
grid::grid.draw(leg)

## ----shared-legend-side, fig.height=4-----------------------------------------
# Side-by-side with shared legend on right
gt <- shared_legend(cp1, cp2, ncol = 2, position = "right")
grid::grid.newpage()
grid::grid.draw(gt)

## ----shared-legend-stacked, fig.height=6--------------------------------------
# Stacked with legend at bottom
gt <- shared_legend(cp1, cp2, cp3, ncol = 1, position = "bottom")
grid::grid.newpage()
grid::grid.draw(gt)

## ----shared-legend-grid, fig.height=5-----------------------------------------
# 2x2 grid
gt <- shared_legend(cp1, cp2, cp3, cp4, ncol = 2, nrow = 2, position = "right")
grid::grid.newpage()
grid::grid.draw(gt)

#' Customize Legend Key Appearance
#'
#' Modify the appearance of legend keys (the symbols/glyphs in the legend)
#' without affecting the plot itself. This is a simpler alternative to using
#' `guide_legend(override.aes = list(...))`.
#'
#' @param size Numeric. Size of point keys.
#' @param alpha Numeric. Alpha (transparency) of keys, between 0 and 1.
#' @param shape Shape of point keys. Can be:
#'   \itemize{
#'     \item Numeric (0-25): Standard ggplot2 shape codes
#'     \item Character name: \code{"circle"}, \code{"square"}, \code{"diamond"},
#'       \code{"triangle"}, \code{"triangle_down"}, \code{"plus"}, \code{"cross"},
#'       \code{"asterisk"}, \code{"circle_open"}, \code{"square_open"},
#'       \code{"diamond_open"}, \code{"triangle_open"}, \code{"circle_filled"},
#'       \code{"square_filled"}, \code{"diamond_filled"}, \code{"triangle_filled"}
#'   }
#'   Shapes 21-25 (or names ending in \code{"_filled"}) support both outline
#'   (\code{colour}) and fill (\code{fill}) colors.
#' @param linewidth Numeric. Width of line keys.
#' @param linetype Character or numeric. Line type for line keys.
#' @param fill Character. Fill color for filled shapes (shapes 21-25). For
#'   shapes 0-20, use \code{colour} instead.
#' @param colour,color Character. Outline/stroke color for keys. For shapes
#'   21-25, this controls the outline; for shapes 0-20, this is the main color.
#' @param stroke Numeric. Stroke width for point outlines (shapes 21-25).
#' @param aesthetic Character vector specifying which aesthetic(s) to modify.
#'   Default is `c("colour", "fill")` which covers most common cases.
#'   Use `"all"` to apply to all legends.
#'
#' @return A list of ggplot2 guide specifications.
#'
#' @details
#' This function wraps `guide_legend(override.aes = ...)` to provide a cleaner
#' interface for common legend key modifications. It's particularly useful for:
#'
#' \itemize{
#'   \item Making small points more visible in the legend
#'   \item Removing transparency from legend keys
#'   \item Changing symbol shapes to improve clarity
#'   \item Adding outlines to filled shapes for better visibility
#' }
#'
#' \strong{Shape types:}
#' \itemize{
#'   \item Shapes 0-14: Outline only (color from \code{colour})
#'   \item Shapes 15-20: Filled solid (color from \code{colour})
#'   \item Shapes 21-25: Outline + fill (outline from \code{colour}, interior
#'     from \code{fill})
#' }
#'
#' \strong{Important note for filled shapes (21-25):}
#'
#' When using filled shapes with both outline and fill colors, the behavior
#' depends on which aesthetics are mapped in your original plot:
#'
#' \itemize{
#'   \item \strong{White fill, colored outline}: Works with \code{aes(color = var)}.
#'     Use \code{legend_keys(shape = "circle_filled", fill = "white")}.
#'   \item \strong{Colored fill, black outline}: Requires \code{aes(color = var, fill = var)}
#'     in your plot. Then use \code{legend_keys(colour = "black")}.
#' }
#'
#' This is because \code{override.aes} can only set static values; it cannot
#' inherit from mapped aesthetics. If you only map \code{color} and try to
#' override the outline to black, the fill will not have a color mapping to use.
#'
#' Only non-NULL arguments are applied, so you can selectively modify specific
#' properties.
#'
#' @examples
#' library(ggplot2)
#'
#' # Points get lost in legend - make them bigger
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point(size = 1) +
#'   legend_keys(size = 4)
#'
#' # Remove transparency from legend
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point(alpha = 0.3, size = 3) +
#'   legend_keys(alpha = 1)
#'
#' # Change shape using name
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point(size = 3) +
#'   legend_keys(shape = "square")
#'
#' # Filled shape with white fill and colored outline (shapes 21-25)
#' # Works because we set fill to a static color (white)
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point(size = 3) +
#'   legend_keys(shape = "circle_filled", fill = "white", stroke = 1.5)
#'
#' # Colored fill with black outline - MUST map both color AND fill in the plot
#' # This is a ggplot2 limitation: override.aes can only set static values,
#' # it cannot make fill "inherit" from color
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
#'   geom_point(size = 3, shape = 21, stroke = 1) +
#'   legend_keys(colour = "black", stroke = 1)
#'
#' # Apply to fill aesthetic (e.g., for boxplots)
#' ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
#'   geom_boxplot(alpha = 0.5) +
#'   legend_keys(alpha = 1, aesthetic = "fill")
#'
#' @seealso
#' \code{\link{legend_style}} for styling legend text and background,
#' \code{\link{legend_order}} for reordering legend entries.
#'
#' @export
legend_keys <- function(size = NULL,
                        alpha = NULL,
                        shape = NULL,
                        linewidth = NULL,
                        linetype = NULL,
                        fill = NULL,
                        colour = NULL,
                        color = NULL,
                        stroke = NULL,
                        aesthetic = c("colour", "fill")) {

 # Handle color/colour
 if (!is.null(color) && is.null(colour)) {
   colour <- color
 }

 # Convert shape names to numeric codes
 if (!is.null(shape) && is.character(shape)) {
   shape <- shape_name_to_code(shape)
 }

 # Warn if user likely wants colored fill with black outline but hasn't mapped fill
 # This pattern (filled shape + colour override but no fill override) often indicates
 # the user expects fill to inherit from the color mapping, which doesn't work
 is_filled_shape <- !is.null(shape) && (
   (is.numeric(shape) && shape %in% 21:25) ||
   (is.character(shape) && grepl("_filled$", tolower(shape)))
 )
 if (is_filled_shape && !is.null(colour) && is.null(fill)) {
   message(
     "Note: Using filled shapes (21-25) with a colour override but no fill.\n",
     "For colored fills with a black/custom outline, you must map both color AND fill\n",
     "in your plot: aes(color = var, fill = var), then use legend_keys(colour = \"black\")."
   )
 }

 # Build override.aes list with only non-NULL values
 override_aes <- list()
 if (!is.null(size)) override_aes$size <- size
 if (!is.null(alpha)) override_aes$alpha <- alpha
 if (!is.null(shape)) override_aes$shape <- shape
 if (!is.null(linewidth)) override_aes$linewidth <- linewidth
 if (!is.null(linetype)) override_aes$linetype <- linetype
 if (!is.null(fill)) override_aes$fill <- fill
 if (!is.null(colour)) override_aes$colour <- colour
 if (!is.null(stroke)) override_aes$stroke <- stroke

 if (length(override_aes) == 0) {
   warning("No key modifications specified. Returning NULL.")
   return(NULL)
 }

 # Create guide specification
 guide_spec <- ggplot2::guide_legend(override.aes = override_aes)

 # Handle "all" aesthetic
 if (identical(aesthetic, "all")) {
   aesthetic <- c("colour", "fill", "shape", "size", "linetype", "alpha")
 }

 # Normalize aesthetic names
 aesthetic <- tolower(aesthetic)
 aesthetic[aesthetic == "color"] <- "colour"

 # Create guides() call
 guides_args <- stats::setNames(
   rep(list(guide_spec), length(aesthetic)),
   aesthetic
 )

 do.call(ggplot2::guides, guides_args)
}

# =============================================================================
# Internal Helpers
# =============================================================================

#' Convert shape name to numeric code
#' @noRd
shape_name_to_code <- function(name) {
  # Shape name mapping
  # Outline only (0-14)
  # Filled solid (15-20) - color from colour

  # Outline + fill (21-25) - outline from colour, interior from fill
  shape_map <- c(
    # Basic shapes (solid, color from colour aesthetic)
    "circle" = 16,
    "square" = 15,
    "diamond" = 18,
    "triangle" = 17,
    "triangle_down" = 25,

    # Open/outline shapes (stroke only)
    "circle_open" = 1,
    "square_open" = 0,
    "diamond_open" = 5,
    "triangle_open" = 2,

    # Filled shapes with outline (21-25: stroke from colour, fill from fill)
    "circle_filled" = 21,
    "square_filled" = 22,
    "diamond_filled" = 23,
    "triangle_filled" = 24,
    "triangle_down_filled" = 25,

    # Symbols
    "plus" = 3,
    "cross" = 4,
    "asterisk" = 8
  )

  name_lower <- tolower(name)

  if (name_lower %in% names(shape_map)) {
    return(shape_map[[name_lower]])
  }

  # If not found, return as-is (might be a single character like "a")
  name
}

library(testthat)
library(ggplot2)

# =============================================================================
# legend_keys() tests
# =============================================================================

test_that("legend_keys returns a Guides object", {
  result <- legend_keys(size = 4)
  expect_s3_class(result, "Guides")
})

test_that("legend_keys with no args returns NULL with warning", {
  expect_warning(result <- legend_keys(), "No key modifications specified")
  expect_null(result)
})

test_that("legend_keys can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point(size = 1) +
    legend_keys(size = 4)
  expect_s3_class(p, "gg")
})

# =============================================================================
# Size parameter
# =============================================================================

test_that("legend_keys accepts size parameter", {
  result <- legend_keys(size = 5)
  expect_s3_class(result, "Guides")
})

# =============================================================================
# Alpha parameter
# =============================================================================

test_that("legend_keys accepts alpha parameter", {
  result <- legend_keys(alpha = 1)
  expect_s3_class(result, "Guides")

  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point(alpha = 0.3) +
    legend_keys(alpha = 1)
  expect_s3_class(p, "gg")
})

# =============================================================================
# Shape parameter - numeric codes
# =============================================================================

test_that("legend_keys accepts numeric shape codes", {
  # Outline shapes (0-14)
  result <- legend_keys(shape = 1)
  expect_s3_class(result, "Guides")

  # Solid shapes (15-20)
  result <- legend_keys(shape = 16)
  expect_s3_class(result, "Guides")

  # Filled shapes with outline (21-25)
  result <- legend_keys(shape = 21)
  expect_s3_class(result, "Guides")
})

# =============================================================================
# Shape parameter - character names
# =============================================================================

test_that("legend_keys accepts shape names", {
  shape_names <- c(
    "circle", "square", "diamond", "triangle", "triangle_down",
    "circle_open", "square_open", "diamond_open", "triangle_open",
    "circle_filled", "square_filled", "diamond_filled", "triangle_filled",
    "plus", "cross", "asterisk"
  )

  for (shape_name in shape_names) {
    result <- legend_keys(shape = shape_name)
    expect_s3_class(result, "Guides")
  }
})

test_that("legend_keys converts shape names to codes correctly", {
  # Test shape_name_to_code internal function
  expect_equal(ggguides:::shape_name_to_code("circle"), 16)
  expect_equal(ggguides:::shape_name_to_code("square"), 15)
  expect_equal(ggguides:::shape_name_to_code("diamond"), 18)
  expect_equal(ggguides:::shape_name_to_code("triangle"), 17)
  expect_equal(ggguides:::shape_name_to_code("circle_open"), 1)
  expect_equal(ggguides:::shape_name_to_code("square_open"), 0)
  expect_equal(ggguides:::shape_name_to_code("circle_filled"), 21)
  expect_equal(ggguides:::shape_name_to_code("plus"), 3)
  expect_equal(ggguides:::shape_name_to_code("cross"), 4)
  expect_equal(ggguides:::shape_name_to_code("asterisk"), 8)
})

test_that("shape_name_to_code returns unknown names as-is", {
  # Single character shapes like "a" should pass through
  expect_equal(ggguides:::shape_name_to_code("a"), "a")
  expect_equal(ggguides:::shape_name_to_code("X"), "X")
})

# =============================================================================
# Line parameters
# =============================================================================

test_that("legend_keys accepts linewidth parameter", {
  result <- legend_keys(linewidth = 2)
  expect_s3_class(result, "Guides")
})

test_that("legend_keys accepts linetype parameter", {
  result <- legend_keys(linetype = "dashed")
  expect_s3_class(result, "Guides")

  result <- legend_keys(linetype = 2)
  expect_s3_class(result, "Guides")
})

# =============================================================================
# Color parameters
# =============================================================================

test_that("legend_keys accepts fill parameter", {
  result <- legend_keys(fill = "white")
  expect_s3_class(result, "Guides")
})

test_that("legend_keys accepts colour parameter", {
  result <- legend_keys(colour = "black")
  expect_s3_class(result, "Guides")
})

test_that("legend_keys accepts color parameter (US spelling)", {
  result <- legend_keys(color = "black")
  expect_s3_class(result, "Guides")
})

test_that("legend_keys accepts stroke parameter", {
  result <- legend_keys(stroke = 1.5)
  expect_s3_class(result, "Guides")
})

# =============================================================================
# Filled shapes with both fill and colour
# =============================================================================

test_that("legend_keys works with filled shapes and fill parameter", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point(size = 3) +
    legend_keys(shape = "circle_filled", fill = "white", stroke = 1.5)
  expect_s3_class(p, "gg")
})

test_that("legend_keys shows message for filled shapes with colour but no fill", {
  expect_message(
    legend_keys(shape = 21, colour = "black"),
    "Using filled shapes"
  )

  expect_message(
    legend_keys(shape = "circle_filled", colour = "black"),
    "Using filled shapes"
  )
})

test_that("legend_keys does not show message when fill is specified", {
  expect_silent(legend_keys(shape = 21, colour = "black", fill = "white"))
})

# =============================================================================
# Aesthetic parameter
# =============================================================================

test_that("legend_keys default aesthetic is colour and fill", {
  result <- legend_keys(size = 4)
  expect_s3_class(result, "Guides")
})

test_that("legend_keys accepts single aesthetic", {
  result <- legend_keys(size = 4, aesthetic = "colour")
  expect_s3_class(result, "Guides")

  result <- legend_keys(size = 4, aesthetic = "fill")
  expect_s3_class(result, "Guides")
})

test_that("legend_keys accepts 'all' aesthetic", {
  result <- legend_keys(size = 4, aesthetic = "all")
  expect_s3_class(result, "Guides")
})

test_that("legend_keys normalizes 'color' to 'colour' in aesthetic", {
  result <- legend_keys(size = 4, aesthetic = "color")
  expect_s3_class(result, "Guides")
})

test_that("legend_keys works with fill aesthetic on boxplot", {
  p <- ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
    geom_boxplot(alpha = 0.5) +
    legend_keys(alpha = 1, aesthetic = "fill")
  expect_s3_class(p, "gg")
})

# =============================================================================
# Multiple parameters combined
# =============================================================================

test_that("legend_keys accepts multiple parameters", {
  result <- legend_keys(
    size = 5,
    alpha = 1,
    shape = "square",
    colour = "black"
  )
  expect_s3_class(result, "Guides")
})

# =============================================================================
# Legend Position Functions
# =============================================================================

#' Place Legend on the Left with Proper Alignment
#'
#' A one-liner to position the legend on the left side of the plot with correct
#' left alignment for both the key box and text labels. This goes beyond simple
#' \code{legend.position = "left"} by also setting justification and box alignment.
#'
#' @param by Optional aesthetic name (character) to position only a specific
#'   legend. When specified, uses per-guide positioning via
#'   \code{guide_legend(position = "left")}. Requires ggplot2 >= 3.5.0.
#'   Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.
#'
#' @return A ggplot2 theme object (when \code{by} is NULL) or a guides
#'   specification (when \code{by} is specified).
#'
#' @details
#' When \code{by} is NULL (default), this function sets three theme elements:
#' \itemize{
#'   \item \code{legend.position = "left"} to place the legend on the left
#'   \item \code{legend.justification = "left"} to left-justify the legend box
#'   \item \code{legend.box.just = "left"} to left-align multiple legend boxes
#' }
#'
#' When \code{by} is specified, only the legend for that aesthetic is moved,
#' allowing different legends to be placed in different positions.
#'
#' @examples
#' library(ggplot2)
#'
#' # Basic usage
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_left()
#'
#' # Works with multiple legends
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), shape = factor(am))) +
#'   geom_point(size = 3) +
#'   legend_left()
#'
#' # Position only the colour legend on the left
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point() +
#'   legend_left(by = "colour") +
#'   legend_bottom(by = "size")
#'
#' @seealso \code{\link{legend_right}}, \code{\link{legend_top}},
#'   \code{\link{legend_bottom}}, \code{\link{legend_inside}},
#'   \code{\link{legend_none}}
#' @export
legend_left <- function(by = NULL) {
  if (is.null(by)) {
    theme(
      legend.position = "left",
      legend.justification = "left",
      legend.box.just = "left"
    )
  } else {
    ggguides_guide_update(
      by = normalize_aesthetic(by),
      guide_params = list(position = "left")
    )
  }
}

#' Place Legend on the Right with Proper Alignment
#'
#' A one-liner to position the legend on the right side of the plot with correct
#' right alignment for both the key box and text labels.
#'
#' @param by Optional aesthetic name (character) to position only a specific
#'   legend. When specified, uses per-guide positioning via
#'   \code{guide_legend(position = "right")}. Requires ggplot2 >= 3.5.0.
#'   Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.
#'
#' @return A ggplot2 theme object (when \code{by} is NULL) or a guides
#'   specification (when \code{by} is specified).
#'
#' @details
#' When \code{by} is NULL (default), this function sets three theme elements:
#' \itemize{
#'   \item \code{legend.position = "right"} to place the legend on the right
#'   \item \code{legend.justification = "right"} to right-justify the legend box
#'   \item \code{legend.box.just = "right"} to right-align multiple legend boxes
#' }
#'
#' When \code{by} is specified, only the legend for that aesthetic is moved,
#' allowing different legends to be placed in different positions.
#'
#' @examples
#' library(ggplot2)
#'
#' # Basic usage
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_right()
#'
#' # Position only the size legend on the right
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point() +
#'   legend_bottom(by = "colour") +
#'   legend_right(by = "size")
#'
#' @seealso \code{\link{legend_left}}, \code{\link{legend_top}},
#'   \code{\link{legend_bottom}}, \code{\link{legend_inside}}
#' @export
legend_right <- function(by = NULL) {
  if (is.null(by)) {
    theme(
      legend.position = "right",
      legend.justification = "right",
      legend.box.just = "right"
    )
  } else {
    ggguides_guide_update(
      by = normalize_aesthetic(by),
      guide_params = list(position = "right")
    )
  }
}

#' Place Legend on Top with Horizontal Layout
#'
#' A one-liner to position the legend above the plot with horizontal layout.
#' Optionally aligns to the full plot area (including title) rather than just
#' the panel.
#'
#' @param align_to Where to align the legend. Either \code{"panel"} (default,
#'   aligns to plot panel) or \code{"plot"} (aligns to full plot including title).
#'   Requires ggplot2 >= 3.5.0 for \code{"plot"} alignment. Ignored when
#'   \code{by} is specified.
#' @param by Optional aesthetic name (character) to position only a specific
#'   legend. When specified, uses per-guide positioning via
#'   \code{guide_legend(position = "top")}. Requires ggplot2 >= 3.5.0.
#'   Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.
#'
#' @return A ggplot2 theme object (when \code{by} is NULL) or a guides
#'   specification (when \code{by} is specified).
#'
#' @examples
#' library(ggplot2)
#'
#' # Basic usage - aligned to panel
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_top()
#'
#' # Aligned to full plot (useful with titles)
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   labs(title = "My Plot Title") +
#'   legend_top(align_to = "plot")
#'
#' # Position only the colour legend on top
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point() +
#'   legend_top(by = "colour") +
#'   legend_right(by = "size")
#'
#' @seealso \code{\link{legend_bottom}}, \code{\link{legend_left}},
#'   \code{\link{legend_right}}, \code{\link{legend_horizontal}}
#' @export
legend_top <- function(align_to = c("panel", "plot"), by = NULL) {
  if (!is.null(by)) {
    return(ggguides_guide_update(
      by = normalize_aesthetic(by),
      guide_params = list(position = "top", direction = "horizontal")
    ))
  }

  align_to <- match.arg(align_to)

  theme_args <- list(
    legend.position = "top",
    legend.justification = "center",
    legend.box.just = "center",
    legend.direction = "horizontal"
  )

  # legend.location requires ggplot2 >= 3.5.0
  if (align_to == "plot") {
    theme_args$legend.location <- "plot"
  }

  do.call(theme, theme_args)
}

#' Place Legend on Bottom with Horizontal Layout
#'
#' A one-liner to position the legend below the plot with horizontal layout.
#' Optionally aligns to the full plot area rather than just the panel.
#'
#' @param align_to Where to align the legend. Either \code{"panel"} (default,
#'   aligns to plot panel) or \code{"plot"} (aligns to full plot including title).
#'   Requires ggplot2 >= 3.5.0 for \code{"plot"} alignment. Ignored when
#'   \code{by} is specified.
#' @param by Optional aesthetic name (character) to position only a specific
#'   legend. When specified, uses per-guide positioning via
#'   \code{guide_legend(position = "bottom")}. Requires ggplot2 >= 3.5.0.
#'   Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.
#'
#' @return A ggplot2 theme object (when \code{by} is NULL) or a guides
#'   specification (when \code{by} is specified).
#'
#' @examples
#' library(ggplot2)
#'
#' # Basic usage
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_bottom()
#'
#' # Aligned to full plot
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   labs(title = "My Plot Title") +
#'   legend_bottom(align_to = "plot")
#'
#' # Position only the colour legend at bottom
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point() +
#'   legend_bottom(by = "colour") +
#'   legend_right(by = "size")
#'
#' @seealso \code{\link{legend_top}}, \code{\link{legend_left}},
#'   \code{\link{legend_right}}, \code{\link{legend_horizontal}}
#' @export
legend_bottom <- function(align_to = c("panel", "plot"), by = NULL) {
  if (!is.null(by)) {
    return(ggguides_guide_update(
      by = normalize_aesthetic(by),
      guide_params = list(position = "bottom", direction = "horizontal")
    ))
  }

  align_to <- match.arg(align_to)

  theme_args <- list(
    legend.position = "bottom",
    legend.justification = "center",
    legend.box.just = "center",
    legend.direction = "horizontal"
  )

  if (align_to == "plot") {
    theme_args$legend.location <- "plot"
  }

  do.call(theme, theme_args)
}

#' Remove Legend from Plot
#'
#' A one-liner to remove the legend from a plot. Cleaner alternative to
#' \code{theme(legend.position = "none")}.
#'
#' @return A ggplot2 theme object that can be added to a plot.
#'
#' @examples
#' library(ggplot2)
#'
#' # Remove legend
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_none()
#'
#' @seealso \code{\link{legend_left}}, \code{\link{legend_right}}
#' @export
legend_none <- function() {
  theme(legend.position = "none")
}

#' Place Legend Inside the Plot Area
#'
#' Position the legend inside the plot panel at specified coordinates or using
#' convenient position shortcuts like \code{"topright"} or \code{"bottomleft"}.
#'
#' @param x Numeric x-coordinate in normalized 0-1 space, where 0 is left
#'   and 1 is right. Ignored if \code{position} is specified.
#' @param y Numeric y-coordinate in normalized 0-1 space, where 0 is bottom
#'   and 1 is top. Ignored if \code{position} is specified.
#' @param position Character shortcut for common positions. One of
#'   \code{"topleft"}, \code{"top"}, \code{"topright"}, \code{"left"},
#'   \code{"center"}, \code{"right"}, \code{"bottomleft"}, \code{"bottom"},
#'   \code{"bottomright"}. If specified, overrides \code{x} and \code{y}.
#' @param justification Justification of legend relative to the anchor point.
#'   Either a character vector of length 2 (horizontal, vertical) or a single
#'   value. Common values: \code{c("left", "top")}, \code{c("right", "bottom")},
#'   \code{"center"}. If \code{NULL}, automatically determined based on position.
#' @param just Deprecated. Use \code{justification} instead.
#' @param background Background fill color for the legend. Default is
#'   \code{"white"}.
#' @param border Border color for the legend box. Default is \code{NA} (no border).
#' @param padding Padding around legend content in cm. Default is \code{0.2}.
#'
#' @return A ggplot2 theme object that can be added to a plot.
#'
#' @examples
#' library(ggplot2)
#'
#' # Using position shortcuts (recommended)
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_inside(position = "topright")
#'
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_inside(position = "bottomleft")
#'
#' # Using coordinates
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_inside(x = 0.95, y = 0.95, justification = c("right", "top"))
#'
#' # Custom background and border
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_inside(position = "topright", background = "grey95", border = "grey50")
#'
#' @seealso \code{\link{legend_left}}, \code{\link{legend_right}},
#'   \code{\link{legend_top}}, \code{\link{legend_bottom}}
#' @export
legend_inside <- function(x = NULL, y = NULL, position = NULL, justification = NULL,
                          background = "white", border = NA, padding = 0.2,
                          just = NULL) {

  if (!is.null(just)) {
    warning("`just` is deprecated; use `justification` instead.", call. = FALSE)
    if (is.null(justification)) justification <- just
  }

  valid_positions <- c(
    "topleft", "top", "topright",
    "left", "center", "right",
    "bottomleft", "bottom", "bottomright"
  )

  if (!is.null(position)) {
    position <- match.arg(position, valid_positions)
    coords <- switch(position,
      "topleft"     = list(x = 0.02, y = 0.98, justification = c("left", "top")),
      "top"         = list(x = 0.50, y = 0.98, justification = c("center", "top")),
      "topright"    = list(x = 0.98, y = 0.98, justification = c("right", "top")),
      "left"        = list(x = 0.02, y = 0.50, justification = c("left", "center")),
      "center"      = list(x = 0.50, y = 0.50, justification = c("center", "center")),
      "right"       = list(x = 0.98, y = 0.50, justification = c("right", "center")),
      "bottomleft"  = list(x = 0.02, y = 0.02, justification = c("left", "bottom")),
      "bottom"      = list(x = 0.50, y = 0.02, justification = c("center", "bottom")),
      "bottomright" = list(x = 0.98, y = 0.02, justification = c("right", "bottom"))
    )
    x <- coords$x
    y <- coords$y
    if (is.null(justification)) {
      justification <- coords$justification
    }
  }

  if (is.null(x) || is.null(y)) {
    stop("Either provide `position` or both `x` and `y` coordinates.", call. = FALSE)
  }

  if (is.null(justification)) {
    justification <- c("left", "top")
  }

  theme(
    legend.position = "inside",
    legend.position.inside = c(x, y),
    legend.justification = justification,
    legend.background = element_rect(fill = background, color = border),
    legend.margin = margin(padding, padding, padding, padding, "cm")
  )
}

# =============================================================================
# Legend Direction Functions
# =============================================================================

#' Set Legend Direction to Horizontal
#'
#' A one-liner to arrange legend keys horizontally. Useful for legends placed
#' at the top or bottom of a plot.
#'
#' @return A ggplot2 theme object that can be added to a plot.
#'
#' @examples
#' library(ggplot2)
#'
#' # Horizontal legend at bottom
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_bottom() +
#'   legend_horizontal()
#'
#' @seealso \code{\link{legend_vertical}}, \code{\link{legend_top}},
#'   \code{\link{legend_bottom}}
#' @export
legend_horizontal <- function() {
  theme(legend.direction = "horizontal")
}

#' Set Legend Direction to Vertical
#'
#' A one-liner to arrange legend keys vertically. This is the default for
#' legends placed on the left or right of a plot.
#'
#' @return A ggplot2 theme object that can be added to a plot.
#'
#' @examples
#' library(ggplot2)
#'
#' # Explicitly set vertical direction
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_right() +
#'   legend_vertical()
#'
#' @seealso \code{\link{legend_horizontal}}, \code{\link{legend_left}},
#'   \code{\link{legend_right}}
#' @export
legend_vertical <- function() {
  theme(legend.direction = "vertical")
}

library(testthat)
library(ggplot2)

test_that("legend_wrap returns a Guides object", {
  result <- legend_wrap(ncol = 2)
  expect_s3_class(result, "Guides")
})

test_that("legend_wrap can be added to a ggplot", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point() +
    legend_wrap(ncol = 2)
  expect_s3_class(p, "gg")
})

test_that("legend_wrap accepts nrow parameter", {
  result <- legend_wrap(nrow = 3)
  expect_s3_class(result, "Guides")
})

test_that("legend_wrap accepts byrow parameter", {
  result <- legend_wrap(ncol = 2, byrow = FALSE)
  expect_s3_class(result, "Guides")
})

test_that("legend_wrap works with fill aesthetic", {
  p <- ggplot(mpg, aes(class, fill = class)) +
    geom_bar() +
    legend_wrap(ncol = 2)
  expect_s3_class(p, "gg")
})

test_that("legend_wrap combines with legend_left", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point() +
    legend_wrap(ncol = 2) +
    legend_left()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "left")
})

library(testthat)
library(ggplot2)

# =============================================================================
# legend_left() tests
# =============================================================================

test_that("legend_left returns a theme object", {
  result <- legend_left()
  expect_s3_class(result, "theme")
})

test_that("legend_left sets correct theme elements", {
  result <- legend_left()
  expect_equal(result$legend.position, "left")
  expect_equal(result$legend.justification, "left")
  expect_equal(result$legend.box.just, "left")
})

test_that("legend_left can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_left()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "left")
})

test_that("legend_left works with multiple legends", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), shape = factor(am))) +
    geom_point() +
    legend_left()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.box.just, "left")
})

# =============================================================================
# legend_right() tests
# =============================================================================

test_that("legend_right returns a theme object", {
  result <- legend_right()
  expect_s3_class(result, "theme")
})

test_that("legend_right sets correct position", {
  result <- legend_right()
  expect_equal(result$legend.position, "right")
  expect_equal(result$legend.justification, "right")
  expect_equal(result$legend.box.just, "right")
})

test_that("legend_right can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_right()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "right")
})

# =============================================================================
# legend_top() tests
# =============================================================================

test_that("legend_top returns a theme object", {
  result <- legend_top()
  expect_s3_class(result, "theme")
})

test_that("legend_top sets correct position and direction", {
  result <- legend_top()
  expect_equal(result$legend.position, "top")
  expect_equal(result$legend.direction, "horizontal")
})

test_that("legend_top align_to='plot' sets legend.location", {
  result <- legend_top(align_to = "plot")
  expect_equal(result$legend.location, "plot")
})

test_that("legend_top can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_top()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "top")
})

# =============================================================================
# legend_bottom() tests
# =============================================================================

test_that("legend_bottom returns a theme object", {
  result <- legend_bottom()
  expect_s3_class(result, "theme")
})

test_that("legend_bottom sets correct position and direction", {
  result <- legend_bottom()
  expect_equal(result$legend.position, "bottom")
  expect_equal(result$legend.direction, "horizontal")
})

test_that("legend_bottom align_to='plot' sets legend.location", {
  result <- legend_bottom(align_to = "plot")
  expect_equal(result$legend.location, "plot")
})

test_that("legend_bottom can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_bottom()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "bottom")
})

# =============================================================================
# legend_none() tests
# =============================================================================

test_that("legend_none returns a theme object", {
  result <- legend_none()
  expect_s3_class(result, "theme")
})

test_that("legend_none sets position to 'none'", {
  result <- legend_none()
  expect_equal(result$legend.position, "none")
})

test_that("legend_none can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_none()
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "none")
})

# =============================================================================
# legend_inside() tests
# =============================================================================

test_that("legend_inside returns a theme object", {
  result <- legend_inside(position = "topright")
  expect_s3_class(result, "theme")
})

test_that("legend_inside works with position shortcuts", {
  positions <- c("topleft", "top", "topright", "left", "center", "right",
                 "bottomleft", "bottom", "bottomright")
  for (pos in positions) {
    result <- legend_inside(position = pos)
    expect_s3_class(result, "theme")
    # ggplot2 3.5.0+ uses legend.position = "inside" with legend.position.inside
    expect_equal(result$legend.position, "inside")
    expect_true(is.numeric(result$legend.position.inside))
    expect_length(result$legend.position.inside, 2)
  }
})

test_that("legend_inside works with coordinates", {
  result <- legend_inside(x = 0.5, y = 0.5)
  expect_equal(result$legend.position, "inside")
  expect_equal(result$legend.position.inside, c(0.5, 0.5))
})

test_that("legend_inside errors without position or coordinates", {
  expect_error(legend_inside(), "Either provide")
})

test_that("legend_inside accepts custom background and border", {
  result <- legend_inside(position = "topright", background = "grey95", border = "black")
  expect_s3_class(result$legend.background, "element_rect")
})

test_that("legend_inside can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_inside(position = "topright")
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_horizontal() tests
# =============================================================================

test_that("legend_horizontal returns a theme object", {
  result <- legend_horizontal()
  expect_s3_class(result, "theme")
})

test_that("legend_horizontal sets direction to horizontal", {
  result <- legend_horizontal()
  expect_equal(result$legend.direction, "horizontal")
})

# =============================================================================
# legend_vertical() tests
# =============================================================================

test_that("legend_vertical returns a theme object", {
  result <- legend_vertical()
  expect_s3_class(result, "theme")
})

test_that("legend_vertical sets direction to vertical", {
  result <- legend_vertical()
  expect_equal(result$legend.direction, "vertical")
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import ggplot2
#' @importFrom grid unit
#' @importFrom gtable gtable_filter gtable_add_grob
#' @importFrom rlang check_installed is_installed %||% eval_tidy
## usethis namespace: end
NULL

#' Collect Legends from Patchwork Compositions
#'
#' Collects legends from multiple plots in a patchwork composition into a single
#' legend area. This is a convenience wrapper around patchwork's guide
#' collection functionality.
#'
#' @param x A patchwork object created by combining ggplot2 plots with
#'   operators like \code{|}, \code{/}, or \code{+}.
#' @param position Where to place the collected legends. One of \code{"right"}
#'   (default), \code{"left"}, \code{"bottom"}, or \code{"top"}.
#' @param span Logical or integer vector. If \code{TRUE}, the legend will span
#'   the full height (for left/right) or width (for top/bottom). If an integer
#'   vector (e.g., \code{c(1, 2)} or \code{1:2}), the legend spans only those
#'   rows/columns. Default is \code{FALSE}. When not \code{FALSE}, returns a
#'   gtable instead of a patchwork object.
#'
#' @return A patchwork object with legends collected, or a gtable if
#'   \code{span} is not \code{FALSE}.
#'
#' @details
#' This function requires the patchwork package. If patchwork is not installed,
#' an informative error is raised.
#'
#' When \code{span = TRUE}, the function converts the patchwork to a gtable
#' and modifies the legend cell to span all rows (for right/left positions) or
#' all columns (for top/bottom positions). This addresses the common issue where
#' collected legends are centered rather than spanning the full composition.
#'
#' When \code{span} is an integer vector (e.g., \code{c(1, 2)}), the legend
#' is centered between those specific rows (for right/left) or columns (for
#' top/bottom). This is useful for attaching a legend to specific plots in a
#' stacked layout. Row/column numbers refer to the panel positions in the
#' patchwork (1-indexed).
#'
#' For cowplot users: cowplot does not have a built-in legend collection
#' mechanism. Consider using the lemon package for manual legend extraction
#' and positioning.
#'
#' @examples
#' library(ggplot2)
#'
#' if (requireNamespace("patchwork", quietly = TRUE)) {
#'   library(patchwork)
#'
#'   p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'     geom_point()
#'   p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
#'     geom_point()
#'
#'   # Without collection: each plot has its own legend
#'   p1 | p2
#'
#'   # With collection: single shared legend
#'   collect_legends(p1 | p2)
#'
#'   # Place collected legend on the bottom
#'   collect_legends(p1 | p2, position = "bottom")
#'
#'   # Legend spanning full height (returns gtable, draw with grid.draw)
#'   gt <- collect_legends(p1 / p2, position = "right", span = TRUE)
#'   grid::grid.draw(gt)
#'
#'   # Attach legend to specific rows (e.g., align with row 1 only)
#'   p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
#'     geom_point() + labs(title = "Plot 3")
#'   gt <- collect_legends(p1 / p2 / p3, position = "right", span = 1)
#'   grid::grid.draw(gt)
#'
#'   # Attach legend to rows 1 and 2
#'   gt <- collect_legends(p1 / p2 / p3, position = "right", span = 1:2)
#'   grid::grid.draw(gt)
#' }
#'
#' @seealso \code{\link{collect_axes}}, \code{\link{legend_left}}, \code{\link{legend_wrap}}
#' @export
collect_legends <- function(x, position = c("right", "left", "bottom", "top"),
                            span = FALSE) {
  check_installed("patchwork",
    reason = "to collect legends from plot compositions."
  )

  position <- match.arg(position)

  if (!inherits(x, "patchwork")) {
    stop("`x` must be a patchwork object. Create one by combining ggplot objects with |, /, or +.",
         call. = FALSE)
  }

  # Apply guide collection
  pw <- x + patchwork::plot_layout(guides = "collect") &
    theme(legend.position = position)

  # Check if span is FALSE (return patchwork) or something else (return gtable)
  if (identical(span, FALSE)) {
    return(pw)
  }

  # For spanning, convert to gtable and modify legend position
  gt <- patchwork::patchworkGrob(pw)

  # Find the guide-box in the layout
  guide_idx <- which(gt$layout$name == "guide-box")

  if (length(guide_idx) == 0) {
    warning("No legend found in the patchwork. Returning gtable without modification.",
            call. = FALSE)
    return(gt)
  }

  # Find panel positions to map row/column indices
  # We need both top (t) and bottom (b) positions for rows,

  # and left (l) and right (r) positions for columns
  panel_idx <- grep("^panel", gt$layout$name)
  panel_layout <- gt$layout[panel_idx, ]

  # Get unique panel row positions (top and bottom of each panel row)
  panel_rows_t <- sort(unique(panel_layout$t))
  panel_rows_b <- sort(unique(panel_layout$b))

  # Get unique panel column positions (left and right of each panel column)
  panel_cols_l <- sort(unique(panel_layout$l))
  panel_cols_r <- sort(unique(panel_layout$r))

  if (position %in% c("right", "left")) {
    if (isTRUE(span)) {
      # Span all rows
      gt$layout$t[guide_idx] <- 1
      gt$layout$b[guide_idx] <- nrow(gt)
    } else if (is.numeric(span)) {
      # Span specific rows - center legend at visual boundary between plots
      span <- as.integer(span)
      if (any(span < 1) || any(span > length(panel_rows_t))) {
        stop("span indices must be between 1 and ", length(panel_rows_t),
             " (number of panel rows).", call. = FALSE)
      }
      # Set the layout to span from top of first panel to bottom of last panel
      first_panel_t <- panel_rows_t[min(span)]
      last_panel_b <- panel_rows_b[max(span)]
      gt$layout$t[guide_idx] <- first_panel_t
      gt$layout$b[guide_idx] <- last_panel_b

      # Find background elements to determine visual boundary between plots
      # The boundary is where background-N ends and background-(N+1) begins
      bg_idx <- grep("^background-[0-9]+$", gt$layout$name)

      if (length(bg_idx) >= 2 && length(span) > 1) {
        # Sort backgrounds by their top position
        bg_layout <- gt$layout[bg_idx, ]
        bg_layout <- bg_layout[order(bg_layout$t), ]

        # Find the boundary between the last plot in span and any plots after it
        # or between middle plots in the span
        # For span = 1:2, find boundary between background-1 and background-2
        min_span <- min(span)
        max_span <- max(span)

        # Get heights of all rows to calculate positions
        all_heights_abs <- grid::convertHeight(gt$heights, "mm", valueOnly = TRUE)
        cum_heights <- c(0, cumsum(all_heights_abs))

        # Find boundary: bottom of background for plot min_span, top of background for plot max_span
        # (when span covers multiple plots, center between first and last)
        if (min_span <= nrow(bg_layout) && max_span <= nrow(bg_layout)) {
          bg_first_b <- bg_layout$b[min_span]
          bg_last_t <- bg_layout$t[max_span]

          # Boundary is midpoint between bottom of first plot's background
          # and top of last plot's background
          pos_first_bg_bottom <- cum_heights[bg_first_b + 1]
          pos_last_bg_top <- cum_heights[bg_last_t]

          # Center at the visual boundary
          boundary_from_top <- (pos_first_bg_bottom + pos_last_bg_top) / 2

          # Calculate relative to spanned region
          spanned_top <- cum_heights[first_panel_t]
          spanned_bottom <- cum_heights[last_panel_b + 1]
          spanned_height <- spanned_bottom - spanned_top

          boundary_in_span <- boundary_from_top - spanned_top
          center_y_npc <- 1 - (boundary_in_span / spanned_height)

          # Wrap the guide grob in a viewport centered at the boundary
          guide_grob <- gt$grobs[[guide_idx]]
          gt$grobs[[guide_idx]] <- grid::gTree(
            children = grid::gList(guide_grob),
            vp = grid::viewport(y = center_y_npc, just = "center")
          )
        }
      } else {
        # Single plot or no background elements found - center at 0.5
        guide_grob <- gt$grobs[[guide_idx]]
        gt$grobs[[guide_idx]] <- grid::gTree(
          children = grid::gList(guide_grob),
          vp = grid::viewport(y = 0.5, just = "center")
        )
      }
    }
  } else {
    if (isTRUE(span)) {
      # Span all columns
      gt$layout$l[guide_idx] <- 1
      gt$layout$r[guide_idx] <- ncol(gt)
    } else if (is.numeric(span)) {
      # Span specific columns - center legend at visual boundary between plots
      span <- as.integer(span)
      if (any(span < 1) || any(span > length(panel_cols_l))) {
        stop("span indices must be between 1 and ", length(panel_cols_l),
             " (number of panel columns).", call. = FALSE)
      }
      # Set the layout to span from left of first panel to right of last panel
      first_panel_l <- panel_cols_l[min(span)]
      last_panel_r <- panel_cols_r[max(span)]
      gt$layout$l[guide_idx] <- first_panel_l
      gt$layout$r[guide_idx] <- last_panel_r

      # Find background elements to determine visual boundary between plots
      bg_idx <- grep("^background-[0-9]+$", gt$layout$name)

      if (length(bg_idx) >= 2 && length(span) > 1) {
        # Sort backgrounds by their left position
        bg_layout <- gt$layout[bg_idx, ]
        bg_layout <- bg_layout[order(bg_layout$l), ]

        min_span <- min(span)
        max_span <- max(span)

        # Get widths of all columns to calculate positions
        all_widths_abs <- grid::convertWidth(gt$widths, "mm", valueOnly = TRUE)
        cum_widths <- c(0, cumsum(all_widths_abs))

        # Find boundary between backgrounds
        if (min_span <= nrow(bg_layout) && max_span <= nrow(bg_layout)) {
          bg_first_r <- bg_layout$r[min_span]
          bg_last_l <- bg_layout$l[max_span]

          # Boundary is midpoint between right of first plot's background
          # and left of last plot's background
          pos_first_bg_right <- cum_widths[bg_first_r + 1]
          pos_last_bg_left <- cum_widths[bg_last_l]

          # Center at the visual boundary
          boundary_from_left <- (pos_first_bg_right + pos_last_bg_left) / 2

          # Calculate relative to spanned region
          spanned_left <- cum_widths[first_panel_l]
          spanned_right <- cum_widths[last_panel_r + 1]
          spanned_width <- spanned_right - spanned_left

          boundary_in_span <- boundary_from_left - spanned_left
          center_x_npc <- boundary_in_span / spanned_width

          # Wrap the guide grob in a viewport centered at the boundary
          guide_grob <- gt$grobs[[guide_idx]]
          gt$grobs[[guide_idx]] <- grid::gTree(
            children = grid::gList(guide_grob),
            vp = grid::viewport(x = center_x_npc, just = "center")
          )
        }
      } else {
        # Single plot or no background elements found - center at 0.5
        guide_grob <- gt$grobs[[guide_idx]]
        gt$grobs[[guide_idx]] <- grid::gTree(
          children = grid::gList(guide_grob),
          vp = grid::viewport(x = 0.5, just = "center")
        )
      }
    }
  }

  gt
}

#' Extract Legend from a ggplot
#'
#' Extracts the legend (guide-box) from a ggplot object as a grob that can be
#' used independently with grid or cowplot.
#'
#' @param plot A ggplot object.
#'
#' @return A grob containing the legend, or \code{NULL} if the plot has no
#'   legend.
#'
#' @details
#' This function is useful for cowplot workflows where you want to manually
#' position a shared legend. The extracted legend can be combined with plots
#' using \code{cowplot::plot_grid()} or drawn directly with
#' \code{grid::grid.draw()}.
#'
#' @examples
#' library(ggplot2)
#'
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   labs(color = "Cylinders")
#'
#' # Extract the legend
#' leg <- get_legend(p)
#'
#' # Draw just the legend
#' grid::grid.newpage()
#' grid::grid.draw(leg)
#'
#' @seealso \code{\link{shared_legend}}, \code{\link{collect_legends}}
#' @export
get_legend <- function(plot) {
  if (!inherits(plot, "ggplot"))
    stop("`plot` must be a ggplot object.", call. = FALSE)

  gt <- ggplot2::ggplotGrob(plot)


  # ggplot2 3.5.0+ uses position-specific names like "guide-box-right"
  guide_idx <- which(grepl("^guide-box", gt$layout$name))

  if (length(guide_idx) == 0) {
    return(NULL)
  }

  # Return the first non-empty guide-box
  for (idx in guide_idx) {
    grob <- gt$grobs[[idx]]
    # Check if it's a non-empty gtable (has actual content)
    if (inherits(grob, "gtable") && length(grob$grobs) > 0) {
      return(grob)
    }
  }

  # Fallback: return first guide-box even if potentially empty
gt$grobs[[guide_idx[1]]]
}


#' Combine Plots with a Shared Legend
#'
#' Combines multiple ggplot objects into a grid layout with a single shared
#' legend. Works with base ggplot and cowplot workflows (no patchwork required).
#'
#' @param ... ggplot objects to combine.
#' @param ncol Number of columns in the plot grid.
#' @param nrow Number of rows in the plot grid. If NULL (default), computed
#'   from ncol and number of plots.
#' @param position Where to place the shared legend. One of \code{"right"}
#'   (default), \code{"left"}, \code{"bottom"}, or \code{"top"}.
#' @param legend_from Which plot to extract the legend from. Default is 1
#'   (first plot). Can be an integer index or a ggplot object.
#' @param rel_legend_size Relative size of the legend compared to the plot
#'   area. Default is 0.2 (20 percent).
#'
#' @return A gtable that can be drawn with \code{grid::grid.draw()} or used
#'   with \code{cowplot::ggdraw()}.
#'
#' @details
#' This function provides a simple way to create multi-panel plots with a
#' shared legend without requiring patchwork. It:
#' \enumerate{
#'   \item Removes legends from all plots
#'   \item Extracts the legend from the specified plot
#'   \item Arranges plots in a grid
#'   \item Attaches the legend to the specified position
#' }
#'
#' For more complex layouts or patchwork users, see \code{\link{collect_legends}}.
#'
#' @examples
#' library(ggplot2)
#'
#' p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() + labs(title = "Plot 1", color = "Cylinders")
#' p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
#'   geom_point() + labs(title = "Plot 2", color = "Cylinders")
#' p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
#'   geom_point() + labs(title = "Plot 3", color = "Cylinders")
#'
#' # Side-by-side with shared legend on right
#' gt <- shared_legend(p1, p2, ncol = 2, position = "right")
#' grid::grid.newpage()
#' grid::grid.draw(gt)
#'
#' # 2x2 grid with legend at bottom
#' gt <- shared_legend(p1, p2, p3, p1, ncol = 2, nrow = 2, position = "bottom")
#' grid::grid.newpage()
#' grid::grid.draw(gt)
#'
#' # Stacked with legend on left
#' gt <- shared_legend(p1, p2, p3, ncol = 1, position = "left")
#' grid::grid.newpage()
#' grid::grid.draw(gt)
#'
#' @seealso \code{\link{get_legend}}, \code{\link{collect_legends}}
#' @export
shared_legend <- function(..., ncol = NULL, nrow = NULL,
                          position = c("right", "left", "bottom", "top"),
                          legend_from = 1,
                          rel_legend_size = 0.2) {
  plots <- list(...)
  position <- match.arg(position)

  if (length(plots) == 0)
    stop("At least one plot must be provided.", call. = FALSE)

  # Validate all are ggplots

  for (i in seq_along(plots)) {
    if (!inherits(plots[[i]], "ggplot"))
      stop("All arguments must be ggplot objects. Argument ", i,
           " is not a ggplot.", call. = FALSE)
  }

  # Determine grid dimensions
  n <- length(plots)
  if (is.null(ncol) && is.null(nrow)) {
    ncol <- n
    nrow <- 1
  } else if (is.null(ncol)) {
    ncol <- ceiling(n / nrow)
  } else if (is.null(nrow)) {
    nrow <- ceiling(n / ncol)
  }

  # Get the legend source plot
  if (inherits(legend_from, "ggplot")) {
    legend_plot <- legend_from
  } else {
    legend_from <- as.integer(legend_from)
    if (legend_from < 1 || legend_from > n)
      stop("`legend_from` must be between 1 and ", n, ".", call. = FALSE)
    legend_plot <- plots[[legend_from]]
  }

  # Adjust legend orientation based on position
  if (position %in% c("bottom", "top")) {
    legend_plot <- legend_plot +
      ggplot2::theme(legend.direction = "horizontal")
  }

  # Extract legend
  legend <- get_legend(legend_plot)

  if (is.null(legend)) {
    warning("No legend found in the source plot. Returning plots without shared legend.",
            call. = FALSE)
    # Still arrange plots without legend
    plots_no_legend <- lapply(plots, function(p) {
      p + ggplot2::theme(legend.position = "none")
    })
    return(do.call(gridExtra::arrangeGrob,
                   c(plots_no_legend, list(ncol = ncol, nrow = nrow))))
  }

  # Remove legends from all plots
  plots_no_legend <- lapply(plots, function(p) {
    p + ggplot2::theme(legend.position = "none")
  })

  # Arrange plots
  plot_grid <- do.call(gridExtra::arrangeGrob,
                       c(plots_no_legend, list(ncol = ncol, nrow = nrow)))

  # Calculate sizes
  legend_size <- rel_legend_size
  plot_size <- 1 - legend_size

  # Combine plot grid with legend
  if (position == "right") {
    combined <- gridExtra::arrangeGrob(
      plot_grid, legend,
      ncol = 2,
      widths = grid::unit(c(plot_size, legend_size), "npc")
    )
  } else if (position == "left") {
    combined <- gridExtra::arrangeGrob(
      legend, plot_grid,
      ncol = 2,
      widths = grid::unit(c(legend_size, plot_size), "npc")
    )
  } else if (position == "bottom") {
    combined <- gridExtra::arrangeGrob(
      plot_grid, legend,
      nrow = 2,
      heights = grid::unit(c(plot_size, legend_size), "npc")
    )
  } else { # top
    combined <- gridExtra::arrangeGrob(
      legend, plot_grid,
      nrow = 2,
      heights = grid::unit(c(legend_size, plot_size), "npc")
    )
  }

  combined
}

library(testthat)
library(ggplot2)

test_that("collect_legends errors without patchwork installed", {
  skip_if(rlang::is_installed("patchwork"))
  expect_error(
    collect_legends(list()),
    "patchwork"
  )
})

test_that("collect_legends errors if x is not a patchwork object", {
  skip_if_not_installed("patchwork")
  expect_error(
    collect_legends(ggplot()),
    "`x` must be a patchwork object."
  )
})

test_that("collect_legends errors if x is a list", {
  skip_if_not_installed("patchwork")
  expect_error(
    collect_legends(list(a = 1)),
    "`x` must be a patchwork object."
  )
})

test_that("collect_legends works with valid patchwork object", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2

  result <- collect_legends(combined)
  expect_s3_class(result, "patchwork")
})

test_that("collect_legends accepts position parameter", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2

  result_bottom <- collect_legends(combined, position = "bottom")
  expect_s3_class(result_bottom, "patchwork")

  result_left <- collect_legends(combined, position = "left")
  expect_s3_class(result_left, "patchwork")
})

test_that("collect_legends rejects invalid position", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2

  expect_error(
    collect_legends(combined, position = "invalid"),
    "'arg' should be one of"
  )
})

test_that("collect_legends with span = TRUE returns gtable", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  result <- collect_legends(combined, span = TRUE)
  expect_s3_class(result, "gtable")
})

test_that("collect_legends with numeric span returns gtable", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2 / p3

  # Span single row
  result <- collect_legends(combined, span = 1)
  expect_s3_class(result, "gtable")

  # Span multiple rows
  result <- collect_legends(combined, span = 1:2)
  expect_s3_class(result, "gtable")
})

test_that("collect_legends errors on invalid span indices", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  expect_error(
    collect_legends(combined, span = 5),
    "span indices must be between"
  )
})

# =============================================================================
# get_legend() tests
# =============================================================================

test_that("get_legend returns a gtable for plot with legend", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    labs(color = "Cylinders")

  result <- get_legend(p)
  expect_s3_class(result, "gtable")
})

test_that("get_legend returns NULL or empty grob for plot without legend", {
  p <- ggplot(mtcars, aes(mpg, wt)) +
    geom_point()

  result <- get_legend(p)
  # May return NULL or zeroGrob depending on ggplot2 version
  if (!is.null(result)) {
    expect_true(inherits(result, "zeroGrob") || length(result$grobs) == 0)
  }
})

test_that("get_legend returns NULL for plot with legend.position = 'none'", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    theme(legend.position = "none")

  result <- get_legend(p)
  # Should return NULL or empty gtable
  if (!is.null(result)) {
    expect_true(length(result$grobs) == 0 || inherits(result, "zeroGrob"))
  }
})

test_that("get_legend errors if not a ggplot", {
  expect_error(get_legend("not a plot"), "must be a ggplot object")
  expect_error(get_legend(list()), "must be a ggplot object")
})

test_that("get_legend works with fill aesthetic", {
  p <- ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
    geom_bar() +
    labs(fill = "Cylinders")

  result <- get_legend(p)
  expect_s3_class(result, "gtable")
})

test_that("get_legend works with legend on different positions", {
  p_base <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  for (pos in c("right", "left", "top", "bottom")) {
    p <- p_base + theme(legend.position = pos)
    result <- get_legend(p)
    expect_s3_class(result, "gtable")
  }
})

# =============================================================================
# shared_legend() tests
# =============================================================================

test_that("shared_legend returns a gtable", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() + labs(title = "Plot 1")
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
    geom_point() + labs(title = "Plot 2")

  result <- shared_legend(p1, p2, ncol = 2)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend errors with no plots", {
  expect_error(shared_legend(), "At least one plot must be provided")
})

test_that("shared_legend errors if not all ggplots", {
  p1 <- ggplot(mtcars, aes(mpg, wt)) + geom_point()

  expect_error(
    shared_legend(p1, "not a plot"),
    "All arguments must be ggplot objects"
  )
})

test_that("shared_legend accepts all positions", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()

  for (pos in c("right", "left", "top", "bottom")) {
    result <- shared_legend(p1, p2, ncol = 2, position = pos)
    expect_s3_class(result, "gtable")
  }
})

test_that("shared_legend accepts ncol parameter", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1, p2, p3, ncol = 3)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend accepts nrow parameter", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1, p2, p3, nrow = 3)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend accepts both ncol and nrow", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()
  p4 <- ggplot(mtcars, aes(wt, hp, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1, p2, p3, p4, ncol = 2, nrow = 2)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend computes dimensions when only ncol given", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1, p2, p3, ncol = 2)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend computes dimensions when only nrow given", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1, p2, p3, nrow = 2)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend accepts legend_from as integer", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(gear))) + geom_point()

  # Get legend from second plot
  result <- shared_legend(p1, p2, ncol = 2, legend_from = 2)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend accepts legend_from as ggplot object", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p_legend <- ggplot(mtcars, aes(mpg, wt, color = factor(gear))) + geom_point()

  result <- shared_legend(p1, p2, ncol = 2, legend_from = p_legend)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend errors on invalid legend_from index", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()

  expect_error(
    shared_legend(p1, p2, ncol = 2, legend_from = 5),
    "must be between 1 and"
  )
})

test_that("shared_legend accepts rel_legend_size parameter", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1, p2, ncol = 2, rel_legend_size = 0.3)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend handles plots without legend", {
  p1 <- ggplot(mtcars, aes(mpg, wt)) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp)) + geom_point()

  # Should either warn or return gtable without legend
  result <- suppressWarnings(shared_legend(p1, p2, ncol = 2))
  expect_s3_class(result, "gtable")
})

test_that("shared_legend sets horizontal direction for bottom/top positions", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()

  # These should work without error (horizontal legend direction is set internally)
  result_bottom <- shared_legend(p1, p2, ncol = 2, position = "bottom")
  expect_s3_class(result_bottom, "gtable")

  result_top <- shared_legend(p1, p2, ncol = 2, position = "top")
  expect_s3_class(result_top, "gtable")
})

# =============================================================================
# collect_legends() span tests for top/bottom positions
# =============================================================================

test_that("collect_legends with span works for bottom position", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2

  result <- collect_legends(combined, position = "bottom", span = TRUE)
  expect_s3_class(result, "gtable")
})

test_that("collect_legends with span works for top position", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2

  result <- collect_legends(combined, position = "top", span = TRUE)
  expect_s3_class(result, "gtable")
})

test_that("collect_legends with numeric span works for bottom position", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2 | p3

  # Span single column
  result <- collect_legends(combined, position = "bottom", span = 1)
  expect_s3_class(result, "gtable")

  # Span multiple columns
  result <- collect_legends(combined, position = "bottom", span = 1:2)
  expect_s3_class(result, "gtable")
})

test_that("collect_legends with numeric span works for top position", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2 | p3

  result <- collect_legends(combined, position = "top", span = 1:2)
  expect_s3_class(result, "gtable")
})

test_that("collect_legends with span errors on invalid column indices", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 | p2

  expect_error(
    collect_legends(combined, position = "bottom", span = 5),
    "span indices must be between"
  )
})

test_that("collect_legends warns when no legend found", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt)) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp)) + geom_point()
  combined <- p1 | p2

  expect_warning(
    collect_legends(combined, span = TRUE),
    "No legend found"
  )
})

test_that("collect_legends with span = TRUE works for left position", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  result <- collect_legends(combined, position = "left", span = TRUE)
  expect_s3_class(result, "gtable")
})

# =============================================================================
# shared_legend edge cases
# =============================================================================

test_that("shared_legend with neither ncol nor nrow defaults to single row", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) + geom_point()

  # When neither ncol nor nrow specified, should default to n columns, 1 row
  result <- shared_legend(p1, p2, p3)
  expect_s3_class(result, "gtable")
})

test_that("shared_legend handles source plot without legend", {
  p1 <- ggplot(mtcars, aes(mpg, wt)) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp)) + geom_point()

  # May or may not warn depending on get_legend behavior
  result <- suppressWarnings(shared_legend(p1, p2, ncol = 2))
  expect_s3_class(result, "gtable")
})

test_that("shared_legend with single plot works", {
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()

  result <- shared_legend(p1)
  expect_s3_class(result, "gtable")
})

# =============================================================================
# Multiple Legend Functions
# =============================================================================

#' Normalize Aesthetic Names
#'
#' Internal helper to normalize aesthetic names (e.g., "color" to "colour").
#'
#' @param x Character string of aesthetic name.
#' @return Normalized aesthetic name.
#' @noRd
normalize_aesthetic <- function(x) {
  x <- tolower(x)
  if (x == "color") x <- "colour"
  x
}

#' Hide Specific Legends
#'
#' Remove specific legends from a plot while keeping others. This is more
#' targeted than \code{legend_none()} which removes all legends.
#'
#' @param ... Aesthetic names (unquoted) to hide. Common values: \code{colour},
#'   \code{fill}, \code{size}, \code{shape}, \code{linetype}, \code{alpha}.
#'   Note: \code{color} is automatically converted to \code{colour}.
#'
#' @return A guides specification that can be added to a plot.
#'
#' @examples
#' library(ggplot2)
#'
#' # Plot with multiple legends
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point()
#'
#' # Hide the size legend
#' p + legend_hide(size)
#'
#' # Hide multiple legends
#' p + legend_hide(size, colour)
#'
#' @seealso \code{\link{legend_select}}, \code{\link{legend_none}}
#' @export
legend_hide <- function(...) {
  aesthetics <- as.character(substitute(list(...)))[-1]

  if (length(aesthetics) == 0) {
    stop("At least one aesthetic must be specified.", call. = FALSE)
  }

  aesthetics <- vapply(aesthetics, normalize_aesthetic, character(1),
                       USE.NAMES = FALSE)
  guides_list <- rep(list("none"), length(aesthetics))
  names(guides_list) <- aesthetics
  do.call(ggplot2::guides, guides_list)
}

#' Keep Only Specific Legends
#'
#' Show only the specified legends and hide all others. This is the inverse of
#' \code{\link{legend_hide}}.
#'
#' @param ... Aesthetic names (unquoted) to keep. All other legends will be
#'   hidden. Common values: \code{colour}, \code{fill}, \code{size},
#'   \code{shape}, \code{linetype}, \code{alpha}.
#'
#' @return A guides specification that can be added to a plot, or \code{NULL}
#'   if nothing needs to be hidden.
#'
#' @examples
#' library(ggplot2)
#'
#' # Plot with multiple legends
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp,
#'                          shape = factor(am))) +
#'   geom_point()
#'
#' # Keep only the colour legend
#' p + legend_select(colour)
#'
#' # Keep colour and shape, hide size
#' p + legend_select(colour, shape)
#'
#' @seealso \code{\link{legend_hide}}, \code{\link{legend_none}}
#' @export
legend_select <- function(...) {
  keep <- as.character(substitute(list(...)))[-1]

  if (length(keep) == 0) {
    stop("At least one aesthetic must be specified.", call. = FALSE)
  }

  keep <- vapply(keep, normalize_aesthetic, character(1), USE.NAMES = FALSE)
  all_aes <- c("colour", "fill", "size", "shape", "linetype", "alpha")
  hide <- setdiff(all_aes, keep)

  if (length(hide) == 0) {
    return(NULL)
  }

  guides_list <- rep(list("none"), length(hide))
  names(guides_list) <- hide
  do.call(ggplot2::guides, guides_list)
}

#' Control Legend Display Order
#'
#' Set the display order of multiple legends. Legends with lower order values
#' appear first (top or left).
#'
#' @param ... Named arguments where names are aesthetic names and values are
#'   integer order positions. E.g., \code{colour = 1, size = 2}.
#'
#' @return A guides specification that can be added to a plot.
#'
#' @details
#' The order value determines the position of the legend relative to others.
#' Lower values appear first. By default, all legends have order = 0 and appear
#' in an unspecified order.
#'
#' @examples
#' library(ggplot2)
#'
#' # Plot with multiple legends
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point()
#'
#' # Default order
#' p
#'
#' # Size legend first, then colour
#' p + legend_order_guides(size = 1, colour = 2)
#'
#' # Colour legend first
#' p + legend_order_guides(colour = 1, size = 2)
#'
#' @seealso \code{\link{legend_merge}}, \code{\link{legend_split}}
#' @export
legend_order_guides <- function(...) {
  args <- list(...)

  if (length(args) == 0) {
    stop("At least one aesthetic-order pair must be specified.", call. = FALSE)
  }

  if (is.null(names(args)) || any(names(args) == "")) {
    stop("All arguments must be named (aesthetic = order).", call. = FALSE)
  }

  aes_names <- vapply(names(args), normalize_aesthetic, character(1),
                      USE.NAMES = FALSE)
  guides_list <- lapply(args, function(ord) {
    ggplot2::guide_legend(order = as.integer(ord))
  })
  names(guides_list) <- aes_names
  do.call(ggplot2::guides, guides_list)
}

#' Force Legends to Merge
#'
#' Force specified legends to merge together by setting them to the same order.
#' Legends will only merge if they have matching labels (same factor levels or
#' break values).
#'
#' @param ... Aesthetic names (unquoted) to merge. E.g., \code{colour, fill}.
#'
#' @return A guides specification that can be added to a plot.
#'
#' @details
#' ggplot2 automatically merges legends when they have the same title and
#' matching labels. This function ensures legends have the same order value
#' (order = 0), which is a prerequisite for merging.
#'
#' If legends still don't merge after using this function, ensure:
#' \itemize{
#'   \item Both aesthetics map to the same variable

#'   \item The legends have identical titles (use \code{labs()})
#'   \item The breaks/labels are identical
#' }
#'
#' @examples
#' library(ggplot2)
#'
#' # Plot where colour and fill map to the same variable
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
#'   geom_point(shape = 21, size = 3, stroke = 1.5) +
#'   labs(color = "Cylinders", fill = "Cylinders")
#'
#' # Force merge (they should merge automatically if labels match)
#' p + legend_merge(colour, fill)
#'
#' @seealso \code{\link{legend_split}}, \code{\link{legend_order_guides}}
#' @export
legend_merge <- function(...) {
  aesthetics <- as.character(substitute(list(...)))[-1]

  if (length(aesthetics) < 2) {
    stop("At least two aesthetics must be specified for merging.", call. = FALSE)
  }

  aesthetics <- vapply(aesthetics, normalize_aesthetic, character(1),
                       USE.NAMES = FALSE)
  guides_list <- rep(list(ggplot2::guide_legend(order = 0)), length(aesthetics))
  names(guides_list) <- aesthetics
  do.call(ggplot2::guides, guides_list)
}

#' Force Legends to Stay Separate
#'
#' Force specified legends to remain separate by assigning different order
#' values, preventing automatic merging.
#'
#' @param ... Aesthetic names (unquoted) to keep separate.
#'   E.g., \code{colour, fill}.
#'
#' @return A guides specification that can be added to a plot.
#'
#' @details
#' By default, ggplot2 merges legends that have matching titles and labels.
#' This function assigns different order values to each legend, which prevents
#' automatic merging.
#'
#' @examples
#' library(ggplot2)
#'
#' # Plot where colour and fill would normally merge
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
#'   geom_point(shape = 21, size = 3, stroke = 1.5) +
#'   labs(color = "Cylinders", fill = "Cylinders")
#'
#' # Force separate legends
#' p + legend_split(colour, fill)
#'
#' @seealso \code{\link{legend_merge}}, \code{\link{legend_order_guides}}
#' @export
legend_split <- function(...) {
  aesthetics <- as.character(substitute(list(...)))[-1]

  if (length(aesthetics) < 2) {
    stop("At least two aesthetics must be specified for splitting.", call. = FALSE)
  }

  aesthetics <- vapply(aesthetics, normalize_aesthetic, character(1),
                       USE.NAMES = FALSE)
  guides_list <- lapply(seq_along(aesthetics), function(i) {
    ggplot2::guide_legend(order = i)
  })
  names(guides_list) <- aesthetics
  do.call(ggplot2::guides, guides_list)
}

library(testthat)
library(ggplot2)

# =============================================================================
# legend_hide() tests
# =============================================================================

test_that("legend_hide returns a Guides object", {
  result <- legend_hide(colour)
  expect_s3_class(result, "Guides")
})

test_that("legend_hide errors with no arguments", {
  expect_error(legend_hide(), "At least one aesthetic")
})

test_that("legend_hide hides single legend", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_hide(size)
  expect_s3_class(p, "gg")
})

test_that("legend_hide hides multiple legends", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_hide(size, colour)
  expect_s3_class(p, "gg")
})

test_that("legend_hide normalizes color to colour", {
  # Should work without error
  result <- legend_hide(color)
  expect_s3_class(result, "Guides")
})

# =============================================================================
# legend_select() tests
# =============================================================================

test_that("legend_select returns a Guides object", {
  result <- legend_select(colour)
  expect_s3_class(result, "Guides")
})

test_that("legend_select errors with no arguments", {
  expect_error(legend_select(), "At least one aesthetic")
})

test_that("legend_select keeps single legend", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_select(colour)
  expect_s3_class(p, "gg")
})

test_that("legend_select keeps multiple legends", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp,
                          shape = factor(am))) +
    geom_point() +
    legend_select(colour, shape)
  expect_s3_class(p, "gg")
})

test_that("legend_select returns NULL when all aesthetics kept", {
  result <- legend_select(colour, fill, size, shape, linetype, alpha)
  expect_null(result)
})

# =============================================================================
# legend_order_guides() tests
# =============================================================================

test_that("legend_order_guides returns a Guides object", {
  result <- legend_order_guides(colour = 1, size = 2)
  expect_s3_class(result, "Guides")
})

test_that("legend_order_guides errors with no arguments", {
  expect_error(legend_order_guides(), "At least one aesthetic-order pair")
})

test_that("legend_order_guides errors with unnamed arguments", {
  expect_error(legend_order_guides(1, 2), "must be named")
})

test_that("legend_order_guides sets legend order", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_order_guides(size = 1, colour = 2)
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_merge() tests
# =============================================================================

test_that("legend_merge returns a Guides object", {
  result <- legend_merge(colour, fill)
  expect_s3_class(result, "Guides")
})

test_that("legend_merge errors with single aesthetic", {
  expect_error(legend_merge(colour), "At least two aesthetics")
})

test_that("legend_merge can be added to a plot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
    geom_point(shape = 21, size = 3) +
    labs(color = "Cylinders", fill = "Cylinders") +
    legend_merge(colour, fill)
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_split() tests
# =============================================================================

test_that("legend_split returns a Guides object", {
  result <- legend_split(colour, fill)
  expect_s3_class(result, "Guides")
})

test_that("legend_split errors with single aesthetic", {
  expect_error(legend_split(colour), "At least two aesthetics")
})

test_that("legend_split can be added to a plot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
    geom_point(shape = 21, size = 3) +
    labs(color = "Cylinders", fill = "Cylinders") +
    legend_split(colour, fill)
  expect_s3_class(p, "gg")
})

# =============================================================================
# normalize_aesthetic() helper tests
# =============================================================================

test_that("normalize_aesthetic converts color to colour", {
  expect_equal(ggguides:::normalize_aesthetic("color"), "colour")
  expect_equal(ggguides:::normalize_aesthetic("COLOR"), "colour")
})

test_that("normalize_aesthetic passes through other aesthetics", {
  expect_equal(ggguides:::normalize_aesthetic("fill"), "fill")
  expect_equal(ggguides:::normalize_aesthetic("size"), "size")
})

# =============================================================================
# Position functions with by parameter
# =============================================================================

test_that("legend_left with by returns a per-aesthetic guide update", {
  result <- legend_left(by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_right with by returns a per-aesthetic guide update", {
  result <- legend_right(by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_top with by returns a per-aesthetic guide update", {
  result <- legend_top(by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_bottom with by returns a per-aesthetic guide update", {
  result <- legend_bottom(by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("position functions can position legends separately", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_left(by = "colour") +
    legend_bottom(by = "size")
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_style with by parameter
# =============================================================================

test_that("legend_style with by returns a per-aesthetic guide update", {
  result <- legend_style(size = 14, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style can style legends separately", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_style(title_face = "bold", by = "colour") +
    legend_style(size = 10, by = "size")
  expect_s3_class(p, "gg")
})

test_that("legend_style with by normalizes color to colour", {
  result <- legend_style(size = 14, by = "color")
  expect_s3_class(result, "ggguides_guide_update")
})

#' Collect Axes from Patchwork Compositions
#'
#' Collects duplicate axes from multiple plots in a patchwork composition,
#' removing redundant axis labels and ticks. This is a convenience wrapper
#' around patchwork's axis collection functionality.
#'
#' @param x A patchwork object created by combining ggplot2 plots.
#' @param guides How to handle guides. Either \code{"collect"} to combine into a
#'   single legend (default), or \code{"keep"} to maintain separate legends.
#'
#' @return A patchwork object with collected axes.
#'
#' @details
#' This function applies patchwork's \code{axes = "collect"} layout option,
#' which removes duplicate axes when plots are stacked or placed side-by-side.
#' For example, in a 2x1 vertical layout, the x-axis labels on the top plot
#' will be removed since they are redundant with the bottom plot's x-axis.
#'
#' When \code{guides = "collect"}, legends are also merged into a single
#' shared legend.
#'
#' For cowplot: Axis alignment in cowplot requires manual use of
#' \code{\link[cowplot]{align_plots}} with \code{align = "h"} or \code{"v"}.
#' This function does not directly support cowplot objects.
#'
#' @examples
#' library(ggplot2)
#'
#' if (requireNamespace("patchwork", quietly = TRUE)) {
#'   library(patchwork)
#'
#'   # Two plots stacked vertically - x-axis is duplicated
#'   p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'     geom_point() +
#'     labs(y = "Weight")
#'   p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
#'     geom_point() +
#'     labs(y = "Horsepower")
#'
#'   # Without collect_axes: both plots show x-axis
#'   p1 / p2
#'
#'   # With collect_axes: removes redundant x-axis from top plot
#'   collect_axes(p1 / p2)
#'
#'   # Keep separate legends
#'   collect_axes(p1 / p2, guides = "keep")
#' }
#'
#' @seealso \code{\link{collect_legends}}, \code{\link{legend_left}}, \code{\link{legend_wrap}}
#' @export
collect_axes <- function(x, guides = c("collect", "keep")) {
  check_installed("patchwork",
    reason = "to collect axes from plot compositions."
  )

  guides <- match.arg(guides)

  if (!inherits(x, "patchwork")) {
    stop("`x` must be a patchwork object. Create one by combining ggplot objects with |, /, or +.",
         call. = FALSE)
  }

  # Determine guide collection setting
  guide_setting <- if (guides == "collect") "collect" else "keep"

  # Apply patchwork layout with axis collection
  x + patchwork::plot_layout(
    guides = guide_setting,
    axes = "collect"
  )
}

#' @rdname collect_axes
#' @export
align_guides_h <- function(x, guides = c("collect", "keep")) {
  .Deprecated("collect_axes")
  collect_axes(x, guides)
}

library(testthat)
library(ggplot2)

test_that("collect_axes errors without patchwork installed", {
  skip_if(rlang::is_installed("patchwork"))
  expect_error(
    collect_axes(list()),
    "patchwork"
  )
})

test_that("collect_axes errors if x is not a patchwork object", {
  skip_if_not_installed("patchwork")
  expect_error(
    collect_axes(ggplot()),
    "`x` must be a patchwork object."
  )
})

test_that("collect_axes errors if x is a list", {
  skip_if_not_installed("patchwork")
  expect_error(
    collect_axes(list(a = 1)),
    "`x` must be a patchwork object."
  )
})

test_that("collect_axes works with valid patchwork object", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  result <- collect_axes(combined)
  expect_s3_class(result, "patchwork")
})

test_that("collect_axes accepts guides = 'keep'", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  result <- collect_axes(combined, guides = "keep")
  expect_s3_class(result, "patchwork")
})

test_that("collect_axes rejects invalid guides parameter", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  expect_error(
    collect_axes(combined, guides = "invalid"),
    "'arg' should be one of"
  )
})

test_that("align_guides_h is deprecated alias for collect_axes", {
  skip_if_not_installed("patchwork")
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) + geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) + geom_point()
  combined <- p1 / p2

  expect_warning(
    result <- align_guides_h(combined),
    "deprecated"
  )
  expect_s3_class(result, "patchwork")
})

#' Reorder Legend Entries
#'
#' Change the order of legend entries without modifying factor levels in your data.
#' This provides a simpler alternative to manually setting factor levels or using
#' the `breaks` argument in scale functions.
#'
#' @param order A character vector specifying the desired order of legend entries,
#'   or a function to apply to the current order (e.g., `rev`, `sort`).
#' @param aesthetic Character string specifying which aesthetic's legend to reorder.
#'   Default is `"colour"`. Common values: `"colour"`, `"color"`, `"fill"`,
#'   `"shape"`, `"linetype"`, `"size"`, `"alpha"`.
#'
#' @return A ggplot2 scale object that reorders the legend.
#'
#' @details
#' This function works by setting the `breaks` argument of the appropriate
#' discrete scale. It automatically detects whether to use `scale_colour_discrete`,
#' `scale_fill_discrete`, etc. based on the `aesthetic` argument.
#'
#' When `order` is a function (like `rev` or `sort`), it will be applied to
#' the default order of legend entries.
#'
#' @examples
#' library(ggplot2)
#'
#' p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point(size = 3)
#'
#' # Specify exact order
#' p + legend_order(c("8", "6", "4"))
#'
#' # Reverse the order
#' p + legend_order(rev)
#'
#' # Sort alphabetically/numerically
#' p + legend_order(sort)
#'
#' # Reorder fill aesthetic
#' ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
#'   geom_bar() +
#'   legend_order(c("8", "4", "6"), aesthetic = "fill")
#'
#' @seealso
#' \code{\link{legend_reverse}} for a simpler way to reverse legend order,
#' \code{\link{legend_style}} for styling legend appearance.
#'
#' @export
legend_order <- function(order, aesthetic = "colour") {

 # Normalize aesthetic name
 aesthetic <- tolower(aesthetic)
 if (aesthetic == "color") aesthetic <- "colour"

 # Store order specification for later application
 structure(
   list(
     order = order,
     aesthetic = aesthetic
   ),
   class = c("ggguides_legend_order", "gg")
 )
}

#' @export
ggplot_add.ggguides_legend_order <- function(object, plot, ...) {
 aesthetic <- object$aesthetic
 order_spec <- object$order

 # Get the scale for this aesthetic
 scale_name <- paste0("scale_", aesthetic, "_discrete")

 # Determine current breaks/levels from the plot data
 # We need to find the mapping for this aesthetic
 aes_mapping <- plot$mapping[[aesthetic]]
 if (is.null(aes_mapping)) {
   # Check layer mappings
   for (layer in plot$layers) {
     if (!is.null(layer$mapping[[aesthetic]])) {
       aes_mapping <- layer$mapping[[aesthetic]]
       break
     }
   }
 }

 if (is.null(aes_mapping)) {
   warning(sprintf("No '%s' aesthetic found in plot.", aesthetic))
   return(plot)
 }

 # Evaluate the aesthetic expression in the context of the plot data
 # This handles cases like factor(cyl) or other transformations
 var_data <- tryCatch(
   rlang::eval_tidy(aes_mapping, data = plot$data),
   error = function(e) NULL
 )

 if (is.null(var_data)) {
   warning("Could not evaluate aesthetic mapping.")
   return(plot)
 }

 # Get current levels
 if (is.factor(var_data)) {
   current_levels <- levels(var_data)
 } else {
   current_levels <- as.character(sort(unique(var_data)))
 }

 # Determine new order
 if (is.function(order_spec)) {
   new_breaks <- order_spec(current_levels)
 } else if (is.character(order_spec)) {
   # Validate that all specified levels exist
   missing <- setdiff(order_spec, current_levels)
   if (length(missing) > 0) {
     warning(sprintf(
       "The following levels are not in the data and will be ignored: %s",
       paste(missing, collapse = ", ")
     ))
     order_spec <- intersect(order_spec, current_levels)
   }
   # Add any levels not specified (at the end)
   extra <- setdiff(current_levels, order_spec)
   new_breaks <- c(order_spec, extra)
 } else {
   stop("'order' must be a character vector or a function.")
 }

 # Create the appropriate scale
 scale_fn <- switch(aesthetic,
   colour = ggplot2::scale_colour_discrete,
   fill = ggplot2::scale_fill_discrete,
   shape = ggplot2::scale_shape_discrete,
   linetype = ggplot2::scale_linetype_discrete,
   size = ggplot2::scale_size_discrete,
   alpha = ggplot2::scale_alpha_discrete,
   stop(sprintf("Unsupported aesthetic: '%s'", aesthetic))
 )

 plot + scale_fn(breaks = new_breaks)
}

library(testthat)
library(ggplot2)

# =============================================================================
# legend_order() basic tests
# =============================================================================

test_that("legend_order returns a ggguides_legend_order object", {
  result <- legend_order(c("8", "6", "4"))
  expect_s3_class(result, "ggguides_legend_order")
  expect_s3_class(result, "gg")
})

test_that("legend_order stores order specification correctly", {
  result <- legend_order(c("8", "6", "4"))
  expect_equal(result$order, c("8", "6", "4"))
  expect_equal(result$aesthetic, "colour")
})

test_that("legend_order can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point(size = 3) +
    legend_order(c("8", "6", "4"))
  expect_s3_class(p, "gg")
})

# =============================================================================
# Order specification types
# =============================================================================

test_that("legend_order accepts character vector order", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_order(c("8", "6", "4"))
  expect_s3_class(p, "gg")
})

test_that("legend_order accepts function order (rev)", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_order(rev)
  expect_s3_class(p, "gg")
})

test_that("legend_order accepts function order (sort)", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_order(sort)
  expect_s3_class(p, "gg")
})

# =============================================================================
# Aesthetic parameter
# =============================================================================

test_that("legend_order default aesthetic is colour", {
  result <- legend_order(c("a", "b"))
  expect_equal(result$aesthetic, "colour")
})

test_that("legend_order normalizes 'color' to 'colour'", {
  result <- legend_order(c("a", "b"), aesthetic = "color")
  expect_equal(result$aesthetic, "colour")
})

test_that("legend_order accepts fill aesthetic", {
  p <- ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
    geom_bar() +
    legend_order(c("8", "4", "6"), aesthetic = "fill")
  expect_s3_class(p, "gg")
})

test_that("legend_order accepts shape aesthetic", {
  p <- ggplot(mtcars, aes(mpg, wt, shape = factor(cyl))) +
    geom_point(size = 3) +
    legend_order(c("8", "6", "4"), aesthetic = "shape")
  expect_s3_class(p, "gg")
})

test_that("legend_order accepts linetype aesthetic", {
  p <- ggplot(mtcars, aes(mpg, wt, linetype = factor(cyl))) +
    geom_line() +
    legend_order(c("8", "6", "4"), aesthetic = "linetype")
  expect_s3_class(p, "gg")
})

# =============================================================================
# Error handling
# =============================================================================

test_that("legend_order errors on invalid order type", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  expect_error(
    p + legend_order(123),
    "'order' must be a character vector or a function"
  )
})

test_that("legend_order warns on missing levels", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  expect_warning(
    p + legend_order(c("8", "6", "4", "nonexistent")),
    "not in the data"
  )
})

test_that("legend_order warns when aesthetic not found", {
  p <- ggplot(mtcars, aes(mpg, wt)) +
    geom_point()

  expect_warning(
    p + legend_order(c("a", "b")),
    "No 'colour' aesthetic found"
  )
})

# =============================================================================
# Partial order specification
# =============================================================================

test_that("legend_order adds unspecified levels at the end", {
  # When only some levels are specified, unspecified should appear at end
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_order(c("8"))
  expect_s3_class(p, "gg")
})

# =============================================================================
# Layer mappings
# =============================================================================

test_that("legend_order finds aesthetic in layer mappings", {
  # When aesthetic is in layer, not in ggplot()
  p <- ggplot(mtcars, aes(mpg, wt)) +
    geom_point(aes(color = factor(cyl))) +
    legend_order(c("8", "6", "4"))
  expect_s3_class(p, "gg")
})

test_that("legend_order handles unsupported aesthetic", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  # Unsupported aesthetics should error when added to a plot
  # The error occurs in the switch statement when trying to create the scale
  result <- legend_order(c("8", "6", "4"), aesthetic = "unsupported")
  expect_s3_class(result, "ggguides_legend_order")
})

# =============================================================================
# legend_order with non-factor data
# =============================================================================

test_that("legend_order works with numeric (non-factor) mapped variable", {
  # When the mapped variable is not a factor, it gets unique values sorted
  p <- ggplot(mtcars, aes(mpg, wt, color = as.character(cyl))) +
    geom_point() +
    legend_order(c("8", "6", "4"))
  expect_s3_class(p, "gg")
})

test_that("legend_order with size aesthetic", {
  p <- ggplot(mtcars, aes(mpg, wt, size = factor(gear))) +
    geom_point() +
    legend_order(c("5", "4", "3"), aesthetic = "size")
  expect_s3_class(p, "gg")
})

test_that("legend_order with alpha aesthetic", {
  p <- ggplot(mtcars, aes(mpg, wt, alpha = factor(gear))) +
    geom_point() +
    legend_order(c("5", "4", "3"), aesthetic = "alpha")
  expect_s3_class(p, "gg")
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)

## ----left-right, fig.show='hold', out.width='48%'-----------------------------
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p + legend_left() + ggtitle("Left")
p + legend_right() + ggtitle("Right")

## ----top-bottom, fig.show='hold', out.width='48%'-----------------------------
p + legend_top() + ggtitle("Top")
p + legend_bottom() + ggtitle("Bottom")

## ----align-to, fig.show='hold', out.width='48%'-------------------------------
p_labeled <- p + labs(title = "Fuel Economy", y = "Weight (1000 lbs)")

# Default: align to panel (legend aligns with the plot area)
p_labeled + legend_top() + ggtitle("Panel alignment (default)")

# Align to full plot (legend spans title/axis labels too)
p_labeled + legend_top(align_to = "plot") + ggtitle("Plot alignment")

## ----inside-shortcuts, fig.show='hold', out.width='48%'-----------------------
p + legend_inside(position = "topright") + ggtitle("Top right")
p + legend_inside(position = "bottomleft") + ggtitle("Bottom left")

## ----inside-coords------------------------------------------------------------
p + legend_inside(x = 0.95, y = 0.95, just = c("right", "top"))

## ----inside-background--------------------------------------------------------
p + legend_inside(
  position = "topright",
  background = "#FFF3E0",
  border = "#FF9800"
)

## ----direction, fig.show='hold', out.width='48%'------------------------------
p + legend_horizontal() + ggtitle("Horizontal")
p + legend_vertical() + ggtitle("Vertical")

## ----direction-position-------------------------------------------------------
p + legend_right() + legend_horizontal()

## ----legend-none--------------------------------------------------------------
p + legend_none()

#' Style Continuous Color Bar Legends
#'
#' Customize the appearance of color bar legends for continuous scales.
#' This provides a simpler interface to `guide_colourbar()`.
#'
#' @param width Numeric. Width of the color bar in lines. Default is 1.
#' @param height Numeric. Height of the color bar in lines. Default is 5.
#' @param ticks Logical. Whether to show tick marks. Default is TRUE.
#' @param ticks_length Numeric. Length of tick marks in lines. Default is 0.2.
#' @param frame Logical or character. If TRUE, draws a black frame. If a color
#'   string, draws a frame in that color. If FALSE or NULL, no frame. Default is FALSE.
#' @param frame_linewidth Numeric. Line width of the frame. Default is 0.5.
#' @param label Logical. Whether to show labels. Default is TRUE.
#' @param label_position Character. Position of labels: "right", "left", "top",
#'   or "bottom". Default depends on bar orientation.
#' @param title_position Character. Position of title: "top", "bottom", "left",
#'   or "right". Default is "top".
#' @param direction Character. Direction of the bar: "vertical" or "horizontal".
#'   Default is "vertical".
#' @param reverse Logical. Whether to reverse the color bar. Default is FALSE.
#' @param nbin Integer. Number of bins used to draw the color bar. Higher values
#'   give smoother gradients. Default is 300.
#' @param aesthetic Character. Which aesthetic to apply to. Default is "colour".
#'   Common values: "colour", "color", "fill".
#'
#' @return A ggplot2 guides specification.
#'
#' @details
#' This function simplifies common color bar customizations:
#'
#' - **Size**: Use `width` and `height` to make thin/wide bars
#' - **Ticks**: Toggle with `ticks`, adjust length with `ticks_length`
#' - **Frame**: Add a border with `frame = TRUE` or `frame = "grey50"`
#' - **Orientation**: Use `direction = "horizontal"` for horizontal bars
#'
#' The function uses the theme system internally, which is the recommended
#' approach in ggplot2 3.5.0+.
#'
#' @examples
#' library(ggplot2)
#'
#' p <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
#'   geom_tile()
#'
#' # Default appearance
#' p
#'
#' # Taller, thinner bar
#' p + colorbar_style(width = 0.5, height = 10, aesthetic = "fill")
#'
#' # Wide horizontal bar
#' p + colorbar_style(width = 10, height = 0.5, direction = "horizontal",
#'                    aesthetic = "fill")
#'
#' # With frame and no ticks
#' p + colorbar_style(frame = "grey50", ticks = FALSE, aesthetic = "fill")
#'
#' # Thin bar with frame
#' p + colorbar_style(width = 0.5, height = 8, frame = TRUE, aesthetic = "fill")
#'
#' @seealso
#' \code{\link{legend_style}} for styling discrete legends,
#' \code{\link{legend_left}}, \code{\link{legend_bottom}} for positioning.
#'
#' @export
colorbar_style <- function(width = NULL,
                           height = NULL,
                           ticks = NULL,
                           ticks_length = NULL,
                           frame = NULL,
                           frame_linewidth = NULL,
                           label = NULL,
                           label_position = NULL,
                           title_position = NULL,
                           direction = NULL,
                           reverse = NULL,
                           nbin = NULL,
                           aesthetic = "colour") {

 # Normalize aesthetic
 aesthetic <- tolower(aesthetic)
 if (aesthetic == "color") aesthetic <- "colour"

 # Build theme for guide
 theme_args <- list()

 if (!is.null(width)) {
   theme_args$legend.key.width <- ggplot2::unit(width, "lines")
 }
 if (!is.null(height)) {
   theme_args$legend.key.height <- ggplot2::unit(height, "lines")
 }
 if (!is.null(ticks) && !ticks) {
   theme_args$legend.ticks <- ggplot2::element_blank()
 }
 if (!is.null(ticks_length)) {
   theme_args$legend.ticks.length <- ggplot2::unit(ticks_length, "lines")
 }
 if (!is.null(label) && !label) {
   theme_args$legend.text <- ggplot2::element_blank()
 }

 # Build guide arguments
 guide_args <- list()

 if (!is.null(title_position)) {
   guide_args$title.position <- title_position
 }
 if (!is.null(label_position)) {
   guide_args$label.position <- label_position
 }
 if (!is.null(direction)) {
   guide_args$direction <- direction
 }
 if (!is.null(reverse)) {
   guide_args$reverse <- reverse
 }
 if (!is.null(nbin)) {
   guide_args$nbin <- nbin
 }

 # Handle frame - must be set via theme in ggplot2 3.5.0+
 if (!is.null(frame)) {
   if (isTRUE(frame)) {
     frame_colour <- "black"
   } else if (is.character(frame)) {
     frame_colour <- frame
   } else {
     frame_colour <- NA
   }
   theme_args$legend.frame <- ggplot2::element_rect(
     colour = frame_colour,
     linewidth = frame_linewidth %||% 0.5
   )
 }

 # Add theme if we have any theme arguments
 if (length(theme_args) > 0) {
   guide_args$theme <- do.call(ggplot2::theme, theme_args)
 }

 # Create the guide
 guide_spec <- do.call(ggplot2::guide_colourbar, guide_args)

 # Return guides() call
 guides_args <- stats::setNames(list(guide_spec), aesthetic)
 do.call(ggplot2::guides, guides_args)
}

#' @rdname colorbar_style
#' @export
colourbar_style <- colorbar_style

library(testthat)
library(ggplot2)

# =============================================================================
# colorbar_style() tests
# =============================================================================

test_that("colorbar_style returns a Guides object", {
  result <- colorbar_style()
  expect_s3_class(result, "Guides")
})

test_that("colorbar_style can be added to a ggplot", {
  p <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
    geom_tile() +
    colorbar_style(aesthetic = "fill")
  expect_s3_class(p, "gg")
})

test_that("colorbar_style accepts width and height", {
  result <- colorbar_style(width = 0.5, height = 10)
  expect_s3_class(result, "Guides")

  p <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
    geom_tile() +
    colorbar_style(width = 0.5, height = 10, aesthetic = "fill")
  expect_s3_class(p, "gg")
})

test_that("colorbar_style accepts ticks parameter", {
  # Ticks enabled (default)
  result_ticks <- colorbar_style(ticks = TRUE)
  expect_s3_class(result_ticks, "Guides")

  # Ticks disabled
  result_no_ticks <- colorbar_style(ticks = FALSE)
  expect_s3_class(result_no_ticks, "Guides")
})

test_that("colorbar_style accepts ticks_length parameter", {
  result <- colorbar_style(ticks_length = 0.5)
  expect_s3_class(result, "Guides")
})

test_that("colorbar_style accepts frame parameter", {
  # Frame as TRUE (black frame)
  result_true <- colorbar_style(frame = TRUE)
  expect_s3_class(result_true, "Guides")

  # Frame as color string
  result_color <- colorbar_style(frame = "grey50")
  expect_s3_class(result_color, "Guides")

  # Frame as FALSE (no frame)
  result_false <- colorbar_style(frame = FALSE)
  expect_s3_class(result_false, "Guides")
})

test_that("colorbar_style accepts frame_linewidth parameter", {
  result <- colorbar_style(frame = TRUE, frame_linewidth = 1)
  expect_s3_class(result, "Guides")
})

test_that("colorbar_style accepts label parameter", {
  # Labels enabled
  result_labels <- colorbar_style(label = TRUE)
  expect_s3_class(result_labels, "Guides")

  # Labels disabled
  result_no_labels <- colorbar_style(label = FALSE)
  expect_s3_class(result_no_labels, "Guides")
})

test_that("colorbar_style accepts label_position parameter", {
  for (pos in c("right", "left", "top", "bottom")) {
    result <- colorbar_style(label_position = pos)
    expect_s3_class(result, "Guides")
  }
})

test_that("colorbar_style accepts title_position parameter", {
  for (pos in c("top", "bottom", "left", "right")) {
    result <- colorbar_style(title_position = pos)
    expect_s3_class(result, "Guides")
  }
})

test_that("colorbar_style accepts direction parameter", {
  result_v <- colorbar_style(direction = "vertical")
  expect_s3_class(result_v, "Guides")

  result_h <- colorbar_style(direction = "horizontal")
  expect_s3_class(result_h, "Guides")
})

test_that("colorbar_style accepts reverse parameter", {
  result <- colorbar_style(reverse = TRUE)
  expect_s3_class(result, "Guides")
})
test_that("colorbar_style accepts nbin parameter", {
  result <- colorbar_style(nbin = 100)
  expect_s3_class(result, "Guides")
})

test_that("colorbar_style normalizes 'color' to 'colour'", {
  # Using "color" should work the same as "colour"
  p <- ggplot(mtcars, aes(mpg, wt, color = hp)) +
    geom_point() +
    colorbar_style(aesthetic = "color")
  expect_s3_class(p, "gg")
})

test_that("colorbar_style works with fill aesthetic", {
  p <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
    geom_tile() +
    colorbar_style(
      width = 0.5,
      height = 8,
      frame = TRUE,
      ticks = FALSE,
      aesthetic = "fill"
    )
  expect_s3_class(p, "gg")
})

test_that("colorbar_style works with horizontal direction", {
  p <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
    geom_tile() +
    colorbar_style(
      width = 10,
      height = 0.5,
      direction = "horizontal",
      aesthetic = "fill"
    )
  expect_s3_class(p, "gg")
})

# =============================================================================
# colourbar_style() alias tests
# =============================================================================

test_that("colourbar_style is an alias for colorbar_style", {
  expect_identical(colourbar_style, colorbar_style)
})

test_that("colourbar_style works the same as colorbar_style", {
  result <- colourbar_style(width = 1, height = 5)
  expect_s3_class(result, "Guides")
})

# =============================================================================
# Legend Styling Functions
# =============================================================================

# =============================================================================
# Custom ggplot class for auto-centering legend titles
# =============================================================================

#' Add legend_style_centered to ggplot
#' @param object A legend_style_centered object
#' @param plot A ggplot object
#' @param ... Additional arguments (ignored)
#' @return A modified ggplot object with additional class \code{gg_centered_title}
#'   or \code{gg_autofit_legend} (for 90-degree rotation), used to trigger
#'   custom rendering behavior.
#' @importFrom ggplot2 ggplot_add
#' @keywords internal
#' @export
ggplot_add.legend_style_centered <- function(object, plot, ...) {
  # Apply the theme to the plot
  plot <- plot + object$theme

  # Mark this plot for title centering at render time
  # For 90° rotation, also mark for auto-fit
  if (!is.null(object$angle) && abs(object$angle) == 90) {
    class(plot) <- c("gg_autofit_legend", class(plot))
  } else {
    class(plot) <- c("gg_centered_title", class(plot))
  }
  plot
}

#' Print method for centered title plots
#' @param x A gg_centered_title object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns the input object. Called for the side effect of
#'   rendering the plot with centered legend titles.
#' @keywords internal
#' @export
print.gg_centered_title <- function(x, ...) {
  # Apply title centering and render
  g <- center_legend_title(x)
  grid::grid.newpage()
  grid::grid.draw(g)
  invisible(x)
}

#' Plot method for centered title plots
#' @param x A gg_centered_title object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns the input object. Called for the side effect of
#'   rendering the plot with centered legend titles.
#' @keywords internal
#' @export
plot.gg_centered_title <- function(x, ...) {
  print.gg_centered_title(x, ...)
}

#' Convert centered title plot to gtable
#' @param x A gg_centered_title object
#' @return A gtable object (grob table) with centered legend titles, suitable for
#'   rendering with \code{grid::grid.draw()} or saving with \code{ggplot2::ggsave()}.
#' @method ggplotGrob gg_centered_title
#' @keywords internal
#' @export
ggplotGrob.gg_centered_title <- function(x) {
  # Remove our class temporarily to avoid recursion
  class(x) <- setdiff(class(x), "gg_centered_title")
  center_legend_title(x)
}

#' Print method for auto-fit legend plots (90° rotation)
#' @param x A gg_autofit_legend object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns the input object. Called for the side effect of
#'   rendering the plot with auto-fitted and centered legend.
#' @method print gg_autofit_legend
#' @keywords internal
#' @export
print.gg_autofit_legend <- function(x, ...) {
  # Remove our class to avoid recursion
  class(x) <- setdiff(class(x), "gg_autofit_legend")
  # Apply auto-fit then center title
  x <- legend_auto_fit(x)
  g <- center_legend_title(x)
  grid::grid.newpage()
  grid::grid.draw(g)
  invisible(x)
}

#' Plot method for auto-fit legend plots
#' @param x A gg_autofit_legend object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns the input object. Called for the side effect of
#'   rendering the plot with auto-fitted and centered legend.
#' @method plot gg_autofit_legend
#' @keywords internal
#' @export
plot.gg_autofit_legend <- function(x, ...) {
  print.gg_autofit_legend(x, ...)
}

#' Convert auto-fit legend plot to gtable
#' @param x A gg_autofit_legend object
#' @return A gtable object (grob table) with auto-fitted and centered legend,
#'   suitable for rendering with \code{grid::grid.draw()} or saving with
#'   \code{ggplot2::ggsave()}.
#' @method ggplotGrob gg_autofit_legend
#' @keywords internal
#' @export
ggplotGrob.gg_autofit_legend <- function(x) {
  # Remove our class to avoid recursion
  class(x) <- setdiff(class(x), "gg_autofit_legend")
  # Apply auto-fit then center title
  x <- legend_auto_fit(x)
  center_legend_title(x)
}

#' Reverse Legend Order
#'
#' Reverses the order of entries in all legends. Useful when the natural data
#' order doesn't match the desired visual order (e.g., when stacking bars).
#'
#' @return A guides specification that can be added to a plot.
#'
#' @details
#' This function applies \code{guide_legend(reverse = TRUE)} to all common
#' discrete aesthetics: colour, fill, shape, size, linetype, and alpha.
#'
#' @examples
#' library(ggplot2)
#'
#' # Default order
#' p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point()
#'
#' # Reversed order
#' p2 <- p1 + legend_reverse()
#'
#' @seealso \code{\link{legend_wrap}}, \code{\link{legend_style}}
#' @export
legend_reverse <- function() {
  guide_spec <- guide_legend(reverse = TRUE)

  guides(
    colour = guide_spec,
    fill = guide_spec,
    shape = guide_spec,
    size = guide_spec,
    linetype = guide_spec,
    alpha = guide_spec
  )
}

#' Style Legend Appearance
#'
#' A comprehensive one-liner to style all legend elements consistently. Instead
#' of setting multiple theme elements separately, use this function to control
#' text, title, keys, spacing, background, and direction in one call.
#'
#' @param size Text size for legend labels (in points).
#' @param family Font family for legend text.
#' @param face Font face for legend text (\code{"plain"}, \code{"bold"},
#'   \code{"italic"}, \code{"bold.italic"}).
#' @param color Text color for legend labels.
#' @param angle Rotation angle for legend labels (in degrees). Supported values:
#'   45, -45, 90, -90. Text justification is set automatically for optimal
#'   alignment with legend keys.
#' @param title_size Text size for legend title (in points). If \code{NULL},
#'   inherits from \code{size}.
#' @param title_face Font face for legend title. If \code{NULL}, inherits from
#'   \code{face}.
#' @param title_color Text color for legend title. If \code{NULL}, inherits from
#'   \code{color}.
#' @param title_angle Rotation angle for legend title (in degrees).
#' @param title_hjust Horizontal justification for rotated title.
#' @param title_vjust Vertical justification for rotated title.
#' @param title_position Position of legend title relative to keys. One of
#'   \code{"top"}, \code{"bottom"}, \code{"left"}, \code{"right"}.
#' @param key_width Width of legend keys. Numeric (in cm) or a \code{unit} object.
#' @param key_height Height of legend keys. Numeric (in cm) or a \code{unit} object.
#' @param key_fill Background fill color for legend keys.
#' @param spacing Spacing between legend entries. Numeric (in cm) or a \code{unit}
#'   object.
#' @param spacing_x Horizontal spacing between legend entries.
#' @param spacing_y Vertical spacing between legend entries.
#' @param margin Margin around entire legend. Single value (all sides) or vector
#'   of 4 values (top, right, bottom, left) in cm.
#' @param background Legend background fill color. Use \code{NA} for transparent.
#' @param background_color Legend background border color. Use \code{NA} for no
#'   border.
#' @param box_background Background fill for the box containing multiple legends.
#'   Ignored when \code{by} is specified.
#' @param box_margin Margin around the legend box. Single value or 4-vector in cm.
#'   Ignored when \code{by} is specified.
#' @param direction Legend direction: \code{"horizontal"} or \code{"vertical"}.
#' @param byrow For multi-column legends, fill by row (\code{TRUE}) or by column
#'   (\code{FALSE}).
#' @param justification Justification of the legend along its side. For legends
#'   on the top or bottom: \code{"left"}, \code{"center"}, \code{"right"}, or a
#'   numeric value in \code{[0, 1]}. For legends on the left or right:
#'   \code{"top"}, \code{"center"}, \code{"bottom"}, or a numeric value in
#'   \code{[0, 1]}. When \code{by} is specified, applies per-guide via
#'   \code{guide_legend(theme = theme(legend.justification = ...))}. When
#'   \code{by} is NULL, sets \code{legend.justification} for the whole plot.
#'   Requires ggplot2 >= 3.5.0 for per-guide use.
#' @param by Optional aesthetic name (character) to style only a specific legend.
#'   When specified, uses per-guide theming via \code{guide_legend(theme = ...)}.
#'   Requires ggplot2 >= 3.5.0. Common values: \code{"colour"}, \code{"fill"},
#'   \code{"size"}.
#'
#' @return A ggplot2 theme object (when \code{by} is NULL) or a guides
#'   specification (when \code{by} is specified).
#'
#' @examples
#' library(ggplot2)
#'
#' # Simple: consistent font
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_style(size = 12, family = "serif")
#'
#' # Styled title and keys
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_style(
#'     size = 10,
#'     title_size = 14,
#'     title_face = "bold",
#'     key_width = 1.5
#'   )
#'
#' # Full styling with background
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
#'   geom_point() +
#'   legend_style(
#'     size = 11,
#'     title_size = 13,
#'     title_face = "bold",
#'     key_fill = "grey95",
#'     background = "white",
#'     background_color = "grey80",
#'     margin = 0.3
#'   )
#'
#' # Rotated labels for long category names
#' ggplot(mpg, aes(displ, hwy, color = class)) +
#'   geom_point() +
#'   legend_style(angle = 45)
#'
#' # Style only the colour legend
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point() +
#'   legend_style(title_face = "bold", background = "grey95", by = "colour") +
#'   legend_style(size = 10, by = "size")
#'
#' # Per-legend justification: slide each legend along its side
#' ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
#'   geom_point() +
#'   legend_top(by = "colour") +
#'   legend_right(by = "size") +
#'   legend_style(by = "colour", justification = "left") +
#'   legend_style(by = "size",   justification = "top")
#'
#' @seealso \code{\link{legend_left}}, \code{\link{legend_wrap}},
#'   \code{\link{legend_reverse}}
#' @export
legend_style <- function(
    size = NULL,
    family = NULL,
    face = NULL,
    color = NULL,
    angle = NULL,
    title_size = NULL,
    title_face = NULL,
    title_color = NULL,
    title_angle = NULL,
    title_hjust = NULL,
    title_vjust = NULL,
    title_position = NULL,
    key_width = NULL,
    key_height = NULL,
    key_fill = NULL,
    spacing = NULL,
    spacing_x = NULL,
    spacing_y = NULL,
    margin = NULL,
    background = NULL,
    background_color = NULL,
    box_background = NULL,
    box_margin = NULL,
    direction = NULL,
    byrow = NULL,
    justification = NULL,
    by = NULL
) {
  # --- Per-guide styling when `by` is specified ---
  if (!is.null(by)) {
    by <- normalize_aesthetic(by)
    return(build_guide_with_style(
      by = by,
      size = size, family = family, face = face, color = color, angle = angle,
      title_size = title_size, title_face = title_face, title_color = title_color,
      title_angle = title_angle, title_hjust = title_hjust, title_vjust = title_vjust,
      title_position = title_position, key_width = key_width, key_height = key_height,
      key_fill = key_fill, spacing = spacing, spacing_x = spacing_x,
      spacing_y = spacing_y, margin = margin, background = background,
      background_color = background_color, direction = direction, byrow = byrow,
      justification = justification
    ))
  }

  args <- list()

  # --- Text styling ---
  text_args <- list()
  if (!is.null(size)) text_args$size <- size
  if (!is.null(family)) text_args$family <- family
  if (!is.null(face)) text_args$face <- face
  if (!is.null(color)) text_args$colour <- color

  # Handle angle with automatic justification
  if (!is.null(angle)) {
    if (!angle %in% c(45, -45, 90, -90)) {
      stop("angle must be one of: 45, -45, 90, -90.", call. = FALSE)
    }
    text_args$angle <- angle
    # Set optimal hjust/vjust for each angle
    if (angle == 45) {
      text_args$hjust <- 0
      text_args$vjust <- 0.5
    } else if (angle == -45) {
      text_args$hjust <- 1
      text_args$vjust <- 0.5
    } else if (angle == 90) {
      text_args$hjust <- 0.5
      text_args$vjust <- 0.5
    } else if (angle == -90) {
      text_args$hjust <- 0.5
      text_args$vjust <- 0.5
    }
  }

  if (length(text_args) > 0) {
    args$legend.text <- do.call(element_text, text_args)
  }

  # --- Title styling ---
  title_args <- list()
  if (!is.null(size)) title_args$size <- size
  if (!is.null(family)) title_args$family <- family
  if (!is.null(face)) title_args$face <- face
  if (!is.null(color)) title_args$colour <- color
  # For rotated labels: set hjust = 0.5 to prepare for center_legend_title()
  if (!is.null(angle)) {
    title_args$hjust <- 0.5
  }
  if (!is.null(title_size)) title_args$size <- title_size
  if (!is.null(title_face)) title_args$face <- title_face
  if (!is.null(title_color)) title_args$colour <- title_color
  if (!is.null(title_angle)) title_args$angle <- title_angle
  if (!is.null(title_hjust)) title_args$hjust <- title_hjust
  if (!is.null(title_vjust)) title_args$vjust <- title_vjust

  if (length(title_args) > 0) {
    args$legend.title <- do.call(element_text, title_args)
  }
  if (!is.null(title_position)) {
    args$legend.title.position <- title_position
  }

  # --- Key styling ---
  # For 90° rotation, auto-set key_height so labels don't overlap vertically
  # Auto-fit will handle wrapping at render time
  if (!is.null(angle) && abs(angle) == 90 && is.null(key_height)) {
    text_size <- if (!is.null(size)) size else 11
    # Conservative estimate: enough for ~8 chars at given text size
    key_height <- text_size * 0.025 * 8
  }
  if (!is.null(key_width)) {
    args$legend.key.width <- as_unit(key_width, "cm")
  }
  if (!is.null(key_height)) {
    args$legend.key.height <- as_unit(key_height, "cm")
  }
  if (!is.null(key_fill)) {
    args$legend.key <- element_rect(fill = key_fill, color = NA)
  }

  # --- Spacing ---
  if (!is.null(spacing)) {
    args$legend.spacing <- as_unit(spacing, "cm")
  }
  if (!is.null(spacing_x)) {
    args$legend.spacing.x <- as_unit(spacing_x, "cm")
  }
  if (!is.null(spacing_y)) {
    args$legend.spacing.y <- as_unit(spacing_y, "cm")
  }

  # --- Margin ---
  if (!is.null(margin)) {
    args$legend.margin <- as_margin(margin)
  }

  # --- Background ---
  if (!is.null(background) || !is.null(background_color)) {
    bg_fill <- if (!is.null(background)) background else NA
    bg_color <- if (!is.null(background_color)) background_color else NA
    args$legend.background <- element_rect(fill = bg_fill, color = bg_color)
  }
  if (!is.null(box_background)) {
    args$legend.box.background <- element_rect(fill = box_background, color = NA)
  }
  if (!is.null(box_margin)) {
    args$legend.box.margin <- as_margin(box_margin)
  }

  # --- Direction ---

  if (!is.null(direction)) {
    args$legend.direction <- match.arg(direction, c("horizontal", "vertical"))
  }
  if (!is.null(byrow)) {
    args$legend.byrow <- byrow
  }

  # --- Justification ---
  # When `by` is NULL, set legend.justification globally (applies to all sides).
  if (!is.null(justification)) {
    args$legend.justification <- justification
  }

  theme_obj <- do.call(theme, args)

  # When angle is set, return a custom object that will apply title centering
  # and auto-fit for 90° rotation
  if (!is.null(angle)) {
    result <- structure(
      list(theme = theme_obj, angle = angle),
      class = "legend_style_centered"
    )
    return(result)
  }

  theme_obj
}

#' Auto-fit Legend to Plot Height
#'
#' Measures the legend height relative to the plot panel and automatically
#' wraps the legend into multiple columns if it would overflow. This function
#' must be called on a complete ggplot object, not added with \code{+}.
#'
#' @param plot A ggplot object.
#' @param max_ratio Maximum ratio of legend height to panel height before
#'   wrapping is triggered. Default is 0.95 (95 percent of panel height).
#'
#' @return A modified ggplot object with adjusted legend layout.
#'
#' @details
#' This function builds the plot to measure actual dimensions, then rebuilds
#' with an appropriate number of legend rows if the legend is too tall.
#' It's particularly useful after applying \code{legend_style(angle = 90)}
#' which can cause legends to exceed the plot height.
#'
#' Because this requires building the plot twice, it has a small performance
#' cost. For static plots this is negligible.
#'
#' @examples
#' library(ggplot2)
#'
#' # Legend with rotated text that might overflow
#' p <- ggplot(mpg, aes(displ, hwy, color = class)) +
#'   geom_point() +
#'   legend_style(angle = 90)
#'
#' # Auto-fit will wrap if needed
#' legend_auto_fit(p)
#'
#' @seealso \code{\link{legend_style}}, \code{\link{legend_wrap}}
#' @export
legend_auto_fit <- function(plot, max_ratio = 0.95) {
  if (!inherits(plot, "ggplot")) {
    stop("plot must be a ggplot object.", call. = FALSE)
  }

  # Build to gtable to measure dimensions
  gt <- ggplot2::ggplotGrob(plot)

  # Find legend and panel
  guide_idx <- which(grepl("^guide-box", gt$layout$name))
  panel_idx <- which(gt$layout$name == "panel")

  if (length(guide_idx) == 0 || length(panel_idx) == 0) {
    return(plot)  # No legend or panel to measure
  }

  # Calculate panel height: device height minus fixed elements
  # Use a reference device size (typical plot)
  device_height_cm <- 12.7  # ~5 inches, typical plot height

  # Sum all non-null heights (fixed elements like titles, axes, margins)
  fixed_height <- 0
  for (i in seq_along(gt$heights)) {
    h <- gt$heights[[i]]
    h_cm <- tryCatch(
      grid::convertHeight(h, "cm", valueOnly = TRUE),
      error = function(e) 0
    )
    # null units return 0, which is what we want to exclude
    if (h_cm > 0) fixed_height <- fixed_height + h_cm
  }
  panel_height <- device_height_cm - fixed_height

  # Get legend height (legend grob heights are absolute)
  legend_grob <- gt$grobs[[guide_idx[1]]]
  legend_height <- 0
  for (i in seq_along(legend_grob$heights)) {
    h <- legend_grob$heights[[i]]
    h_cm <- tryCatch(
      grid::convertHeight(h, "cm", valueOnly = TRUE),
      error = function(e) 0
    )
    legend_height <- legend_height + h_cm
  }

  # Check if legend fits
  if (legend_height <= panel_height * max_ratio) {
    return(plot)  # Fits fine, no wrapping needed
  }

  # Calculate how many rows we can fit
  # Get number of legend items from the built plot
  built <- ggplot2::ggplot_build(plot)
  n_items <- 0

  # Try to get items from colour scale
  colour_scale <- built$plot$scales$get_scales("colour")
  if (!is.null(colour_scale) && !is.null(colour_scale$get_limits)) {
    limits <- colour_scale$get_limits()
    if (!is.null(limits)) n_items <- length(limits)
  }

  # Fallback to fill scale
 if (n_items == 0) {
    fill_scale <- built$plot$scales$get_scales("fill")
    if (!is.null(fill_scale) && !is.null(fill_scale$get_limits)) {
      limits <- fill_scale$get_limits()
      if (!is.null(limits)) n_items <- length(limits)
    }
  }

  if (n_items == 0) {
    return(plot)  # Can't determine legend items
  }

  item_height <- legend_height / n_items
  max_rows <- floor((panel_height * max_ratio) / item_height)
  max_rows <- max(1, max_rows)  # At least 1 row

  if (max_rows >= n_items) {
    return(plot)  # All items fit in calculated rows
  }

  # Wrap legend
  message(sprintf(
    "legend_auto_fit: Legend (%.1fcm) exceeds %.0f%% of panel (%.1fcm). Wrapping to %d rows.",
    legend_height, max_ratio * 100, panel_height, max_rows
  ))

  guide_spec <- guide_legend(nrow = max_rows)
  plot + guides(
    colour = guide_spec,
    fill = guide_spec,
    shape = guide_spec,
    size = guide_spec,
    linetype = guide_spec,
    alpha = guide_spec
  )
}

# =============================================================================
# Internal Helpers
# =============================================================================

#' Convert numeric to unit
#' @noRd
as_unit <- function(x, default_unit = "cm") {
  if (inherits(x, "unit")) {
    x
  } else {
    unit(x, default_unit)
  }
}

#' Convert numeric to margin
#' @noRd
as_margin <- function(x, default_unit = "cm") {
  if (inherits(x, "margin")) {
    x
  } else if (length(x) == 1) {
    margin(x, x, x, x, default_unit)
  } else if (length(x) == 4) {
    margin(x[1], x[2], x[3], x[4], default_unit)
  } else {
    stop("margin must be a single value or a vector of 4 values.", call. = FALSE)
  }
}

#' Build guide_legend with embedded theme for per-legend styling
#' @noRd
build_guide_with_style <- function(
    by,
    size = NULL, family = NULL, face = NULL, color = NULL, angle = NULL,
    title_size = NULL, title_face = NULL, title_color = NULL,
    title_angle = NULL, title_hjust = NULL, title_vjust = NULL,
    title_position = NULL, key_width = NULL, key_height = NULL,
    key_fill = NULL, spacing = NULL, spacing_x = NULL, spacing_y = NULL,
    margin = NULL, background = NULL, background_color = NULL,
    direction = NULL, byrow = NULL, justification = NULL
) {
  theme_args <- list()

  # --- Text styling ---
  text_args <- list()
  if (!is.null(size)) text_args$size <- size
  if (!is.null(family)) text_args$family <- family
  if (!is.null(face)) text_args$face <- face
  if (!is.null(color)) text_args$colour <- color

  if (!is.null(angle)) {
    if (!angle %in% c(45, -45, 90, -90)) {
      stop("angle must be one of: 45, -45, 90, -90.", call. = FALSE)
    }
    text_args$angle <- angle
    if (angle == 45) {
      text_args$hjust <- 0
      text_args$vjust <- 0.5
    } else if (angle == -45) {
      text_args$hjust <- 1
      text_args$vjust <- 0.5
    } else if (angle == 90) {
      text_args$hjust <- 0.5
      text_args$vjust <- 0.5
    } else if (angle == -90) {
      text_args$hjust <- 0.5
      text_args$vjust <- 0.5
    }
  }

  if (length(text_args) > 0) {
    theme_args$legend.text <- do.call(element_text, text_args)
  }

  # --- Title styling ---
  title_args <- list()
  if (!is.null(size)) title_args$size <- size
  if (!is.null(family)) title_args$family <- family
  if (!is.null(face)) title_args$face <- face
  if (!is.null(color)) title_args$colour <- color
  # For rotated labels: set hjust = 0.5 to prepare for center_legend_title()
  if (!is.null(angle)) {
    title_args$hjust <- 0.5
  }
  if (!is.null(title_size)) title_args$size <- title_size
  if (!is.null(title_face)) title_args$face <- title_face
  if (!is.null(title_color)) title_args$colour <- title_color
  if (!is.null(title_angle)) title_args$angle <- title_angle
  if (!is.null(title_hjust)) title_args$hjust <- title_hjust
  if (!is.null(title_vjust)) title_args$vjust <- title_vjust

  if (length(title_args) > 0) {
    theme_args$legend.title <- do.call(element_text, title_args)
  }
  if (!is.null(title_position)) {
    theme_args$legend.title.position <- title_position
  }

  # --- Key styling ---
  # For 90° rotation, auto-set key_height so labels don't overlap vertically
  if (!is.null(angle) && abs(angle) == 90 && is.null(key_height)) {
    text_size <- if (!is.null(size)) size else 11
    key_height <- text_size * 0.025 * 8
  }
  if (!is.null(key_width)) {
    theme_args$legend.key.width <- as_unit(key_width, "cm")
  }
  if (!is.null(key_height)) {
    theme_args$legend.key.height <- as_unit(key_height, "cm")
  }
  if (!is.null(key_fill)) {
    theme_args$legend.key <- element_rect(fill = key_fill, color = NA)
  }

  # --- Spacing ---
  if (!is.null(spacing)) {
    theme_args$legend.spacing <- as_unit(spacing, "cm")
  }
  if (!is.null(spacing_x)) {
    theme_args$legend.spacing.x <- as_unit(spacing_x, "cm")
  }
  if (!is.null(spacing_y)) {
    theme_args$legend.spacing.y <- as_unit(spacing_y, "cm")
  }

  # --- Margin ---
  if (!is.null(margin)) {
    theme_args$legend.margin <- as_margin(margin)
  }

  # --- Background ---
  if (!is.null(background) || !is.null(background_color)) {
    bg_fill <- if (!is.null(background)) background else NA
    bg_color <- if (!is.null(background_color)) background_color else NA
    theme_args$legend.background <- element_rect(fill = bg_fill, color = bg_color)
  }

  # --- Direction ---
  if (!is.null(direction)) {
    theme_args$legend.direction <- match.arg(direction, c("horizontal", "vertical"))
  }
  if (!is.null(byrow)) {
    theme_args$legend.byrow <- byrow
  }

  # --- Per-guide justification ---
  if (!is.null(justification)) {
    theme_args$legend.justification <- justification
  }

  # Build embedded theme
  embedded_theme <- if (length(theme_args) > 0) {
    do.call(theme, theme_args)
  } else {
    NULL
  }

  ggguides_guide_update(
    by = by,
    guide_params = list(),
    theme_delta = embedded_theme
  )
}

#' Build a ggguides per-aesthetic guide update object
#' @noRd
ggguides_guide_update <- function(by, guide_params = list(), theme_delta = NULL) {
  structure(
    list(
      by = by,
      guide_params = guide_params,
      theme_delta = theme_delta
    ),
    class = c("ggguides_guide_update", "gg")
  )
}

#' @export
ggplot_add.ggguides_guide_update <- function(object, plot, ...) {
  by <- object$by

  existing <- tryCatch(plot$guides$guides[[by]], error = function(e) NULL)

  if (is.null(existing) || !inherits(existing, "Guide")) {
    new_params <- object$guide_params
    if (!is.null(object$theme_delta)) new_params$theme <- object$theme_delta
    new_guide <- do.call(ggplot2::guide_legend, new_params)
    guides_arg <- stats::setNames(list(new_guide), by)
    return(plot + do.call(ggplot2::guides, guides_arg))
  }

  for (nm in names(object$guide_params)) {
    existing$params[[nm]] <- object$guide_params[[nm]]
  }

  if (!is.null(object$theme_delta)) {
    existing_theme <- existing$params$theme
    if (inherits(existing_theme, "theme")) {
      existing$params$theme <- existing_theme + object$theme_delta
    } else {
      existing$params$theme <- object$theme_delta
    }
  }

  plot
}

#' Center Legend Title Over Keys
#'
#' Modifies a ggplot so that legend titles are centered over the key column
#' only, rather than over the full legend width (keys + labels). This is
#' particularly useful when legend labels are rotated, as the default centering
#' places the title too far to the right.
#'
#' @param plot A ggplot object.
#' @param position Legend position to modify. One of \code{"right"}, \code{"left"},
#'   \code{"top"}, \code{"bottom"}, or \code{"all"} (default).
#'
#' @return A modified gtable object that can be drawn with \code{grid::grid.draw()}
#'   or saved with \code{ggplot2::ggsave()}.
#'
#' @details
#' This function works by modifying the legend's internal gtable structure,
#' restricting the title's column span to only the keys column. Long titles
#' will automatically wrap to fit within the key column width, and proper
#' spacing is added to prevent overlap with rotated labels.
#'
#' The title should have \code{hjust = 0.5} set (done automatically by
#' \code{legend_style(angle = ...)}) for proper centering.
#'
#' @examples
#' library(ggplot2)
#'
#' p <- ggplot(mpg, aes(displ, hwy, color = class)) +
#'   geom_point() +
#'   legend_style(angle = 45) +
#'   labs(color = "Vehicle Class")
#'
#' # Center title over keys only (long titles wrap automatically)
#' # Returns a gtable - use grid::grid.draw() to render
#' g <- center_legend_title(p)
#' grid::grid.draw(g)
#'
#' @seealso \code{\link{legend_style}}
#' @export
center_legend_title <- function(plot, position = "all") {
  if (!inherits(plot, "gg")) {
    stop("plot must be a ggplot object.", call. = FALSE)
  }

  # Convert to gtable
  g <- ggplot2::ggplotGrob(plot)

  # Determine which guide boxes to modify
  positions <- if (position == "all") {
    c("right", "left", "top", "bottom")
  } else {
    position
  }

  for (pos in positions) {
    box_name <- paste0("guide-box-", pos)
    legend_idx <- which(g$layout$name == box_name)

    if (length(legend_idx) == 0) next

    legend <- g$grobs[[legend_idx]]
    if (!inherits(legend, "gtable")) next

    # Find the guides grob within the legend box
    guides_idx <- which(legend$layout$name == "guides")
    if (length(guides_idx) == 0) next

    guides_grob <- legend$grobs[[guides_idx]]
    if (!inherits(guides_grob, "gtable")) next

    # Find the title
    title_idx <- which(guides_grob$layout$name == "title")
    if (length(title_idx) == 0) next

    # Get key column width and label start position
    key_col <- guides_grob$layout[title_idx, "l"]
    key_width <- guides_grob$widths[[key_col]]
    key_width_cm <- grid::convertWidth(key_width, "cm", valueOnly = TRUE)

    # Get the title grob and check if wrapping is needed
    title_grob <- guides_grob$grobs[[title_idx]]

    # Only wrap if title would overlap with rotated labels
    if (inherits(title_grob, "titleGrob")) {
      title_children <- title_grob$children
      text_grob_idx <- which(sapply(title_children, inherits, "text"))

      if (length(text_grob_idx) > 0) {
        text_grob <- title_children[[text_grob_idx[1]]]
        title_text <- text_grob$label

        # Get text properties
        gp <- text_grob$gp
        fontsize <- if (!is.null(gp$fontsize)) gp$fontsize else 11

        # Estimate title width
        char_width_cm <- fontsize * 0.035 * 0.6
        title_width_cm <- nchar(title_text) * char_width_cm

        # Find the first label to check for overlap
        # Labels at 45° start at the key edge and extend diagonally
        # The "safe zone" is approximately key_width + small buffer
        # Only wrap if title extends significantly beyond the key column
        safe_zone_cm <- key_width_cm + 0.3  # key width + small margin

        if (title_width_cm > safe_zone_cm) {
          # Wrap to fit within the safe zone
          max_chars <- floor(safe_zone_cm / char_width_cm)
          wrapped <- strwrap(title_text, width = max(max_chars, 1))
          new_label <- paste(wrapped, collapse = "\n")
          text_grob$label <- new_label

          # Update the grob
          title_grob$children[[text_grob_idx[1]]] <- text_grob
          guides_grob$grobs[[title_idx]] <- title_grob

          # Add extra row height for wrapped title
          title_row <- guides_grob$layout[title_idx, "t"]
          n_lines <- length(wrapped)
          if (n_lines > 1) {
            line_height_cm <- fontsize * 0.035 * 1.2
            new_height <- grid::unit(line_height_cm * n_lines, "cm")
            guides_grob$heights[[title_row]] <- new_height
          }
        }
      }
    }

    # Restrict title to only span the keys column
    guides_grob$layout[title_idx, "r"] <- guides_grob$layout[title_idx, "l"]

    # Put modified grobs back
    legend$grobs[[guides_idx]] <- guides_grob
    g$grobs[[legend_idx]] <- legend
  }

  g
}

library(testthat)
library(ggplot2)

# =============================================================================
# legend_reverse() tests
# =============================================================================

test_that("legend_reverse returns a Guides object", {
  result <- legend_reverse()
  expect_s3_class(result, "Guides")
})

test_that("legend_reverse can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_reverse()
  expect_s3_class(p, "gg")
})

test_that("legend_reverse works with fill aesthetic", {
  p <- ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
    geom_bar() +
    legend_reverse()
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_style() tests
# =============================================================================

test_that("legend_style returns a theme object", {
  result <- legend_style(size = 12)
  expect_s3_class(result, "theme")
})

test_that("legend_style with no args returns empty theme", {
  result <- legend_style()
  expect_s3_class(result, "theme")
})

test_that("legend_style sets text size", {
  result <- legend_style(size = 14)
  expect_s3_class(result$legend.text, "element_text")
})

test_that("legend_style sets font family", {
  result <- legend_style(family = "serif")
  expect_s3_class(result$legend.text, "element_text")
})

test_that("legend_style sets title properties", {
  result <- legend_style(title_size = 16, title_face = "bold")
  expect_s3_class(result$legend.title, "element_text")
})

test_that("legend_style sets key dimensions", {
  result <- legend_style(key_width = 2, key_height = 1)
  expect_s3_class(result$legend.key.width, "unit")
  expect_s3_class(result$legend.key.height, "unit")
})

test_that("legend_style sets background", {
  result <- legend_style(background = "grey90", background_color = "black")
  expect_s3_class(result$legend.background, "element_rect")
})

test_that("legend_style sets direction", {
  result <- legend_style(direction = "horizontal")
  expect_equal(result$legend.direction, "horizontal")
})

test_that("legend_style sets margin as single value", {
  result <- legend_style(margin = 0.5)
  # ggplot2 3.5.0+ uses "ggplot2::margin" class
  expect_true(inherits(result$legend.margin, "unit"))
})

test_that("legend_style sets margin as 4-vector", {
  result <- legend_style(margin = c(0.1, 0.2, 0.3, 0.4))
  expect_true(inherits(result$legend.margin, "unit"))
})

test_that("legend_style can be added to a ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_style(size = 12, family = "serif", title_face = "bold")
  expect_s3_class(p, "gg")
})

test_that("legend_style combines with position functions", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_left() +
    legend_style(size = 14)
  expect_s3_class(p, "gg")
  expect_equal(p$theme$legend.position, "left")
})

# =============================================================================
# as_unit() helper tests
# =============================================================================

test_that("as_unit converts numeric to unit", {
  result <- ggguides:::as_unit(2, "cm")
  expect_s3_class(result, "unit")
})

test_that("as_unit passes through unit objects", {
  input <- unit(3, "mm")
  result <- ggguides:::as_unit(input, "cm")
  expect_identical(result, input)
})

# =============================================================================
# as_margin() helper tests
# =============================================================================

test_that("as_margin converts single value", {
  result <- ggguides:::as_margin(0.5)
  # ggplot2 3.5.0+ uses "ggplot2::margin" class
  expect_true(inherits(result, "unit"))
})

test_that("as_margin converts 4-vector", {
  result <- ggguides:::as_margin(c(1, 2, 3, 4))
  expect_true(inherits(result, "unit"))
})

test_that("as_margin errors on invalid length", {
  expect_error(ggguides:::as_margin(c(1, 2)), "single value or a vector of 4")
})

# =============================================================================
# legend_style with angle tests
# =============================================================================

test_that("legend_style with angle returns legend_style_centered object", {
  result <- legend_style(angle = 45)
  expect_s3_class(result, "legend_style_centered")
})

test_that("legend_style accepts valid angles", {
  for (angle in c(45, -45, 90, -90)) {
    result <- legend_style(angle = angle)
    expect_s3_class(result, "legend_style_centered")
  }
})

test_that("legend_style with angle can be added to ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_style(angle = 45)
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_auto_fit() tests
# =============================================================================

test_that("legend_auto_fit returns a ggplot object", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point()

  result <- legend_auto_fit(p)
  expect_s3_class(result, "ggplot")
})

test_that("legend_auto_fit errors if not a ggplot", {
  expect_error(legend_auto_fit("not a plot"), "must be a ggplot object")
  expect_error(legend_auto_fit(list()), "must be a ggplot object")
})

test_that("legend_auto_fit returns unchanged plot when legend fits", {
  # Small legend should not trigger wrapping
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  result <- legend_auto_fit(p)
  expect_s3_class(result, "ggplot")
})

test_that("legend_auto_fit accepts max_ratio parameter", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point()

  result <- legend_auto_fit(p, max_ratio = 0.5)
  expect_s3_class(result, "ggplot")
})

test_that("legend_auto_fit returns unchanged plot when no legend", {
  p <- ggplot(mtcars, aes(mpg, wt)) +
    geom_point()

  result <- legend_auto_fit(p)
  expect_s3_class(result, "ggplot")
})

test_that("legend_auto_fit works with fill aesthetic", {
  p <- ggplot(mpg, aes(class, fill = class)) +
    geom_bar()

  result <- legend_auto_fit(p)
  expect_s3_class(result, "ggplot")
})

# =============================================================================
# center_legend_title() tests
# =============================================================================

test_that("center_legend_title returns a gtable", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point() +
    legend_style(angle = 45)

  result <- center_legend_title(p)
  expect_s3_class(result, "gtable")
})

test_that("center_legend_title errors if not a ggplot", {
  expect_error(center_legend_title("not a plot"), "must be a ggplot object")
})

test_that("center_legend_title accepts position parameter", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  # Test specific positions
  for (pos in c("right", "left", "top", "bottom")) {
    result <- center_legend_title(p, position = pos)
    expect_s3_class(result, "gtable")
  }
})

test_that("center_legend_title accepts position = 'all'", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()

  result <- center_legend_title(p, position = "all")
  expect_s3_class(result, "gtable")
})

test_that("center_legend_title works with rotated labels", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point() +
    legend_style(angle = 45) +
    labs(color = "Vehicle Class")

  result <- center_legend_title(p)
  expect_s3_class(result, "gtable")
})

test_that("center_legend_title works with long titles", {
  p <- ggplot(mpg, aes(displ, hwy, color = class)) +
    geom_point() +
    legend_style(angle = 45) +
    labs(color = "This is a Very Long Legend Title That Should Wrap")

  result <- center_legend_title(p)
  expect_s3_class(result, "gtable")
})

# =============================================================================
# S3 method tests for gg_centered_title
# =============================================================================

test_that("gg_centered_title class is applied with angle", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_style(angle = 45)

  expect_true("gg_centered_title" %in% class(p))
})

test_that("gg_autofit_legend class is applied with 90-degree angle", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_style(angle = 90)

  expect_true("gg_autofit_legend" %in% class(p))
})

test_that("ggplotGrob.gg_centered_title returns gtable", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_style(angle = 45)

  result <- ggplot2::ggplotGrob(p)
  expect_s3_class(result, "gtable")
})

test_that("ggplotGrob.gg_autofit_legend returns gtable", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    legend_style(angle = 90)

  result <- ggplot2::ggplotGrob(p)
  expect_s3_class(result, "gtable")
})

# =============================================================================
# legend_style with by parameter
# =============================================================================

test_that("legend_style with by parameter returns per-aesthetic guide update", {
  result <- legend_style(size = 12, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by normalizes color to colour", {
  result <- legend_style(size = 12, by = "color")
  expect_s3_class(result, "ggguides_guide_update")
})

# =============================================================================
# legend_style with by parameter - comprehensive tests
# =============================================================================

test_that("legend_style with by accepts all styling parameters", {
  result <- legend_style(
    size = 12,
    family = "serif",
    face = "bold",
    color = "red",
    title_size = 14,
    title_face = "bold",
    title_color = "blue",
    key_width = 1.5,
    key_height = 1,
    background = "grey90",
    by = "colour"
  )
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by and angle works", {
  result <- legend_style(angle = 45, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by and title_position works", {
  result <- legend_style(title_position = "top", by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by and spacing works", {
  result <- legend_style(spacing = 0.5, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")

  result <- legend_style(spacing_x = 0.3, spacing_y = 0.5, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by and margin works", {
  result <- legend_style(margin = 0.5, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by and direction works", {
  result <- legend_style(direction = "horizontal", by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by and byrow works", {
  result <- legend_style(byrow = TRUE, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("legend_style with by can be added to ggplot", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
    geom_point() +
    legend_style(size = 12, by = "colour") +
    legend_style(size = 10, by = "size")
  expect_s3_class(p, "gg")
})

# =============================================================================
# legend_style additional parameters
# =============================================================================

test_that("legend_style sets title_angle", {
  result <- legend_style(title_angle = 45)
  expect_s3_class(result, "theme")
})

test_that("legend_style sets title_hjust and title_vjust", {
  result <- legend_style(title_hjust = 0.5, title_vjust = 0.5)
  expect_s3_class(result, "theme")
})

test_that("legend_style sets title_position", {
  for (pos in c("top", "bottom", "left", "right")) {
    result <- legend_style(title_position = pos)
    expect_s3_class(result, "theme")
  }
})

test_that("legend_style sets key_fill", {
  result <- legend_style(key_fill = "white")
  expect_s3_class(result, "theme")
})

test_that("legend_style sets spacing", {
  result <- legend_style(spacing = 0.5)
  expect_s3_class(result, "theme")
})

test_that("legend_style sets spacing_x and spacing_y separately", {
  result <- legend_style(spacing_x = 0.3, spacing_y = 0.5)
  expect_s3_class(result, "theme")
})

test_that("legend_style sets box_background and box_margin", {
  result <- legend_style(box_background = "grey95", box_margin = 0.2)
  expect_s3_class(result, "theme")
})

test_that("legend_style sets byrow", {
  result <- legend_style(byrow = TRUE)
  expect_s3_class(result, "theme")
})

# =============================================================================
# legend_auto_fit with large legends
# =============================================================================

test_that("legend_auto_fit triggers wrapping for many items", {
  # Create plot with many legend items that might overflow
  df <- data.frame(
    x = 1:20,
    y = 1:20,
    group = factor(paste0("Category_", sprintf("%02d", 1:20)))
  )

  p <- ggplot(df, aes(x, y, color = group)) +
    geom_point() +
    legend_style(angle = 90)

  # This should potentially trigger wrapping
  result <- suppressMessages(legend_auto_fit(p, max_ratio = 0.3))
  expect_s3_class(result, "ggplot")
})

# =============================================================================
# build_guide_with_style tests (via legend_style with by)
# =============================================================================

test_that("build_guide_with_style handles all text parameters", {
  # Test through legend_style with by parameter
  result <- legend_style(
    size = 14,
    family = "mono",
    face = "italic",
    color = "darkblue",
    by = "colour"
  )
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("build_guide_with_style handles title parameters", {
  result <- legend_style(
    title_size = 16,
    title_face = "bold.italic",
    title_color = "darkred",
    title_angle = 0,
    title_hjust = 0,
    title_vjust = 1,
    title_position = "left",
    by = "colour"
  )
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("build_guide_with_style handles key parameters", {
  result <- legend_style(
    key_width = 2,
    key_height = 0.5,
    key_fill = "lightgrey",
    by = "colour"
  )
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("build_guide_with_style handles background parameters", {
  result <- legend_style(
    background = "white",
    background_color = "black",
    by = "colour"
  )
  expect_s3_class(result, "ggguides_guide_update")
})

# =============================================================================
# legend_style face and color without by parameter
# =============================================================================

test_that("legend_style sets face without by parameter", {
  result <- legend_style(face = "bold")
  expect_s3_class(result, "theme")
  expect_s3_class(result$legend.text, "element_text")
})

test_that("legend_style sets color without by parameter", {
  result <- legend_style(color = "red")
  expect_s3_class(result, "theme")
})

test_that("legend_style sets title_color without by parameter", {
  result <- legend_style(title_color = "blue")
  expect_s3_class(result, "theme")
})

test_that("legend_style errors on invalid angle", {
  expect_error(legend_style(angle = 30), "angle must be one of")
  expect_error(legend_style(angle = 180), "angle must be one of")
})

# =============================================================================
# legend_style with negative angles
# =============================================================================

test_that("legend_style with angle = -45 sets correct hjust/vjust", {
  result <- legend_style(angle = -45)
  expect_s3_class(result, "legend_style_centered")
})

test_that("legend_style with angle = -90 sets correct hjust/vjust", {
  result <- legend_style(angle = -90)
  expect_s3_class(result, "legend_style_centered")
})

# =============================================================================
# build_guide_with_style angle tests
# =============================================================================

test_that("build_guide_with_style handles angle = -45", {
  result <- legend_style(angle = -45, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("build_guide_with_style handles angle = -90", {
  result <- legend_style(angle = -90, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("build_guide_with_style handles angle = 90", {
  result <- legend_style(angle = 90, by = "colour")
  expect_s3_class(result, "ggguides_guide_update")
})

test_that("build_guide_with_style errors on invalid angle", {
  expect_error(legend_style(angle = 30, by = "colour"), "angle must be one of")
})

# =============================================================================
# legend_auto_fit with fill aesthetic
# =============================================================================

test_that("legend_auto_fit uses fill scale when colour scale not available", {
  df <- data.frame(
    x = 1:15,
    y = 1:15,
    group = factor(paste0("Cat_", sprintf("%02d", 1:15)))
  )

  p <- ggplot(df, aes(x, y, fill = group)) +
    geom_tile()

  result <- suppressMessages(legend_auto_fit(p, max_ratio = 0.3))
  expect_s3_class(result, "ggplot")
})

# =============================================================================
# as_margin with margin class input
# =============================================================================

test_that("as_margin passes through margin objects", {
  m <- ggplot2::margin(1, 2, 3, 4, "cm")
  result <- ggguides:::as_margin(m)
  expect_identical(result, m)
})

# =============================================================================
# center_legend_title with specific positions
# =============================================================================

test_that("center_legend_title with position = 'right' only", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    theme(legend.position = "right")

  result <- center_legend_title(p, position = "right")
  expect_s3_class(result, "gtable")
})

test_that("center_legend_title with position = 'left' only", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    theme(legend.position = "left")

  result <- center_legend_title(p, position = "left")
  expect_s3_class(result, "gtable")
})

test_that("center_legend_title with position = 'bottom' only", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    theme(legend.position = "bottom")

  result <- center_legend_title(p, position = "bottom")
  expect_s3_class(result, "gtable")
})

test_that("center_legend_title with position = 'top' only", {
  p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    theme(legend.position = "top")

  result <- center_legend_title(p, position = "top")
  expect_s3_class(result, "gtable")
})

#' Wrap Legend Entries into Columns or Rows
#'
#' A one-liner to arrange legend entries in a grid layout. Works with discrete
#' legends by applying the specified layout to all color, fill, shape, size,
#' linetype, and alpha aesthetics.
#'
#' @param ncol Number of columns for the legend layout. If \code{NULL}, determined
#'   automatically based on \code{nrow}.
#' @param nrow Number of rows for the legend layout. If \code{NULL}, determined
#'   automatically based on \code{ncol}.
#' @param byrow Logical. If \code{TRUE} (default), fills by row first. If \code{FALSE},
#'   fills by column first.
#'
#' @return A list of guide specifications that can be added to a plot.
#'
#' @details
#' This function creates a \code{guides()} specification that applies the same
#' column/row layout to all common discrete aesthetics. At least one of \code{ncol}
#' or \code{nrow} should be specified.
#'
#' @examples
#' library(ggplot2)
#'
#' # Wrap a long legend into 2 columns
#' ggplot(mpg, aes(displ, hwy, color = class)) +
#'   geom_point() +
#'   legend_wrap(ncol = 2)
#'
#' # Wrap into 3 rows, filling by column
#' ggplot(mpg, aes(displ, hwy, color = class)) +
#'   geom_point() +
#'   legend_wrap(nrow = 3, byrow = FALSE)
#'
#' # Combine with legend_left for left-aligned wrapped legends
#' ggplot(mpg, aes(displ, hwy, color = class)) +
#'   geom_point() +
#'   legend_wrap(ncol = 2) +
#'   legend_left()
#'
#' @seealso \code{\link{legend_left}}, \code{\link{collect_legends}}
#' @export
legend_wrap <- function(ncol = NULL, nrow = NULL, byrow = TRUE) {
  guide_spec <- guide_legend(ncol = ncol, nrow = nrow, byrow = byrow)

  guides(
    colour = guide_spec,
    fill = guide_spec,
    shape = guide_spec,
    size = guide_spec,
    linetype = guide_spec,
    alpha = guide_spec
  )
}

---
title: "Legend Positioning"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Legend Positioning}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

ggguides provides position helpers that handle legend placement without
requiring knowledge of ggplot2's theme system. Each function sets the
appropriate combination of `legend.position`, `legend.justification`,
and `legend.box.just` to achieve proper alignment.

## Basic Positioning

### Left and Right

```{r left-right, fig.show='hold', out.width='48%'}
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p + legend_left() + ggtitle("Left")
p + legend_right() + ggtitle("Right")
```

### Top and Bottom

When placing legends at the top or bottom, the legend is automatically
oriented horizontally:

```{r top-bottom, fig.show='hold', out.width='48%'}
p + legend_top() + ggtitle("Top")
p + legend_bottom() + ggtitle("Bottom")
```

### Alignment with Titles

Use the `align_to` parameter to control how the legend aligns with the plot.
The difference is most visible when there's a y-axis label:

```{r align-to, fig.show='hold', out.width='48%'}
p_labeled <- p + labs(title = "Fuel Economy", y = "Weight (1000 lbs)")

# Default: align to panel (legend aligns with the plot area)
p_labeled + legend_top() + ggtitle("Panel alignment (default)")

# Align to full plot (legend spans title/axis labels too)
p_labeled + legend_top(align_to = "plot") + ggtitle("Plot alignment")
```

## Inside Positioning

`legend_inside()` places the legend within the plot panel. You can use
either coordinate values or named shortcuts.

### Using Shortcuts

Available shortcuts: `"topright"`, `"topleft"`, `"bottomright"`, `"bottomleft"`, `"center"`

```{r inside-shortcuts, fig.show='hold', out.width='48%'}
p + legend_inside(position = "topright") + ggtitle("Top right")
p + legend_inside(position = "bottomleft") + ggtitle("Bottom left")
```

### Using Coordinates

For precise control, specify x/y coordinates (0-1 scale) and justification:

```{r inside-coords}
p + legend_inside(x = 0.95, y = 0.95, justification = c("right", "top"))
```

### Adding Background

Inside legends often need a background for readability:

```{r inside-background}
p + legend_inside(
  position = "topright",
  background = "#FFF3E0",
  border = "#FF9800"
)
```

## Direction Control

### Horizontal and Vertical

Control legend entry direction independently of position:

```{r direction, fig.show='hold', out.width='48%'}
p + legend_horizontal() + ggtitle("Horizontal")
p + legend_vertical() + ggtitle("Vertical")
```

### Combining Direction and Position

```{r direction-position}
p + legend_right() + legend_horizontal()
```

## Removing the Legend

```{r legend-none}
p + legend_none()
```

## Summary

| Function | Position | Auto-Direction |
|----------|----------|----------------|
| `legend_left()` | Left of panel | Vertical |
| `legend_right()` | Right of panel | Vertical |
| `legend_top()` | Above panel | Horizontal |
| `legend_bottom()` | Below panel | Horizontal |
| `legend_inside()` | Inside panel | Unchanged |
| `legend_none()` | Hidden | N/A |

**Learn more:**

- [Styling & Customization](styling.html) for font, background, and border options

- [Patchwork Integration](patchwork.html) for multi-panel workflows

---
title: "Patchwork Integration"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Patchwork Integration}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

When combining multiple plots with patchwork, legends often need special handling:

- **Duplicate legends** - Same aesthetic mapped in multiple plots creates redundancy

- **Alignment issues** - Legends may not align properly across panels

- **Spanning** - Legends should sometimes span multiple rows

ggguides provides `collect_legends()` and `collect_axes()` to address these challenges.

## The Problem: Duplicate Legends

```{r problem, fig.height=4}
library(patchwork)

p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1", color = "Cylinders")
p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2", color = "Cylinders")

# Default patchwork: duplicate legends
p1 | p2
```

## Basic Legend Collection

Use `collect_legends()` to gather legends from all plots in a composition:

```{r collect-basic, fig.height=4}
collect_legends(p1 | p2)
```

### Position Options

Control where the collected legend appears:

```{r collect-bottom, fig.height=5}
collect_legends(p1 | p2, position = "bottom")
```

```{r collect-left, fig.height=4}
collect_legends(p1 | p2, position = "left")
```

## Stacked Plots

For vertically stacked plots, legends can be centered or span the full height.
Using different plot heights makes the spanning behavior more visible.

### Default: Centered

```{r stacked-default, fig.height=7}
# Create plots with different heights using plot_layout
p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3", color = "Cylinders")

# Stack with different heights: 1, 1/2, 1/4
stacked <- (p1 / p2 / p3) + plot_layout(heights = c(4, 2, 1))
collect_legends(stacked, position = "right")
```

### Spanning Full Height

Use `span = TRUE` to make the legend fill the full height:

```{r stacked-span, fig.height=7}
gt <- collect_legends(stacked, position = "right", span = TRUE)
grid::grid.draw(gt)
```

### Attaching to Specific Rows

Attach the legend to specific row(s) instead of spanning all:

```{r span-row1, fig.height=7}
# Legend attached to row 1 only (the tallest plot)
gt <- collect_legends(stacked, position = "right", span = 1)
grid::grid.draw(gt)
```
```{r span-row12, fig.height=7}
# Legend attached to rows 1 and 2
gt <- collect_legends(stacked, position = "right", span = 1:2)
grid::grid.draw(gt)
```

## Collecting Duplicate Axes

When stacking plots vertically, the x-axis may be duplicated. Use `collect_axes()`
to remove redundant axes:

```{r collect-axes, fig.height=5}
# Plots stacked vertically - x-axis is duplicated
p_top <- ggplot(mtcars, aes(mpg, wt)) +
  geom_point() + labs(y = "Weight")

p_bottom <- ggplot(mtcars, aes(mpg, disp)) +
  geom_point() + labs(y = "Displacement")

# Without axis collection (both have x-axis)
p_top / p_bottom
```

```{r collect-axes-fixed, fig.height=5}
# With axis collection (removes redundant x-axis from top)
collect_axes(p_top / p_bottom)
```

## Combining with Styling

ggguides functions work together:

```{r combined, fig.height=5}
p1_styled <- p1 + legend_style(size = 11, title_face = "bold")
p2_styled <- p2 + legend_style(size = 11, title_face = "bold")

collect_legends(p1_styled | p2_styled, position = "right")
```

## Complex Layouts

Handle more complex patchwork layouts:

```{r complex, fig.height=6}
p4 <- ggplot(mtcars, aes(qsec, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 4", color = "Cylinders")

# 2x2 grid
layout <- (p1 | p2) / (p3 | p4)
collect_legends(layout, position = "right")
```

## cowplot / Grid Users

ggguides provides cowplot-compatible functions that work without patchwork:

```{r cowplot-setup, include=FALSE}
# Fresh plots for cowplot examples (not modified by patchwork)
cp1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1", color = "Cylinders")
cp2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2", color = "Cylinders")
cp3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3", color = "Cylinders")
cp4 <- ggplot(mtcars, aes(qsec, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 4", color = "Cylinders")
```

### `get_legend()` - Extract Legend

```{r get-legend}
# Extract legend from a plot
leg <- get_legend(cp1)

# Use with cowplot::plot_grid() or grid::grid.draw()
grid::grid.newpage()
grid::grid.draw(leg)
```

### `shared_legend()` - Combine Plots with Shared Legend

```{r shared-legend-side, fig.height=4}
# Side-by-side with shared legend on right
gt <- shared_legend(cp1, cp2, ncol = 2, position = "right")
grid::grid.newpage()
grid::grid.draw(gt)
```

```{r shared-legend-stacked, fig.height=6}
# Stacked with legend at bottom
gt <- shared_legend(cp1, cp2, cp3, ncol = 1, position = "bottom")
grid::grid.newpage()
grid::grid.draw(gt)
```

```{r shared-legend-grid, fig.height=5}
# 2x2 grid
gt <- shared_legend(cp1, cp2, cp3, cp4, ncol = 2, nrow = 2, position = "right")
grid::grid.newpage()
grid::grid.draw(gt)
```

All ggguides styling functions (`legend_style()`, `legend_wrap()`, etc.) work
on individual plots regardless of layout package.

## Summary

| Function | Purpose | Key Parameters |
|----------|---------|----------------|
| `collect_legends()` | Gather legends from patchwork | `position`, `span` |
| `collect_axes()` | Remove duplicate axes | `guides` |
| `get_legend()` | Extract legend as grob | - |
| `shared_legend()` | Combine plots with shared legend | `ncol`, `nrow`, `position` |

**Learn more:**

- [Legend Positioning](positioning.html) for single-plot placement

- [Styling & Customization](styling.html) for legend appearance

---
title: "Getting Started with ggguides"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with ggguides}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

**ggguides** simplifies common legend operations in ggplot2. Instead of memorizing
theme element names and their nested structure, you can use intuitive one-liner
functions to position, style, and manage legends.

**Key features:**

- Position legends with `legend_left()`, `legend_right()`, `legend_top()`, `legend_bottom()`, `legend_inside()`

- Style legends with `legend_style()` for fonts, backgrounds, and borders

- Wrap legend entries with `legend_wrap()` for multi-column layouts

- Collect legends from patchwork compositions with `collect_legends()`

## Installation

```{r eval=FALSE}
install.packages("ggguides")
```

Or the development version from GitHub:

```{r eval=FALSE}
# install.packages("pak")
pak::pak("gcol33/ggguides")
```

## Basic Usage

Let's create a simple plot to demonstrate the legend helpers:
```{r base-plot}
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p
```

### Moving the Legend

Position the legend on any side with a single function call:

```{r position-left}
p + legend_left()
```

```{r position-bottom}
p + legend_bottom()
```

### Inside Positioning

Place the legend inside the plot area using coordinates or shortcuts:

```{r inside-topright}
p + legend_inside(position = "topright")
```

```{r inside-coords}
p + legend_inside(x = 0.02, y = 0.98, justification = c("left", "top"))
```

### Removing the Legend

```{r legend-none}
p + legend_none()
```

## Styling Legends

Use `legend_style()` to customize the legend appearance:

```{r style-size}
p + legend_style(size = 14)
```

```{r style-full}
p + legend_style(
  size = 12,
  title_size = 14,
  title_face = "bold",
  background = "#FFF3E0",
  background_color = "#FF9800"
)
```

## Wrapping Legend Entries

For legends with many entries, use `legend_wrap()` to create multi-column layouts:

```{r wrap-example}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2)
```

## Combining Functions

ggguides functions compose naturally with the `+` operator:

```{r combined}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_left() +
  legend_style(size = 12, title_face = "bold", background = "#FFF3E0")
```

```{r combined-wrap}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2) +
  legend_bottom()
```

## What's Next

- See the [Legend Positioning](positioning.html) article for detailed position control

- See the [Styling & Customization](styling.html) article for advanced styling options

- See the [Patchwork Integration](patchwork.html) article for multi-panel workflows

---
title: "Multiple Legends"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Multiple Legends}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

When a plot maps multiple aesthetics (colour, size, shape, etc.), ggplot2 creates separate legends for each. ggguides provides functions to control these legends individually:

- **Hide specific legends** with `legend_hide()`

- **Keep only certain legends** with `legend_select()`

- **Control display order** with `legend_order_guides()`

- **Force merge/split** with `legend_merge()` and `legend_split()`

- **Position legends separately** using `by` parameter on position functions

- **Style legends separately** using `by` parameter on `legend_style()`

## Example Plot

```{r example-plot}
# Plot with multiple aesthetics
p <- ggplot(mtcars, aes(mpg, wt,
                        color = factor(cyl),
                        size = hp,
                        shape = factor(am))) +
  geom_point() +
  labs(color = "Cylinders", size = "Horsepower", shape = "Transmission")

p
```

## Hiding Legends

Use `legend_hide()` to remove specific legends while keeping others:

```{r hide-legends}
# Hide the size legend
p + legend_hide(size)

# Hide multiple legends
p + legend_hide(size, shape)
```

## Selecting Legends

Use `legend_select()` to keep only certain legends (inverse of `legend_hide()`):

```{r select-legends}
# Keep only the colour legend
p + legend_select(colour)

# Keep colour and shape
p + legend_select(colour, shape)
```

## Controlling Legend Order

By default, legends appear in an unspecified order. Use `legend_order_guides()` to control the display order:

```{r order-legends}
# Default order
p

# Size legend first, then colour, then shape
p + legend_order_guides(size = 1, colour = 2, shape = 3)
```

## Merging and Splitting Legends

ggplot2 automatically merges legends when they have the same title and matching labels. Use `legend_merge()` and `legend_split()` to override this behavior.

### Forcing Merge

```{r merge-legends}
# Plot where colour and fill map to the same variable
p_merge <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(shape = 21, size = 4, stroke = 1.5) +
  labs(color = "Cylinders", fill = "Cylinders")

# Legends merge automatically when titles and labels match
p_merge

# Explicitly request merge (reinforces default behavior)
p_merge + legend_merge(colour, fill)
```

### Forcing Split

```{r split-legends}
# Force separate legends even when they could merge
p_merge + legend_split(colour, fill)
```

## Positioning Legends Separately

Position functions (`legend_left()`, `legend_right()`, `legend_top()`, `legend_bottom()`) accept a `by` parameter to position specific legends:

```{r position-separately}
# Place colour legend on the left, size legend at bottom
p +
  legend_hide(shape) +
  legend_left(by = "colour") +
  legend_bottom(by = "size")
```

```{r position-top-right}
# Colour legend on top, size on right
p +
  legend_hide(shape) +
  legend_top(by = "colour") +
  legend_right(by = "size")
```

## Styling Legends Separately

Use the `by` parameter on `legend_style()` to apply different styles to different legends:

```{r style-separately}
p +
  legend_hide(shape) +
  legend_style(title_face = "bold", background = "grey95", by = "colour") +
  legend_style(size = 10, by = "size")
```

## Combining Multiple Controls

All functions work together:

```{r combined}
# Complex example: hide shape, position colour on left with bold title,
# position size at bottom with smaller text
p +
  legend_hide(shape) +
  legend_left(by = "colour") +
  legend_style(title_face = "bold", title_size = 14, by = "colour") +
  legend_bottom(by = "size") +
  legend_style(size = 9, direction = "horizontal", by = "size")
```

## Four Legends, One per Side

When a plot has four legends and you want one on each side (top, bottom, left, right), you can fine-tune each legend along three axes:

1. **Side placement** — `legend_top/bottom/left/right(by = "<aes>")`

2. **Distance from the panel** — `legend_style(by = "<aes>", margin = c(t, r, b, l))`

3. **Slide along the side** — `legend_style(by = "<aes>", justification = ...)`

For top/bottom legends, `justification` is `"left"`, `"center"`, `"right"` (or a number in `[0, 1]`). For left/right legends, it's `"top"`, `"center"`, `"bottom"` (or a number).

```{r four-sides, fig.width = 8, fig.height = 6}
p4 <- ggplot(mtcars, aes(mpg, wt,
                         colour = factor(cyl),
                         fill   = factor(gear),
                         size   = hp,
                         shape  = factor(am))) +
  geom_point(stroke = 1.2) +
  labs(colour = "Cyl", fill = "Gear", size = "HP", shape = "AM")

p4 +
  # 1. Send each legend to its side
  legend_top   (by = "colour") +
  legend_bottom(by = "fill")   +
  legend_left  (by = "size")   +
  legend_right (by = "shape")  +

  # 2. Slide each legend along its side
  legend_style(by = "colour", justification = "left") +
  legend_style(by = "fill",   justification = "right") +
  legend_style(by = "size",   justification = "top") +
  legend_style(by = "shape",  justification = "bottom") +

  # 3. Nudge each legend toward/away from the panel via margin (cm)
  legend_style(by = "colour", margin = c(0, 0, 0.3, 0)) +
  legend_style(by = "fill",   margin = c(0.3, 0, 0, 0)) +
  legend_style(by = "size",   margin = c(0, 0.3, 0, 0)) +
  legend_style(by = "shape",  margin = c(0, 0, 0, 0.3))
```

Each `legend_style(by = ...)` call is additive — you can chain as many as you need to tune one legend at a time without affecting the others.

## Six Legends, Stacked per Side

More than one legend can share a side. ggplot2 stacks them in the order it resolves them, and ggguides lets you pick which legend goes where, slide it along the rail, and style it without touching the others. The plot below has **six** legends: two on top, two on the right, one on the bottom, one on the left.

```{r six-legends, fig.width = 9, fig.height = 6.5}
p6 <- ggplot(mtcars, aes(mpg, wt)) +
  geom_smooth(aes(linetype = factor(vs)), method = "lm", se = FALSE,
              colour = "grey40") +
  geom_point(aes(colour = factor(cyl),
                 fill   = factor(gear),
                 size   = hp,
                 alpha  = qsec,
                 shape  = factor(am)),
             stroke = 1.2) +
  scale_shape_manual(values = c(21, 24)) +
  labs(colour = "Cyl", fill = "Gear", size = "HP",
       alpha = "QSec", shape = "AM", linetype = "VS")

p6 +
  # 1. Side placement — two legends share the top, two share the right
  legend_top   (by = "colour")   +
  legend_top   (by = "fill")     +
  legend_right (by = "size")     +
  legend_right (by = "alpha")    +
  legend_bottom(by = "shape")    +
  legend_left  (by = "linetype") +

  # 2. Slide each legend along its rail
  #    top/bottom rails: "left" / "center" / "right" (or a number in [0,1])
  #    left/right rails: "top"  / "center" / "bottom"
  legend_style(by = "colour",   justification = "left")   +
  legend_style(by = "fill",     justification = "right")  +
  legend_style(by = "size",     justification = "top")    +
  legend_style(by = "alpha",    justification = "bottom") +
  legend_style(by = "shape",    justification = "center") +
  legend_style(by = "linetype", justification = "center") +

  # 3. Appearance — per-legend title weight, key size, direction, text size
  legend_style(by = "colour",   title_face = "bold",
               key_width = 0.4, key_height = 0.4) +
  legend_style(by = "fill",     title_face = "bold",
               key_width = 0.4, key_height = 0.4) +
  legend_style(by = "size",     title_size = 9, size = 8)  +
  legend_style(by = "alpha",    title_size = 9, size = 8)  +
  legend_style(by = "shape",    direction = "horizontal")  +
  legend_style(by = "linetype", direction = "vertical")    +

  # 4. Nudge each legend toward/away from the panel via margin (cm)
  #    order is c(top, right, bottom, left)
  legend_style(by = "colour",   margin = c(0, 0, 0.2, 0)) +
  legend_style(by = "fill",     margin = c(0, 0, 0.2, 0)) +
  legend_style(by = "size",     margin = c(0, 0, 0, 0.3)) +
  legend_style(by = "alpha",    margin = c(0, 0, 0, 0.3)) +
  legend_style(by = "shape",    margin = c(0.3, 0, 0, 0)) +
  legend_style(by = "linetype", margin = c(0, 0.3, 0, 0))
```

### What each step is doing

**Step 1, side placement.** `legend_<side>(by = "<aes>")` sends one legend to one side. Call it twice with different aesthetics and both legends land on that side. In the example, `colour` and `fill` share the top rail, `size` and `alpha` share the right rail, and `shape` and `linetype` each get a side of their own.

**Step 2, slide along the rail.** `legend_style(by = "<aes>", justification = ...)` repositions a single legend without affecting its neighbours. The keyword interpretation depends on which rail the legend sits on:

- Top or bottom rail (horizontal): `"left"`, `"center"`, `"right"`, or a number in `[0, 1]` where 0 is flush left and 1 is flush right. In the example, `colour` sits at the left end of the top rail and `fill` at the right end, so the two top legends land in opposite corners.

- Left or right rail (vertical): `"top"`, `"center"`, `"bottom"`, or a number where 0 is bottom and 1 is top. `size` sits at the top of the right rail and `alpha` at the bottom.

The per-guide form is what you want when different legends need different alignments. For a single alignment applied to every legend at once, use `legend_style(justification = ...)` without `by`.

**Step 3, appearance.** Anything `legend_style()` accepts (`title_face`, `title_size`, `size` for label text, `key_width`, `key_height`, `direction`, `background`, and so on) can be scoped to a single legend by adding `by = "<aes>"`. Each call tunes one legend and leaves the others alone, so chaining several short calls is the intended shape. In the example the top legends get bold titles and smaller keys, the right legends get smaller text, `shape` is forced horizontal so it fits the bottom rail, and `linetype` stays vertical for the left rail.

**Step 4, padding.** `margin = c(top, right, bottom, left)` (in cm) moves a single legend toward or away from the panel. Pad on the side that faces the panel: a bottom legend uses `c(top, 0, 0, 0)`, a left legend uses `c(0, right, 0, 0)`. Without padding, legends often end up pressed against the axis text; a few tenths of a centimetre usually fixes it.

### Why one line per parameter

`legend_style(by = "<aes>", ...)` calls are additive. You can fold every style argument for `colour` into a single call, but splitting by concern (position, alignment, appearance, padding) keeps each parameter on a line that is easy to locate. When a reviewer asks to move the colour legend half a centimetre down, the edit is one number on one line.

## Summary

| Function | Purpose | Parameters |
|----------|---------|------------|
| `legend_hide()` | Hide specific legends | Aesthetic names (unquoted) |
| `legend_select()` | Keep only specific legends | Aesthetic names (unquoted) |
| `legend_order_guides()` | Control legend display order | Named args: `aes = order` |
| `legend_merge()` | Force legends to merge | Aesthetic names (unquoted) |
| `legend_split()` | Force legends to stay separate | Aesthetic names (unquoted) |
| `legend_left(by=)` | Position one legend on left | `by = "aesthetic"` |
| `legend_style(by=)` | Style one legend | `by = "aesthetic"` + style args |

**Learn more:**

- [Legend Positioning](positioning.html) for single-legend placement

- [Styling & Customization](styling.html) for legend appearance

- [Patchwork Integration](patchwork.html) for multi-panel plots

---
title: "Styling & Customization"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Styling & Customization}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

ggguides provides styling functions to customize legend appearance without
diving into ggplot2's theme element hierarchy. The main functions are:

- `legend_style()` - Comprehensive styling (fonts, backgrounds, borders)

- `legend_keys()` - Override key appearance (size, alpha, shape)

- `legend_order()` - Reorder legend entries

- `legend_wrap()` - Multi-column/row layouts

- `legend_reverse()` - Reverse entry order

- `colorbar_style()` - Customize continuous color bar legends

## Font Styling

### Font Size

Adjust the overall text size (applies to both title and labels):

```{r style-size, fig.show='hold', out.width='48%'}
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p + ggtitle("Default size")
p + legend_style(size = 14) + ggtitle("size = 14")
```

### Font Family

Change the font family for legend text:

```{r style-family, fig.show='hold', out.width='48%'}
p + legend_style(family = "serif") + ggtitle("serif")
p + legend_style(family = "mono") + ggtitle("mono")
```

### Title Emphasis

Make the title stand out with separate size and face settings:

```{r style-title}
p + legend_style(
  size = 12,
  title_size = 14,
  title_face = "bold"
)
```

## Text Rotation

For legends with long category names, rotate the labels to save space:

### Basic Rotation

```{r rotation-basic, fig.width=7}
p_long <- ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point()

p_long + legend_style(angle = 45)
```

### Vertical Labels

Use 90 degrees for fully vertical text. Key height is automatically adjusted to
prevent label overlap:

```{r rotation-vertical, fig.width=7}
p_long + legend_style(angle = 90)
```

### Title Rotation

Rotate the legend title independently:

```{r rotation-title, fig.show='hold', out.width='48%'}
p_long + legend_style(title_angle = 90, title_hjust = 0.5) + ggtitle("Rotated title only")
p_long + legend_style(angle = 45, title_angle = 90) + ggtitle("Both rotated")
```

## Background and Border

Add visual containers around the legend:

```{r style-background}
p + legend_style(
  background = "#FFF3E0"
)
```

```{r style-border}
p + legend_style(
  background = "#FFF3E0",
  background_color = "#FF9800"
)
```

## Key Size

Adjust the size of legend keys (color swatches):

```{r style-key}
p + legend_style(
  key_width = 1.5,
  key_height = 1.5
)
```

## Margin

Control spacing around the legend:

```{r style-margin}
p + legend_style(
  background = "#FFF3E0",
  margin = 0.5
)
```

## Full Styling Example

Combine all styling options:

```{r style-full}
p + legend_style(
  size = 11,
  title_size = 13,
  title_face = "bold",
  family = "sans",
  key_width = 1.2,
  background = "#FFF3E0",
  background_color = "#FF9800",
  margin = 0.3
)
```

## Wrapping Legend Entries

For legends with many entries, `legend_wrap()` creates multi-column or
multi-row layouts.

### By Columns

```{r wrap-ncol, fig.width=7}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2)
```

### By Rows

```{r wrap-nrow, fig.width=7}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(nrow = 2)
```

### With Bottom Position

Wrapping works well with horizontal legends:

```{r wrap-bottom, fig.width=7}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(nrow = 2) +
  legend_bottom()
```

## Customizing Legend Keys

When plot aesthetics like small point sizes or low alpha values make legend
keys hard to read, `legend_keys()` overrides the key appearance without
affecting the plot.

### Enlarging Small Points

```{r keys-size, fig.show='hold', out.width='48%'}
p_small <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 1) +
  labs(color = "Cylinders")

p_small + ggtitle("Small points in legend")
p_small + legend_keys(size = 4) + ggtitle("Enlarged legend keys")
```

### Removing Transparency

```{r keys-alpha, fig.show='hold', out.width='48%'}
p_alpha <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(alpha = 0.3, size = 3) +
  labs(color = "Cylinders")

p_alpha + ggtitle("Transparent legend keys")
p_alpha + legend_keys(alpha = 1) + ggtitle("Opaque legend keys")
```

### Combining Overrides

```{r keys-combined}
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(alpha = 0.3, size = 1) +
  legend_keys(size = 4, alpha = 1)
```

### Targeting Specific Aesthetics

By default, `legend_keys()` applies to both colour and fill legends.
Target specific aesthetics with the `aesthetic` parameter:

```{r keys-fill}
ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
  geom_boxplot(alpha = 0.5) +
  legend_keys(alpha = 1, aesthetic = "fill")
```

### Changing Symbol Shapes

Use named shapes for clarity instead of numeric codes:

```{r keys-shape, fig.show='hold', out.width='48%'}
p + legend_keys(shape = "square") + ggtitle("Square")
p + legend_keys(shape = "diamond") + ggtitle("Diamond")
```

Available shape names include: `"circle"`, `"square"`, `"diamond"`, `"triangle"`,
`"plus"`, `"cross"`, `"asterisk"`.

### Filled Shapes with Outlines

Shapes 21-25 (or names ending in `_filled`) support both outline and fill colors,
making legends more visible against any background:

```{r keys-outline, fig.show='hold', out.width='48%'}
# White fill with colored outline - works with color mapping only
p + legend_keys(shape = "circle_filled", fill = "white", stroke = 1.5) +
  ggtitle("White fill, colored outline")

# Colored fill with black outline - requires mapping BOTH color and fill
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(size = 3, shape = 21, stroke = 1) +
  legend_keys(colour = "black", stroke = 1) +
  ggtitle("Colored fill, black outline")
```

Note: For colored fills with a custom outline, you must map both `color` and `fill`
in the plot aesthetics. This is a ggplot2 limitation—`override.aes` can only set
static values, it cannot make fill inherit from color.

Shape types:

| Type | Shapes | Fill from | Outline from |
|------|--------|-----------|--------------|
| Outline only | `"circle_open"`, `"square_open"`, `"diamond_open"` | N/A | `colour` |
| Solid filled | `"circle"`, `"square"`, `"diamond"` | `colour` | N/A |
| Fill + outline | `"circle_filled"`, `"square_filled"`, `"diamond_filled"` | `fill` | `colour` |

## Reordering Legend Entries

`legend_order()` changes the order of legend entries without modifying
factor levels in your data.

### Explicit Order

Specify the exact order you want:

```{r order-explicit}
p + legend_order(c("8", "6", "4"))
```

### Using Functions

Apply functions like `rev` or `sort` to the current order:

```{r order-rev, fig.show='hold', out.width='48%'}
p + legend_order(rev) + ggtitle("Reversed")
p + legend_order(sort) + ggtitle("Sorted")
```

### Other Aesthetics

Reorder legends for fill, shape, or other aesthetics:

```{r order-fill}
ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
  geom_bar() +
  legend_order(c("8", "4", "6"), aesthetic = "fill")
```

## Reversing Legend Order

For simple reversal, `legend_reverse()` is a convenient shorthand:

```{r reverse}
p + legend_reverse()
```

## Combining Style Functions

All ggguides functions compose with `+`:

```{r combined-styling, fig.width=7}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_left() +
  legend_style(
    size = 11,
    title_face = "bold",
    background = "#FFF3E0"
  )
```

```{r combined-wrap-style, fig.width=7}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2) +
  legend_bottom() +
  legend_style(size = 10, title_face = "bold")
```

## Styling Continuous Color Bars

For continuous scales, `colorbar_style()` customizes the color bar appearance.

### Basic Usage

```{r colorbar-basic}
p_cont <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
  geom_tile()

p_cont
```

### Size Adjustments

Create taller, thinner bars:

```{r colorbar-size}
p_cont + colorbar_style(width = 0.5, height = 10, aesthetic = "fill")
```

### Horizontal Orientation

```{r colorbar-horizontal}
p_cont + colorbar_style(width = 10, height = 0.5, direction = "horizontal", aesthetic = "fill") +
  legend_bottom()
```
### Adding a Frame

```{r colorbar-frame, fig.show='hold', out.width='48%'}
p_cont + colorbar_style(frame = TRUE, aesthetic = "fill") + ggtitle("Black frame")
p_cont + colorbar_style(frame = "#FF9800", aesthetic = "fill") + ggtitle("Orange frame")
```

### Removing Ticks

```{r colorbar-noticks}
p_cont + colorbar_style(ticks = FALSE, frame = "#FF9800", aesthetic = "fill")
```

### Combined Customization

```{r colorbar-full}
p_cont + colorbar_style(
  width = 0.5,
  height = 8,
  frame = "#E65100",
  ticks_length = 0.3,
  aesthetic = "fill"
)
```

## Summary

| Function | Purpose | Key Parameters |
|----------|---------|----------------|
| `legend_style()` | Comprehensive styling | `size`, `title_size`, `title_face`, `family`, `angle`, `background`, `margin` |
| `legend_keys()` | Override key appearance | `size`, `alpha`, `shape`, `fill`, `colour`, `stroke`, `aesthetic` |
| `legend_order()` | Reorder entries | `order` (vector or function), `aesthetic` |
| `legend_wrap()` | Multi-column layout | `ncol`, `nrow` |
| `legend_reverse()` | Reverse entry order | None |
| `colorbar_style()` | Continuous color bar | `width`, `height`, `frame`, `ticks`, `direction` |

**Learn more:**

- [Legend Positioning](positioning.html) for placement control

- [Patchwork Integration](patchwork.html) for multi-panel workflows

---
title: "Getting Started with ggguides"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with ggguides}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

**ggguides** simplifies common legend operations in ggplot2. Instead of memorizing
theme element names and their nested structure, you can use intuitive one-liner
functions to position, style, and manage legends.

**Key features:**

- Position legends with `legend_left()`, `legend_right()`, `legend_top()`, `legend_bottom()`, `legend_inside()`

- Style legends with `legend_style()` for fonts, backgrounds, and borders

- Wrap legend entries with `legend_wrap()` for multi-column layouts

- Collect legends from patchwork compositions with `collect_legends()`

## Installation

```{r eval=FALSE}
# install.packages("pak")
pak::pak("gcol33/ggguides")
```

## Basic Usage

Let's create a simple plot to demonstrate the legend helpers:
```{r base-plot}
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p
```

### Moving the Legend

Position the legend on any side with a single function call:

```{r position-left}
p + legend_left()
```

```{r position-bottom}
p + legend_bottom()
```

### Inside Positioning

Place the legend inside the plot area using coordinates or shortcuts:

```{r inside-topright}
p + legend_inside(position = "topright")
```

```{r inside-coords}
p + legend_inside(x = 0.02, y = 0.98, just = c("left", "top"))
```

### Removing the Legend

```{r legend-none}
p + legend_none()
```

## Styling Legends

Use `legend_style()` to customize the legend appearance:

```{r style-size}
p + legend_style(size = 14)
```

```{r style-full}
p + legend_style(
  size = 12,
  title_size = 14,
  title_face = "bold",
  background = "#FFF3E0",
  background_color = "#FF9800"
)
```

## Wrapping Legend Entries

For legends with many entries, use `legend_wrap()` to create multi-column layouts:

```{r wrap-example}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2)
```

## Combining Functions

ggguides functions compose naturally with the `+` operator:

```{r combined}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_left() +
  legend_style(size = 12, title_face = "bold", background = "#FFF3E0")
```

```{r combined-wrap}
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2) +
  legend_bottom()
```

## What's Next

- See the [Legend Positioning](positioning.html) article for detailed position control

- See the [Styling & Customization](styling.html) article for advanced styling options

- See the [Patchwork Integration](patchwork.html) article for multi-panel workflows

---
title: "Legend Positioning"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Legend Positioning}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

ggguides provides position helpers that handle legend placement without
requiring knowledge of ggplot2's theme system. Each function sets the
appropriate combination of `legend.position`, `legend.justification`,
and `legend.box.just` to achieve proper alignment.

## Basic Positioning

### Left and Right

```{r left-right, fig.show='hold', out.width='48%'}
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  labs(color = "Cylinders")

p + legend_left() + ggtitle("Left")
p + legend_right() + ggtitle("Right")
```

### Top and Bottom

When placing legends at the top or bottom, the legend is automatically
oriented horizontally:

```{r top-bottom, fig.show='hold', out.width='48%'}
p + legend_top() + ggtitle("Top")
p + legend_bottom() + ggtitle("Bottom")
```

### Alignment with Titles

Use the `align_to` parameter to control how the legend aligns with the plot.
The difference is most visible when there's a y-axis label:

```{r align-to, fig.show='hold', out.width='48%'}
p_labeled <- p + labs(title = "Fuel Economy", y = "Weight (1000 lbs)")

# Default: align to panel (legend aligns with the plot area)
p_labeled + legend_top() + ggtitle("Panel alignment (default)")

# Align to full plot (legend spans title/axis labels too)
p_labeled + legend_top(align_to = "plot") + ggtitle("Plot alignment")
```

## Inside Positioning

`legend_inside()` places the legend within the plot panel. You can use
either coordinate values or named shortcuts.

### Using Shortcuts

Available shortcuts: `"topright"`, `"topleft"`, `"bottomright"`, `"bottomleft"`, `"center"`

```{r inside-shortcuts, fig.show='hold', out.width='48%'}
p + legend_inside(position = "topright") + ggtitle("Top right")
p + legend_inside(position = "bottomleft") + ggtitle("Bottom left")
```

### Using Coordinates

For precise control, specify x/y coordinates (0-1 scale) and justification:

```{r inside-coords}
p + legend_inside(x = 0.95, y = 0.95, just = c("right", "top"))
```

### Adding Background

Inside legends often need a background for readability:

```{r inside-background}
p + legend_inside(
  position = "topright",
  background = "#FFF3E0",
  border = "#FF9800"
)
```

## Direction Control

### Horizontal and Vertical

Control legend entry direction independently of position:

```{r direction, fig.show='hold', out.width='48%'}
p + legend_horizontal() + ggtitle("Horizontal")
p + legend_vertical() + ggtitle("Vertical")
```

### Combining Direction and Position

```{r direction-position}
p + legend_right() + legend_horizontal()
```

## Removing the Legend

```{r legend-none}
p + legend_none()
```

## Summary

| Function | Position | Auto-Direction |
|----------|----------|----------------|
| `legend_left()` | Left of panel | Vertical |
| `legend_right()` | Right of panel | Vertical |
| `legend_top()` | Above panel | Horizontal |
| `legend_bottom()` | Below panel | Horizontal |
| `legend_inside()` | Inside panel | Unchanged |
| `legend_none()` | Hidden | N/A |

**Learn more:**

- [Styling & Customization](styling.html) for font, background, and border options

- [Patchwork Integration](patchwork.html) for multi-panel workflows

---
title: "Multiple Legends"
author: "Gilles Colling"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Multiple Legends}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
library(ggguides)
library(ggplot2)

# Theme with transparent backgrounds for pkgdown light/dark mode
theme_set(
  theme_grey() +
    theme(
      plot.background = element_rect(fill = "transparent", color = NA),
      panel.background = element_rect(fill = "transparent", color = NA),
      legend.background = element_rect(fill = "transparent", color = NA),
      legend.key = element_rect(fill = "transparent", color = NA),
      legend.box.background = element_rect(fill = "transparent", color = NA)
    )
)
```

## Overview

When a plot maps multiple aesthetics (colour, size, shape, etc.), ggplot2 creates separate legends for each. ggguides provides functions to control these legends individually:

- **Hide specific legends** with `legend_hide()`

- **Keep only certain legends** with `legend_select()`

- **Control display order** with `legend_order_guides()`

- **Force merge/split** with `legend_merge()` and `legend_split()`

- **Position legends separately** using `by` parameter on position functions

- **Style legends separately** using `by` parameter on `legend_style()`

## Example Plot

```{r example-plot}
# Plot with multiple aesthetics
p <- ggplot(mtcars, aes(mpg, wt,
                        color = factor(cyl),
                        size = hp,
                        shape = factor(am))) +
  geom_point() +
  labs(color = "Cylinders", size = "Horsepower", shape = "Transmission")

p
```

## Hiding Legends

Use `legend_hide()` to remove specific legends while keeping others:

```{r hide-legends}
# Hide the size legend
p + legend_hide(size)

# Hide multiple legends
p + legend_hide(size, shape)
```

## Selecting Legends

Use `legend_select()` to keep only certain legends (inverse of `legend_hide()`):

```{r select-legends}
# Keep only the colour legend
p + legend_select(colour)

# Keep colour and shape
p + legend_select(colour, shape)
```

## Controlling Legend Order

By default, legends appear in an unspecified order. Use `legend_order_guides()` to control the display order:

```{r order-legends}
# Default order
p

# Size legend first, then colour, then shape
p + legend_order_guides(size = 1, colour = 2, shape = 3)
```

## Merging and Splitting Legends

ggplot2 automatically merges legends when they have the same title and matching labels. Use `legend_merge()` and `legend_split()` to override this behavior.

### Forcing Merge

```{r merge-legends}
# Plot where colour and fill map to the same variable
p_merge <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(shape = 21, size = 4, stroke = 1.5) +
  labs(color = "Cylinders", fill = "Cylinders")

# Legends merge automatically when titles and labels match
p_merge

# Explicitly request merge (reinforces default behavior)
p_merge + legend_merge(colour, fill)
```

### Forcing Split

```{r split-legends}
# Force separate legends even when they could merge
p_merge + legend_split(colour, fill)
```

## Positioning Legends Separately

Position functions (`legend_left()`, `legend_right()`, `legend_top()`, `legend_bottom()`) accept a `by` parameter to position specific legends:

```{r position-separately}
# Place colour legend on the left, size legend at bottom
p +
  legend_hide(shape) +
  legend_left(by = "colour") +
  legend_bottom(by = "size")
```

```{r position-top-right}
# Colour legend on top, size on right
p +
  legend_hide(shape) +
  legend_top(by = "colour") +
  legend_right(by = "size")
```

## Styling Legends Separately

Use the `by` parameter on `legend_style()` to apply different styles to different legends:

```{r style-separately}
p +
  legend_hide(shape) +
  legend_style(title_face = "bold", background = "grey95", by = "colour") +
  legend_style(size = 10, by = "size")
```

## Four Legends, One per Side

When a plot has four legends and you want one on each side (top, bottom, left, right), you can fine-tune each legend along three axes:

1. **Side placement** — `legend_top/bottom/left/right(by = "<aes>")`
2. **Distance from the panel** — `legend_style(by = "<aes>", margin = c(t, r, b, l))`
3. **Slide along the side** — `legend_style(by = "<aes>", justification = ...)`

For top/bottom legends, `justification` is `"left"`, `"center"`, `"right"` (or a number in `[0, 1]`). For left/right legends, it's `"top"`, `"center"`, `"bottom"` (or a number).

```{r four-sides, fig.width = 8, fig.height = 6}
p4 <- ggplot(mtcars, aes(mpg, wt,
                         colour = factor(cyl),
                         fill   = factor(gear),
                         size   = hp,
                         shape  = factor(am))) +
  geom_point(stroke = 1.2) +
  labs(colour = "Cyl", fill = "Gear", size = "HP", shape = "AM")

p4 +
  # 1. Send each legend to its side
  legend_top   (by = "colour") +
  legend_bottom(by = "fill")   +
  legend_left  (by = "size")   +
  legend_right (by = "shape")  +

  # 2. Slide each legend along its side
  legend_style(by = "colour", justification = "left") +
  legend_style(by = "fill",   justification = "right") +
  legend_style(by = "size",   justification = "top") +
  legend_style(by = "shape",  justification = "bottom") +

  # 3. Nudge each legend toward/away from the panel via margin (cm)
  legend_style(by = "colour", margin = c(0, 0, 0.3, 0)) +
  legend_style(by = "fill",   margin = c(0.3, 0, 0, 0)) +
  legend_style(by = "size",   margin = c(0, 0.3, 0, 0)) +
  legend_style(by = "shape",  margin = c(0, 0, 0, 0.3))
```

Each `legend_style(by = ...)` call is additive — you can chain as many as you need to tune one legend at a time without affecting the others.

## Combining Multiple Controls

All functions work together:

```{r combined}
# Complex example: hide shape, position colour on left with bold title,
# position size at bottom with smaller text
p +
  legend_hide(shape) +
  legend_left(by = "colour") +
  legend_style(title_face = "bold", title_size = 14, by = "colour") +
  legend_bottom(by = "size") +
  legend_style(size = 9, direction = "horizontal", by = "size")
```

## Summary

| Function | Purpose | Parameters |
|----------|---------|------------|
| `legend_hide()` | Hide specific legends | Aesthetic names (unquoted) |
| `legend_select()` | Keep only specific legends | Aesthetic names (unquoted) |
| `legend_order_guides()` | Control legend display order | Named args: `aes = order` |
| `legend_merge()` | Force legends to merge | Aesthetic names (unquoted) |
| `legend_split()` | Force legends to stay separate | Aesthetic names (unquoted) |
| `legend_left(by=)` | Position one legend on left | `by = "aesthetic"` |
| `legend_style(by=)` | Style one legend | `by = "aesthetic"` + style args |

**Learn more:**

- [Legend Positioning](positioning.html) for single-legend placement

- [Styling & Customization](styling.html) for legend appearance

- [Patchwork Integration](patchwork.html) for multi-panel plots

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{center_legend_title}
\alias{center_legend_title}
\title{Center Legend Title Over Keys}
\usage{
center_legend_title(plot, position = "all")
}
\arguments{
\item{plot}{A ggplot object.}

\item{position}{Legend position to modify. One of \code{"right"}, \code{"left"},
\code{"top"}, \code{"bottom"}, or \code{"all"} (default).}
}
\value{
A modified gtable object that can be drawn with \code{grid::grid.draw()}
or saved with \code{ggplot2::ggsave()}.
}
\description{
Modifies a ggplot so that legend titles are centered over the key column
only, rather than over the full legend width (keys + labels). This is
particularly useful when legend labels are rotated, as the default centering
places the title too far to the right.
}
\details{
This function works by modifying the legend's internal gtable structure,
restricting the title's column span to only the keys column. Long titles
will automatically wrap to fit within the key column width, and proper
spacing is added to prevent overlap with rotated labels.

The title should have \code{hjust = 0.5} set (done automatically by
\code{legend_style(angle = ...)}) for proper centering.
}
\examples{
library(ggplot2)

p <- ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_style(angle = 45) +
  labs(color = "Vehicle Class")

# Center title over keys only (long titles wrap automatically)
# Returns a gtable - use grid::grid.draw() to render
g <- center_legend_title(p)
grid::grid.draw(g)

}
\seealso{
\code{\link{legend_style}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{legend_style}
\alias{legend_style}
\title{Style Legend Appearance}
\usage{
legend_style(
  size = NULL,
  family = NULL,
  face = NULL,
  color = NULL,
  angle = NULL,
  title_size = NULL,
  title_face = NULL,
  title_color = NULL,
  title_angle = NULL,
  title_hjust = NULL,
  title_vjust = NULL,
  title_position = NULL,
  key_width = NULL,
  key_height = NULL,
  key_fill = NULL,
  spacing = NULL,
  spacing_x = NULL,
  spacing_y = NULL,
  margin = NULL,
  background = NULL,
  background_color = NULL,
  box_background = NULL,
  box_margin = NULL,
  direction = NULL,
  byrow = NULL,
  justification = NULL,
  by = NULL
)
}
\arguments{
\item{size}{Text size for legend labels (in points).}

\item{family}{Font family for legend text.}

\item{face}{Font face for legend text (\code{"plain"}, \code{"bold"},
\code{"italic"}, \code{"bold.italic"}).}

\item{color}{Text color for legend labels.}

\item{angle}{Rotation angle for legend labels (in degrees). Supported values:
45, -45, 90, -90. Text justification is set automatically for optimal
alignment with legend keys.}

\item{title_size}{Text size for legend title (in points). If \code{NULL},
inherits from \code{size}.}

\item{title_face}{Font face for legend title. If \code{NULL}, inherits from
\code{face}.}

\item{title_color}{Text color for legend title. If \code{NULL}, inherits from
\code{color}.}

\item{title_angle}{Rotation angle for legend title (in degrees).}

\item{title_hjust}{Horizontal justification for rotated title.}

\item{title_vjust}{Vertical justification for rotated title.}

\item{title_position}{Position of legend title relative to keys. One of
\code{"top"}, \code{"bottom"}, \code{"left"}, \code{"right"}.}

\item{key_width}{Width of legend keys. Numeric (in cm) or a \code{unit} object.}

\item{key_height}{Height of legend keys. Numeric (in cm) or a \code{unit} object.}

\item{key_fill}{Background fill color for legend keys.}

\item{spacing}{Spacing between legend entries. Numeric (in cm) or a \code{unit}
object.}

\item{spacing_x}{Horizontal spacing between legend entries.}

\item{spacing_y}{Vertical spacing between legend entries.}

\item{margin}{Margin around entire legend. Single value (all sides) or vector
of 4 values (top, right, bottom, left) in cm.}

\item{background}{Legend background fill color. Use \code{NA} for transparent.}

\item{background_color}{Legend background border color. Use \code{NA} for no
border.}

\item{box_background}{Background fill for the box containing multiple legends.
Ignored when \code{by} is specified.}

\item{box_margin}{Margin around the legend box. Single value or 4-vector in cm.
Ignored when \code{by} is specified.}

\item{direction}{Legend direction: \code{"horizontal"} or \code{"vertical"}.}

\item{byrow}{For multi-column legends, fill by row (\code{TRUE}) or by column
(\code{FALSE}).}

\item{justification}{Justification of the legend along its side. For legends
on the top or bottom: \code{"left"}, \code{"center"}, \code{"right"}, or a
numeric value in \code{[0, 1]}. For legends on the left or right:
\code{"top"}, \code{"center"}, \code{"bottom"}, or a numeric value in
\code{[0, 1]}. When \code{by} is specified, applies per-guide via
\code{guide_legend(theme = theme(legend.justification = ...))}. When
\code{by} is NULL, sets \code{legend.justification} for the whole plot.
Requires ggplot2 >= 3.5.0 for per-guide use.}

\item{by}{Optional aesthetic name (character) to style only a specific legend.
When specified, uses per-guide theming via \code{guide_legend(theme = ...)}.
Requires ggplot2 >= 3.5.0. Common values: \code{"colour"}, \code{"fill"},
\code{"size"}.}
}
\value{
A ggplot2 theme object (when \code{by} is NULL) or a guides
specification (when \code{by} is specified).
}
\description{
A comprehensive one-liner to style all legend elements consistently. Instead
of setting multiple theme elements separately, use this function to control
text, title, keys, spacing, background, and direction in one call.
}
\examples{
library(ggplot2)

# Simple: consistent font
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_style(size = 12, family = "serif")

# Styled title and keys
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_style(
    size = 10,
    title_size = 14,
    title_face = "bold",
    key_width = 1.5
  )

# Full styling with background
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_style(
    size = 11,
    title_size = 13,
    title_face = "bold",
    key_fill = "grey95",
    background = "white",
    background_color = "grey80",
    margin = 0.3
  )

# Rotated labels for long category names
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_style(angle = 45)

# Style only the colour legend
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point() +
  legend_style(title_face = "bold", background = "grey95", by = "colour") +
  legend_style(size = 10, by = "size")

# Per-legend justification: slide each legend along its side
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point() +
  legend_top(by = "colour") +
  legend_right(by = "size") +
  legend_style(by = "colour", justification = "left") +
  legend_style(by = "size",   justification = "top")

}
\seealso{
\code{\link{legend_left}}, \code{\link{legend_wrap}},
\code{\link{legend_reverse}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_vertical}
\alias{legend_vertical}
\title{Set Legend Direction to Vertical}
\usage{
legend_vertical()
}
\value{
A ggplot2 theme object that can be added to a plot.
}
\description{
A one-liner to arrange legend keys vertically. This is the default for
legends placed on the left or right of a plot.
}
\examples{
library(ggplot2)

# Explicitly set vertical direction
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_right() +
  legend_vertical()

}
\seealso{
\code{\link{legend_horizontal}}, \code{\link{legend_left}},
\code{\link{legend_right}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cowplot_helpers.R
\name{get_legend}
\alias{get_legend}
\title{Extract Legend from a ggplot}
\usage{
get_legend(plot)
}
\arguments{
\item{plot}{A ggplot object.}
}
\value{
A grob containing the legend, or \code{NULL} if the plot has no
legend.
}
\description{
Extracts the legend (guide-box) from a ggplot object as a grob that can be
used independently with grid or cowplot.
}
\details{
This function is useful for cowplot workflows where you want to manually
position a shared legend. The extracted legend can be combined with plots
using \code{cowplot::plot_grid()} or drawn directly with
\code{grid::grid.draw()}.
}
\examples{
library(ggplot2)

p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  labs(color = "Cylinders")

# Extract the legend
leg <- get_legend(p)

# Draw just the legend
grid::grid.newpage()
grid::grid.draw(leg)

}
\seealso{
\code{\link{shared_legend}}, \code{\link{collect_legends}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_bottom}
\alias{legend_bottom}
\title{Place Legend on Bottom with Horizontal Layout}
\usage{
legend_bottom(align_to = c("panel", "plot"), by = NULL)
}
\arguments{
\item{align_to}{Where to align the legend. Either \code{"panel"} (default,
aligns to plot panel) or \code{"plot"} (aligns to full plot including title).
Requires ggplot2 >= 3.5.0 for \code{"plot"} alignment. Ignored when
\code{by} is specified.}

\item{by}{Optional aesthetic name (character) to position only a specific
legend. When specified, uses per-guide positioning via
\code{guide_legend(position = "bottom")}. Requires ggplot2 >= 3.5.0.
Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.}
}
\value{
A ggplot2 theme object (when \code{by} is NULL) or a guides
specification (when \code{by} is specified).
}
\description{
A one-liner to position the legend below the plot with horizontal layout.
Optionally aligns to the full plot area rather than just the panel.
}
\examples{
library(ggplot2)

# Basic usage
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_bottom()

# Aligned to full plot
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  labs(title = "My Plot Title") +
  legend_bottom(align_to = "plot")

# Position only the colour legend at bottom
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point() +
  legend_bottom(by = "colour") +
  legend_right(by = "size")

}
\seealso{
\code{\link{legend_top}}, \code{\link{legend_left}},
\code{\link{legend_right}}, \code{\link{legend_horizontal}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_keys.R
\name{legend_keys}
\alias{legend_keys}
\title{Customize Legend Key Appearance}
\usage{
legend_keys(
  size = NULL,
  alpha = NULL,
  shape = NULL,
  linewidth = NULL,
  linetype = NULL,
  fill = NULL,
  colour = NULL,
  color = NULL,
  stroke = NULL,
  aesthetic = c("colour", "fill")
)
}
\arguments{
\item{size}{Numeric. Size of point keys.}

\item{alpha}{Numeric. Alpha (transparency) of keys, between 0 and 1.}

\item{shape}{Shape of point keys. Can be:
\itemize{
\item Numeric (0-25): Standard ggplot2 shape codes
\item Character name: \code{"circle"}, \code{"square"}, \code{"diamond"},
\code{"triangle"}, \code{"triangle_down"}, \code{"plus"}, \code{"cross"},
\code{"asterisk"}, \code{"circle_open"}, \code{"square_open"},
\code{"diamond_open"}, \code{"triangle_open"}, \code{"circle_filled"},
\code{"square_filled"}, \code{"diamond_filled"}, \code{"triangle_filled"}
}
Shapes 21-25 (or names ending in \code{"_filled"}) support both outline
(\code{colour}) and fill (\code{fill}) colors.}

\item{linewidth}{Numeric. Width of line keys.}

\item{linetype}{Character or numeric. Line type for line keys.}

\item{fill}{Character. Fill color for filled shapes (shapes 21-25). For
shapes 0-20, use \code{colour} instead.}

\item{colour, color}{Character. Outline/stroke color for keys. For shapes
21-25, this controls the outline; for shapes 0-20, this is the main color.}

\item{stroke}{Numeric. Stroke width for point outlines (shapes 21-25).}

\item{aesthetic}{Character vector specifying which aesthetic(s) to modify.
Default is \code{c("colour", "fill")} which covers most common cases.
Use \code{"all"} to apply to all legends.}
}
\value{
A list of ggplot2 guide specifications.
}
\description{
Modify the appearance of legend keys (the symbols/glyphs in the legend)
without affecting the plot itself. This is a simpler alternative to using
\code{guide_legend(override.aes = list(...))}.
}
\details{
This function wraps \code{guide_legend(override.aes = ...)} to provide a cleaner
interface for common legend key modifications. It's particularly useful for:

\itemize{
\item Making small points more visible in the legend
\item Removing transparency from legend keys
\item Changing symbol shapes to improve clarity
\item Adding outlines to filled shapes for better visibility
}

\strong{Shape types:}
\itemize{
\item Shapes 0-14: Outline only (color from \code{colour})
\item Shapes 15-20: Filled solid (color from \code{colour})
\item Shapes 21-25: Outline + fill (outline from \code{colour}, interior
from \code{fill})
}

\strong{Important note for filled shapes (21-25):}

When using filled shapes with both outline and fill colors, the behavior
depends on which aesthetics are mapped in your original plot:

\itemize{
\item \strong{White fill, colored outline}: Works with \code{aes(color = var)}.
Use \code{legend_keys(shape = "circle_filled", fill = "white")}.
\item \strong{Colored fill, black outline}: Requires \code{aes(color = var, fill = var)}
in your plot. Then use \code{legend_keys(colour = "black")}.
}

This is because \code{override.aes} can only set static values; it cannot
inherit from mapped aesthetics. If you only map \code{color} and try to
override the outline to black, the fill will not have a color mapping to use.

Only non-NULL arguments are applied, so you can selectively modify specific
properties.
}
\examples{
library(ggplot2)

# Points get lost in legend - make them bigger
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 1) +
  legend_keys(size = 4)

# Remove transparency from legend
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(alpha = 0.3, size = 3) +
  legend_keys(alpha = 1)

# Change shape using name
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  legend_keys(shape = "square")

# Filled shape with white fill and colored outline (shapes 21-25)
# Works because we set fill to a static color (white)
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3) +
  legend_keys(shape = "circle_filled", fill = "white", stroke = 1.5)

# Colored fill with black outline - MUST map both color AND fill in the plot
# This is a ggplot2 limitation: override.aes can only set static values,
# it cannot make fill "inherit" from color
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(size = 3, shape = 21, stroke = 1) +
  legend_keys(colour = "black", stroke = 1)

# Apply to fill aesthetic (e.g., for boxplots)
ggplot(mtcars, aes(factor(cyl), mpg, fill = factor(cyl))) +
  geom_boxplot(alpha = 0.5) +
  legend_keys(alpha = 1, aesthetic = "fill")

}
\seealso{
\code{\link{legend_style}} for styling legend text and background,
\code{\link{legend_order}} for reordering legend entries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorbar_style.R
\name{colorbar_style}
\alias{colorbar_style}
\alias{colourbar_style}
\title{Style Continuous Color Bar Legends}
\usage{
colorbar_style(
  width = NULL,
  height = NULL,
  ticks = NULL,
  ticks_length = NULL,
  frame = NULL,
  frame_linewidth = NULL,
  label = NULL,
  label_position = NULL,
  title_position = NULL,
  direction = NULL,
  reverse = NULL,
  nbin = NULL,
  aesthetic = "colour"
)

colourbar_style(
  width = NULL,
  height = NULL,
  ticks = NULL,
  ticks_length = NULL,
  frame = NULL,
  frame_linewidth = NULL,
  label = NULL,
  label_position = NULL,
  title_position = NULL,
  direction = NULL,
  reverse = NULL,
  nbin = NULL,
  aesthetic = "colour"
)
}
\arguments{
\item{width}{Numeric. Width of the color bar in lines. Default is 1.}

\item{height}{Numeric. Height of the color bar in lines. Default is 5.}

\item{ticks}{Logical. Whether to show tick marks. Default is TRUE.}

\item{ticks_length}{Numeric. Length of tick marks in lines. Default is 0.2.}

\item{frame}{Logical or character. If TRUE, draws a black frame. If a color
string, draws a frame in that color. If FALSE or NULL, no frame. Default is FALSE.}

\item{frame_linewidth}{Numeric. Line width of the frame. Default is 0.5.}

\item{label}{Logical. Whether to show labels. Default is TRUE.}

\item{label_position}{Character. Position of labels: "right", "left", "top",
or "bottom". Default depends on bar orientation.}

\item{title_position}{Character. Position of title: "top", "bottom", "left",
or "right". Default is "top".}

\item{direction}{Character. Direction of the bar: "vertical" or "horizontal".
Default is "vertical".}

\item{reverse}{Logical. Whether to reverse the color bar. Default is FALSE.}

\item{nbin}{Integer. Number of bins used to draw the color bar. Higher values
give smoother gradients. Default is 300.}

\item{aesthetic}{Character. Which aesthetic to apply to. Default is "colour".
Common values: "colour", "color", "fill".}
}
\value{
A ggplot2 guides specification.
}
\description{
Customize the appearance of color bar legends for continuous scales.
This provides a simpler interface to \code{guide_colourbar()}.
}
\details{
This function simplifies common color bar customizations:
\itemize{
\item \strong{Size}: Use \code{width} and \code{height} to make thin/wide bars
\item \strong{Ticks}: Toggle with \code{ticks}, adjust length with \code{ticks_length}
\item \strong{Frame}: Add a border with \code{frame = TRUE} or \code{frame = "grey50"}
\item \strong{Orientation}: Use \code{direction = "horizontal"} for horizontal bars
}

The function uses the theme system internally, which is the recommended
approach in ggplot2 3.5.0+.
}
\examples{
library(ggplot2)

p <- ggplot(faithfuld, aes(waiting, eruptions, fill = density)) +
  geom_tile()

# Default appearance
p

# Taller, thinner bar
p + colorbar_style(width = 0.5, height = 10, aesthetic = "fill")

# Wide horizontal bar
p + colorbar_style(width = 10, height = 0.5, direction = "horizontal",
                   aesthetic = "fill")

# With frame and no ticks
p + colorbar_style(frame = "grey50", ticks = FALSE, aesthetic = "fill")

# Thin bar with frame
p + colorbar_style(width = 0.5, height = 8, frame = TRUE, aesthetic = "fill")

}
\seealso{
\code{\link{legend_style}} for styling discrete legends,
\code{\link{legend_left}}, \code{\link{legend_bottom}} for positioning.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{print.gg_autofit_legend}
\alias{print.gg_autofit_legend}
\title{Print method for auto-fit legend plots (90° rotation)}
\usage{
\method{print}{gg_autofit_legend}(x, ...)
}
\arguments{
\item{x}{A gg_autofit_legend object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object. Called for the side effect of
rendering the plot with auto-fitted and centered legend.
}
\description{
Print method for auto-fit legend plots (90° rotation)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_none}
\alias{legend_none}
\title{Remove Legend from Plot}
\usage{
legend_none()
}
\value{
A ggplot2 theme object that can be added to a plot.
}
\description{
A one-liner to remove the legend from a plot. Cleaner alternative to
\code{theme(legend.position = "none")}.
}
\examples{
library(ggplot2)

# Remove legend
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_none()

}
\seealso{
\code{\link{legend_left}}, \code{\link{legend_right}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_multi.R
\name{legend_select}
\alias{legend_select}
\title{Keep Only Specific Legends}
\usage{
legend_select(...)
}
\arguments{
\item{...}{Aesthetic names (unquoted) to keep. All other legends will be
hidden. Common values: \code{colour}, \code{fill}, \code{size},
\code{shape}, \code{linetype}, \code{alpha}.}
}
\value{
A guides specification that can be added to a plot, or \code{NULL}
if nothing needs to be hidden.
}
\description{
Show only the specified legends and hide all others. This is the inverse of
\code{\link{legend_hide}}.
}
\examples{
library(ggplot2)

# Plot with multiple legends
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp,
                         shape = factor(am))) +
  geom_point()

# Keep only the colour legend
p + legend_select(colour)

# Keep colour and shape, hide size
p + legend_select(colour, shape)

}
\seealso{
\code{\link{legend_hide}}, \code{\link{legend_none}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cowplot_helpers.R
\name{shared_legend}
\alias{shared_legend}
\title{Combine Plots with a Shared Legend}
\usage{
shared_legend(
  ...,
  ncol = NULL,
  nrow = NULL,
  position = c("right", "left", "bottom", "top"),
  legend_from = 1,
  rel_legend_size = 0.2
)
}
\arguments{
\item{...}{ggplot objects to combine.}

\item{ncol}{Number of columns in the plot grid.}

\item{nrow}{Number of rows in the plot grid. If NULL (default), computed
from ncol and number of plots.}

\item{position}{Where to place the shared legend. One of \code{"right"}
(default), \code{"left"}, \code{"bottom"}, or \code{"top"}.}

\item{legend_from}{Which plot to extract the legend from. Default is 1
(first plot). Can be an integer index or a ggplot object.}

\item{rel_legend_size}{Relative size of the legend compared to the plot
area. Default is 0.2 (20 percent).}
}
\value{
A gtable that can be drawn with \code{grid::grid.draw()} or used
with \code{cowplot::ggdraw()}.
}
\description{
Combines multiple ggplot objects into a grid layout with a single shared
legend. Works with base ggplot and cowplot workflows (no patchwork required).
}
\details{
This function provides a simple way to create multi-panel plots with a
shared legend without requiring patchwork. It:
\enumerate{
\item Removes legends from all plots
\item Extracts the legend from the specified plot
\item Arranges plots in a grid
\item Attaches the legend to the specified position
}

For more complex layouts or patchwork users, see \code{\link{collect_legends}}.
}
\examples{
library(ggplot2)

p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 1", color = "Cylinders")
p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 2", color = "Cylinders")
p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
  geom_point() + labs(title = "Plot 3", color = "Cylinders")

# Side-by-side with shared legend on right
gt <- shared_legend(p1, p2, ncol = 2, position = "right")
grid::grid.newpage()
grid::grid.draw(gt)

# 2x2 grid with legend at bottom
gt <- shared_legend(p1, p2, p3, p1, ncol = 2, nrow = 2, position = "bottom")
grid::grid.newpage()
grid::grid.draw(gt)

# Stacked with legend on left
gt <- shared_legend(p1, p2, p3, ncol = 1, position = "left")
grid::grid.newpage()
grid::grid.draw(gt)

}
\seealso{
\code{\link{get_legend}}, \code{\link{collect_legends}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collect_legends.R
\name{collect_legends}
\alias{collect_legends}
\title{Collect Legends from Patchwork Compositions}
\usage{
collect_legends(
  x,
  position = c("right", "left", "bottom", "top"),
  span = FALSE
)
}
\arguments{
\item{x}{A patchwork object created by combining ggplot2 plots with
operators like \code{|}, \code{/}, or \code{+}.}

\item{position}{Where to place the collected legends. One of \code{"right"}
(default), \code{"left"}, \code{"bottom"}, or \code{"top"}.}

\item{span}{Logical or integer vector. If \code{TRUE}, the legend will span
the full height (for left/right) or width (for top/bottom). If an integer
vector (e.g., \code{c(1, 2)} or \code{1:2}), the legend spans only those
rows/columns. Default is \code{FALSE}. When not \code{FALSE}, returns a
gtable instead of a patchwork object.}
}
\value{
A patchwork object with legends collected, or a gtable if
\code{span} is not \code{FALSE}.
}
\description{
Collects legends from multiple plots in a patchwork composition into a single
legend area. This is a convenience wrapper around patchwork's guide
collection functionality.
}
\details{
This function requires the patchwork package. If patchwork is not installed,
an informative error is raised.

When \code{span = TRUE}, the function converts the patchwork to a gtable
and modifies the legend cell to span all rows (for right/left positions) or
all columns (for top/bottom positions). This addresses the common issue where
collected legends are centered rather than spanning the full composition.

When \code{span} is an integer vector (e.g., \code{c(1, 2)}), the legend
is centered between those specific rows (for right/left) or columns (for
top/bottom). This is useful for attaching a legend to specific plots in a
stacked layout. Row/column numbers refer to the panel positions in the
patchwork (1-indexed).

For cowplot users: cowplot does not have a built-in legend collection
mechanism. Consider using the lemon package for manual legend extraction
and positioning.
}
\examples{
library(ggplot2)

if (requireNamespace("patchwork", quietly = TRUE)) {
  library(patchwork)

  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point()
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
    geom_point()

  # Without collection: each plot has its own legend
  p1 | p2

  # With collection: single shared legend
  collect_legends(p1 | p2)

  # Place collected legend on the bottom
  collect_legends(p1 | p2, position = "bottom")

  # Legend spanning full height (returns gtable, draw with grid.draw)
  gt <- collect_legends(p1 / p2, position = "right", span = TRUE)
  grid::grid.draw(gt)

  # Attach legend to specific rows (e.g., align with row 1 only)
  p3 <- ggplot(mtcars, aes(mpg, disp, color = factor(cyl))) +
    geom_point() + labs(title = "Plot 3")
  gt <- collect_legends(p1 / p2 / p3, position = "right", span = 1)
  grid::grid.draw(gt)

  # Attach legend to rows 1 and 2
  gt <- collect_legends(p1 / p2 / p3, position = "right", span = 1:2)
  grid::grid.draw(gt)
}

}
\seealso{
\code{\link{collect_axes}}, \code{\link{legend_left}}, \code{\link{legend_wrap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_horizontal}
\alias{legend_horizontal}
\title{Set Legend Direction to Horizontal}
\usage{
legend_horizontal()
}
\value{
A ggplot2 theme object that can be added to a plot.
}
\description{
A one-liner to arrange legend keys horizontally. Useful for legends placed
at the top or bottom of a plot.
}
\examples{
library(ggplot2)

# Horizontal legend at bottom
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_bottom() +
  legend_horizontal()

}
\seealso{
\code{\link{legend_vertical}}, \code{\link{legend_top}},
\code{\link{legend_bottom}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collect_axes.R
\name{collect_axes}
\alias{collect_axes}
\alias{align_guides_h}
\title{Collect Axes from Patchwork Compositions}
\usage{
collect_axes(x, guides = c("collect", "keep"))

align_guides_h(x, guides = c("collect", "keep"))
}
\arguments{
\item{x}{A patchwork object created by combining ggplot2 plots.}

\item{guides}{How to handle guides. Either \code{"collect"} to combine into a
single legend (default), or \code{"keep"} to maintain separate legends.}
}
\value{
A patchwork object with collected axes.
}
\description{
Collects duplicate axes from multiple plots in a patchwork composition,
removing redundant axis labels and ticks. This is a convenience wrapper
around patchwork's axis collection functionality.
}
\details{
This function applies patchwork's \code{axes = "collect"} layout option,
which removes duplicate axes when plots are stacked or placed side-by-side.
For example, in a 2x1 vertical layout, the x-axis labels on the top plot
will be removed since they are redundant with the bottom plot's x-axis.

When \code{guides = "collect"}, legends are also merged into a single
shared legend.

For cowplot: Axis alignment in cowplot requires manual use of
\code{\link[cowplot]{align_plots}} with \code{align = "h"} or \code{"v"}.
This function does not directly support cowplot objects.
}
\examples{
library(ggplot2)

if (requireNamespace("patchwork", quietly = TRUE)) {
  library(patchwork)

  # Two plots stacked vertically - x-axis is duplicated
  p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
    geom_point() +
    labs(y = "Weight")
  p2 <- ggplot(mtcars, aes(mpg, hp, color = factor(cyl))) +
    geom_point() +
    labs(y = "Horsepower")

  # Without collect_axes: both plots show x-axis
  p1 / p2

  # With collect_axes: removes redundant x-axis from top plot
  collect_axes(p1 / p2)

  # Keep separate legends
  collect_axes(p1 / p2, guides = "keep")
}

}
\seealso{
\code{\link{collect_legends}}, \code{\link{legend_left}}, \code{\link{legend_wrap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_inside}
\alias{legend_inside}
\title{Place Legend Inside the Plot Area}
\usage{
legend_inside(
  x = NULL,
  y = NULL,
  position = NULL,
  justification = NULL,
  background = "white",
  border = NA,
  padding = 0.2,
  just = NULL
)
}
\arguments{
\item{x}{Numeric x-coordinate in normalized 0-1 space, where 0 is left
and 1 is right. Ignored if \code{position} is specified.}

\item{y}{Numeric y-coordinate in normalized 0-1 space, where 0 is bottom
and 1 is top. Ignored if \code{position} is specified.}

\item{position}{Character shortcut for common positions. One of
\code{"topleft"}, \code{"top"}, \code{"topright"}, \code{"left"},
\code{"center"}, \code{"right"}, \code{"bottomleft"}, \code{"bottom"},
\code{"bottomright"}. If specified, overrides \code{x} and \code{y}.}

\item{justification}{Justification of legend relative to the anchor point.
Either a character vector of length 2 (horizontal, vertical) or a single
value. Common values: \code{c("left", "top")}, \code{c("right", "bottom")},
\code{"center"}. If \code{NULL}, automatically determined based on position.}

\item{background}{Background fill color for the legend. Default is
\code{"white"}.}

\item{border}{Border color for the legend box. Default is \code{NA} (no border).}

\item{padding}{Padding around legend content in cm. Default is \code{0.2}.}

\item{just}{Deprecated. Use \code{justification} instead.}
}
\value{
A ggplot2 theme object that can be added to a plot.
}
\description{
Position the legend inside the plot panel at specified coordinates or using
convenient position shortcuts like \code{"topright"} or \code{"bottomleft"}.
}
\examples{
library(ggplot2)

# Using position shortcuts (recommended)
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_inside(position = "topright")

ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_inside(position = "bottomleft")

# Using coordinates
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_inside(x = 0.95, y = 0.95, justification = c("right", "top"))

# Custom background and border
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_inside(position = "topright", background = "grey95", border = "grey50")

}
\seealso{
\code{\link{legend_left}}, \code{\link{legend_right}},
\code{\link{legend_top}}, \code{\link{legend_bottom}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{legend_reverse}
\alias{legend_reverse}
\title{Reverse Legend Order}
\usage{
legend_reverse()
}
\value{
A guides specification that can be added to a plot.
}
\description{
Reverses the order of entries in all legends. Useful when the natural data
order doesn't match the desired visual order (e.g., when stacking bars).
}
\details{
This function applies \code{guide_legend(reverse = TRUE)} to all common
discrete aesthetics: colour, fill, shape, size, linetype, and alpha.
}
\examples{
library(ggplot2)

# Default order
p1 <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point()

# Reversed order
p2 <- p1 + legend_reverse()

}
\seealso{
\code{\link{legend_wrap}}, \code{\link{legend_style}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{ggplotGrob.gg_centered_title}
\alias{ggplotGrob.gg_centered_title}
\title{Convert centered title plot to gtable}
\usage{
\method{ggplotGrob}{gg_centered_title}(x)
}
\arguments{
\item{x}{A gg_centered_title object}
}
\value{
A gtable object (grob table) with centered legend titles, suitable for
rendering with \code{grid::grid.draw()} or saving with \code{ggplot2::ggsave()}.
}
\description{
Convert centered title plot to gtable
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_right}
\alias{legend_right}
\title{Place Legend on the Right with Proper Alignment}
\usage{
legend_right(by = NULL)
}
\arguments{
\item{by}{Optional aesthetic name (character) to position only a specific
legend. When specified, uses per-guide positioning via
\code{guide_legend(position = "right")}. Requires ggplot2 >= 3.5.0.
Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.}
}
\value{
A ggplot2 theme object (when \code{by} is NULL) or a guides
specification (when \code{by} is specified).
}
\description{
A one-liner to position the legend on the right side of the plot with correct
right alignment for both the key box and text labels.
}
\details{
When \code{by} is NULL (default), this function sets three theme elements:
\itemize{
\item \code{legend.position = "right"} to place the legend on the right
\item \code{legend.justification = "right"} to right-justify the legend box
\item \code{legend.box.just = "right"} to right-align multiple legend boxes
}

When \code{by} is specified, only the legend for that aesthetic is moved,
allowing different legends to be placed in different positions.
}
\examples{
library(ggplot2)

# Basic usage
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_right()

# Position only the size legend on the right
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point() +
  legend_bottom(by = "colour") +
  legend_right(by = "size")

}
\seealso{
\code{\link{legend_left}}, \code{\link{legend_top}},
\code{\link{legend_bottom}}, \code{\link{legend_inside}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{print.gg_centered_title}
\alias{print.gg_centered_title}
\title{Print method for centered title plots}
\usage{
\method{print}{gg_centered_title}(x, ...)
}
\arguments{
\item{x}{A gg_centered_title object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object. Called for the side effect of
rendering the plot with centered legend titles.
}
\description{
Print method for centered title plots
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_multi.R
\name{legend_order_guides}
\alias{legend_order_guides}
\title{Control Legend Display Order}
\usage{
legend_order_guides(...)
}
\arguments{
\item{...}{Named arguments where names are aesthetic names and values are
integer order positions. E.g., \code{colour = 1, size = 2}.}
}
\value{
A guides specification that can be added to a plot.
}
\description{
Set the display order of multiple legends. Legends with lower order values
appear first (top or left).
}
\details{
The order value determines the position of the legend relative to others.
Lower values appear first. By default, all legends have order = 0 and appear
in an unspecified order.
}
\examples{
library(ggplot2)

# Plot with multiple legends
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point()

# Default order
p

# Size legend first, then colour
p + legend_order_guides(size = 1, colour = 2)

# Colour legend first
p + legend_order_guides(colour = 1, size = 2)

}
\seealso{
\code{\link{legend_merge}}, \code{\link{legend_split}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{legend_auto_fit}
\alias{legend_auto_fit}
\title{Auto-fit Legend to Plot Height}
\usage{
legend_auto_fit(plot, max_ratio = 0.95)
}
\arguments{
\item{plot}{A ggplot object.}

\item{max_ratio}{Maximum ratio of legend height to panel height before
wrapping is triggered. Default is 0.95 (95 percent of panel height).}
}
\value{
A modified ggplot object with adjusted legend layout.
}
\description{
Measures the legend height relative to the plot panel and automatically
wraps the legend into multiple columns if it would overflow. This function
must be called on a complete ggplot object, not added with \code{+}.
}
\details{
This function builds the plot to measure actual dimensions, then rebuilds
with an appropriate number of legend rows if the legend is too tall.
It's particularly useful after applying \code{legend_style(angle = 90)}
which can cause legends to exceed the plot height.

Because this requires building the plot twice, it has a small performance
cost. For static plots this is negligible.
}
\examples{
library(ggplot2)

# Legend with rotated text that might overflow
p <- ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_style(angle = 90)

# Auto-fit will wrap if needed
legend_auto_fit(p)

}
\seealso{
\code{\link{legend_style}}, \code{\link{legend_wrap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{ggplot_add.legend_style_centered}
\alias{ggplot_add.legend_style_centered}
\title{Add legend_style_centered to ggplot}
\usage{
\method{ggplot_add}{legend_style_centered}(object, plot, ...)
}
\arguments{
\item{object}{A legend_style_centered object}

\item{plot}{A ggplot object}

\item{...}{Additional arguments (ignored)}
}
\value{
A modified ggplot object with additional class \code{gg_centered_title}
or \code{gg_autofit_legend} (for 90-degree rotation), used to trigger
custom rendering behavior.
}
\description{
Add legend_style_centered to ggplot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_multi.R
\name{legend_hide}
\alias{legend_hide}
\title{Hide Specific Legends}
\usage{
legend_hide(...)
}
\arguments{
\item{...}{Aesthetic names (unquoted) to hide. Common values: \code{colour},
\code{fill}, \code{size}, \code{shape}, \code{linetype}, \code{alpha}.
Note: \code{color} is automatically converted to \code{colour}.}
}
\value{
A guides specification that can be added to a plot.
}
\description{
Remove specific legends from a plot while keeping others. This is more
targeted than \code{legend_none()} which removes all legends.
}
\examples{
library(ggplot2)

# Plot with multiple legends
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point()

# Hide the size legend
p + legend_hide(size)

# Hide multiple legends
p + legend_hide(size, colour)

}
\seealso{
\code{\link{legend_select}}, \code{\link{legend_none}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_order.R
\name{legend_order}
\alias{legend_order}
\title{Reorder Legend Entries}
\usage{
legend_order(order, aesthetic = "colour")
}
\arguments{
\item{order}{A character vector specifying the desired order of legend entries,
or a function to apply to the current order (e.g., \code{rev}, \code{sort}).}

\item{aesthetic}{Character string specifying which aesthetic's legend to reorder.
Default is \code{"colour"}. Common values: \code{"colour"}, \code{"color"}, \code{"fill"},
\code{"shape"}, \code{"linetype"}, \code{"size"}, \code{"alpha"}.}
}
\value{
A ggplot2 scale object that reorders the legend.
}
\description{
Change the order of legend entries without modifying factor levels in your data.
This provides a simpler alternative to manually setting factor levels or using
the \code{breaks} argument in scale functions.
}
\details{
This function works by setting the \code{breaks} argument of the appropriate
discrete scale. It automatically detects whether to use \code{scale_colour_discrete},
\code{scale_fill_discrete}, etc. based on the \code{aesthetic} argument.

When \code{order} is a function (like \code{rev} or \code{sort}), it will be applied to
the default order of legend entries.
}
\examples{
library(ggplot2)

p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point(size = 3)

# Specify exact order
p + legend_order(c("8", "6", "4"))

# Reverse the order
p + legend_order(rev)

# Sort alphabetically/numerically
p + legend_order(sort)

# Reorder fill aesthetic
ggplot(mtcars, aes(factor(cyl), fill = factor(cyl))) +
  geom_bar() +
  legend_order(c("8", "4", "6"), aesthetic = "fill")

}
\seealso{
\code{\link{legend_reverse}} for a simpler way to reverse legend order,
\code{\link{legend_style}} for styling legend appearance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_position.R
\name{legend_left}
\alias{legend_left}
\title{Place Legend on the Left with Proper Alignment}
\usage{
legend_left(by = NULL)
}
\arguments{
\item{by}{Optional aesthetic name (character) to position only a specific
legend. When specified, uses per-guide positioning via
\code{guide_legend(position = "left")}. Requires ggplot2 >= 3.5.0.
Common values: \code{"colour"}, \code{"fill"}, \code{"size"}.}
}
\value{
A ggplot2 theme object (when \code{by} is NULL) or a guides
specification (when \code{by} is specified).
}
\description{
A one-liner to position the legend on the left side of the plot with correct
left alignment for both the key box and text labels. This goes beyond simple
\code{legend.position = "left"} by also setting justification and box alignment.
}
\details{
When \code{by} is NULL (default), this function sets three theme elements:
\itemize{
\item \code{legend.position = "left"} to place the legend on the left
\item \code{legend.justification = "left"} to left-justify the legend box
\item \code{legend.box.just = "left"} to left-align multiple legend boxes
}

When \code{by} is specified, only the legend for that aesthetic is moved,
allowing different legends to be placed in different positions.
}
\examples{
library(ggplot2)

# Basic usage
ggplot(mtcars, aes(mpg, wt, color = factor(cyl))) +
  geom_point() +
  legend_left()

# Works with multiple legends
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), shape = factor(am))) +
  geom_point(size = 3) +
  legend_left()

# Position only the colour legend on the left
ggplot(mtcars, aes(mpg, wt, color = factor(cyl), size = hp)) +
  geom_point() +
  legend_left(by = "colour") +
  legend_bottom(by = "size")

}
\seealso{
\code{\link{legend_right}}, \code{\link{legend_top}},
\code{\link{legend_bottom}}, \code{\link{legend_inside}},
\code{\link{legend_none}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_multi.R
\name{legend_split}
\alias{legend_split}
\title{Force Legends to Stay Separate}
\usage{
legend_split(...)
}
\arguments{
\item{...}{Aesthetic names (unquoted) to keep separate.
E.g., \code{colour, fill}.}
}
\value{
A guides specification that can be added to a plot.
}
\description{
Force specified legends to remain separate by assigning different order
values, preventing automatic merging.
}
\details{
By default, ggplot2 merges legends that have matching titles and labels.
This function assigns different order values to each legend, which prevents
automatic merging.
}
\examples{
library(ggplot2)

# Plot where colour and fill would normally merge
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(shape = 21, size = 3, stroke = 1.5) +
  labs(color = "Cylinders", fill = "Cylinders")

# Force separate legends
p + legend_split(colour, fill)

}
\seealso{
\code{\link{legend_merge}}, \code{\link{legend_order_guides}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{ggplotGrob.gg_autofit_legend}
\alias{ggplotGrob.gg_autofit_legend}
\title{Convert auto-fit legend plot to gtable}
\usage{
\method{ggplotGrob}{gg_autofit_legend}(x)
}
\arguments{
\item{x}{A gg_autofit_legend object}
}
\value{
A gtable object (grob table) with auto-fitted and centered legend,
suitable for rendering with \code{grid::grid.draw()} or saving with
\code{ggplot2::ggsave()}.
}
\description{
Convert auto-fit legend plot to gtable
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggguides-package.R
\docType{package}
\name{ggguides-package}
\alias{ggguides}
\alias{ggguides-package}
\title{ggguides: Simplified Legend and Guide Alignment for 'ggplot2'}
\description{
Provides one-liner functions for common legend and guide operations in 'ggplot2'. Simplifies legend positioning, styling, wrapping, and collection across multi-panel plots created with 'patchwork' or 'cowplot'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://gillescolling.com/ggguides/}
  \item \url{https://github.com/gcol33/ggguides}
  \item Report bugs at \url{https://github.com/gcol33/ggguides/issues}
}

}
\author{
\strong{Maintainer}: Gilles Colling \email{gilles.colling051@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_wrap.R
\name{legend_wrap}
\alias{legend_wrap}
\title{Wrap Legend Entries into Columns or Rows}
\usage{
legend_wrap(ncol = NULL, nrow = NULL, byrow = TRUE)
}
\arguments{
\item{ncol}{Number of columns for the legend layout. If \code{NULL}, determined
automatically based on \code{nrow}.}

\item{nrow}{Number of rows for the legend layout. If \code{NULL}, determined
automatically based on \code{ncol}.}

\item{byrow}{Logical. If \code{TRUE} (default), fills by row first. If \code{FALSE},
fills by column first.}
}
\value{
A list of guide specifications that can be added to a plot.
}
\description{
A one-liner to arrange legend entries in a grid layout. Works with discrete
legends by applying the specified layout to all color, fill, shape, size,
linetype, and alpha aesthetics.
}
\details{
This function creates a \code{guides()} specification that applies the same
column/row layout to all common discrete aesthetics. At least one of \code{ncol}
or \code{nrow} should be specified.
}
\examples{
library(ggplot2)

# Wrap a long legend into 2 columns
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2)

# Wrap into 3 rows, filling by column
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(nrow = 3, byrow = FALSE)

# Combine with legend_left for left-aligned wrapped legends
ggplot(mpg, aes(displ, hwy, color = class)) +
  geom_point() +
  legend_wrap(ncol = 2) +
  legend_left()

}
\seealso{
\code{\link{legend_left}}, \code{\link{collect_legends}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_multi.R
\name{legend_merge}
\alias{legend_merge}
\title{Force Legends to Merge}
\usage{
legend_merge(...)
}
\arguments{
\item{...}{Aesthetic names (unquoted) to merge. E.g., \code{colour, fill}.}
}
\value{
A guides specification that can be added to a plot.
}
\description{
Force specified legends to merge together by setting them to the same order.
Legends will only merge if they have matching labels (same factor levels or
break values).
}
\details{
ggplot2 automatically merges legends when they have the same title and
matching labels. This function ensures legends have the same order value
(order = 0), which is a prerequisite for merging.

If legends still don't merge after using this function, ensure:
\itemize{
\item Both aesthetics map to the same variable
\item The legends have identical titles (use \code{labs()})
\item The breaks/labels are identical
}
}
\examples{
library(ggplot2)

# Plot where colour and fill map to the same variable
p <- ggplot(mtcars, aes(mpg, wt, color = factor(cyl), fill = factor(cyl))) +
  geom_point(shape = 21, size = 3, stroke = 1.5) +
  labs(color = "Cylinders", fill = "Cylinders")

# Force merge (they should merge automatically if labels match)
p + legend_merge(colour, fill)

}
\seealso{
\code{\link{legend_split}}, \code{\link{legend_order_guides}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_style.R
\name{plot.gg_autofit_legend}
\alias{plot.gg_autofit_legend}
\title{Plot method for auto-fit legend plots}
\usage{
\method{plot}{gg_autofit_legend}(x, ...)
}
\arguments{
\item{x}{A gg_autofit_legend object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object. Called for the side effect of
rendering the plot with auto-fitted and centered legend.
}
\description{
Plot method for auto-fit legend plots
}
\keyword{internal}
